@@ -5,15 +5,16 @@
  * Base URL Parameters:
  *   - page: (string) The current page view.
  *
- * Copyright 2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2013-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
- * @author   Michael Slusarz <slusarz@horde.org>
- * @category Horde
- * @license  http://www.horde.org/licenses/gpl GPL
- * @package  IMP
+ * @author    Michael Slusarz <slusarz@horde.org>
+ * @category  Horde
+ * @copyright 2013-2014 Horde LLC
+ * @license   http://www.horde.org/licenses/gpl GPL
+ * @package   IMP
  */
 
 require_once __DIR__ . '/lib/Application.php';
@@ -33,8 +34,12 @@ if (!class_exists($class)) {
 try {
     $ob = new $class($vars);
 } catch (Exception $e) {
-    $notification->push($e);
-    $ob = new IMP_Basic_Error($vars);
+    if ($registry->getView() == $registry::VIEW_BASIC) {
+        $notification->push($e);
+        $ob = new IMP_Basic_Error($vars);
+    } else {
+        throw $e;
+    }
 }
 
 $status = $ob->status();
