@@ -5,15 +5,16 @@
  * Base URL Parameters:
  *   - page: (string) The current page view.
  *
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
- * @author   Michael Slusarz <slusarz@horde.org>
- * @category Horde
- * @license  http://www.horde.org/licenses/gpl GPL
- * @package  IMP
+ * @author    Michael Slusarz <slusarz@horde.org>
+ * @category  Horde
+ * @copyright 2012-2014 Horde LLC
+ * @license   http://www.horde.org/licenses/gpl GPL
+ * @package   IMP
  */
 
 require_once __DIR__ . '/lib/Application.php';
@@ -28,7 +29,7 @@ if (!class_exists($class)) {
     throw new IMP_Exception('Page not found: ' . $vars->page);
 }
 
-$ob = new $class($injector->getInstance('Horde_Variables'));
+$ob = new $class($vars);
 
 $page_output->header(array(
     'body_class' => 'horde-ajax',
