@@ -1,8 +1,9 @@
 /**
  * Provides the javascript for the ACL preferences management view.
  *
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.horde.org/licenses/gpl.
+ * @author     Michael Slusarz <slusarz@horde.org>
+ * @copyright  2014 Horde LLC
+ * @license    GPL-2 (http://www.horde.org/licenses/gpl)
  */
 
 document.observe('dom:loaded', function() {
@@ -11,6 +12,9 @@ document.observe('dom:loaded', function() {
         $('change_acl_mbox').click();
     });
 
+    /* Disable selection of container elements. */
+    $('aclmbox').select('OPTION[value=""]').invoke('writeAttribute', 'disabled', true);
+
     $$('TABLE.prefsAclTable')[0].on('change', 'SELECT.aclTemplate', function(e, elt) {
         var acl = $F(elt);
 
