@@ -1,10 +1,9 @@
 /**
- * dimpcore.js - Dimp UI application logic.
+ * Core dynamic view logic.
  *
- * Copyright 2005-2013 Horde LLC (http://www.horde.org/)
- *
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.horde.org/licenses/gpl.
+ * @author     Michael Slusarz <slusarz@horde.org>
+ * @copyright  2005-2014 Horde LLC
+ * @license    GPL-2 (http://www.horde.org/licenses/gpl)
  */
 
 /* DimpCore object. */
@@ -17,6 +16,24 @@ var DimpCore = {
     context: {},
     text: {},
 
+    // Preferences variables
+    prefs: {
+        preview: 'horiz',
+        qsearch_field: 'all',
+        splitbar_horiz: 0,
+        splitbar_vert: 0,
+        toggle_hdrs: 0
+    },
+    prefs_special: function(n) {
+        switch (n) {
+        case 'preview_old':
+            return this.getPref('preview');
+
+        case 'splitbar_side':
+            return this.conf.sidebar_width;
+        }
+    },
+
     // Wrapper methods around HordeCore functions.
 
     // IMP specific 'opts': uids
@@ -41,16 +58,17 @@ var DimpCore = {
         params = params || {};
         params.type = type;
 
-        HordeCore.popupWindow(DimpCore.conf.URI_COMPOSE, params, {
+        HordeCore.popupWindow(this.conf.URI_COMPOSE, params, {
             name: 'compose' + new Date().getTime()
         });
     },
 
     toggleButtons: function(elts, disable)
     {
-        elts.each(function(b) {
-            [ b.up() ].invoke(disable ? 'addClassName' : 'removeClassName', 'disabled');
-        });
+        elts.compact().invoke('up').invoke(
+            disable ? 'addClassName' : 'removeClassName',
+            'disabled'
+        );
     },
 
     // p = (Element) Parent element
@@ -68,6 +86,10 @@ var DimpCore = {
             ins = {};
         p = $(p);
 
+        if (!p) {
+            return;
+        }
+
         ins[o.insert ? o.insert : 'after'] = elt;
         p.insert(ins);
 
@@ -133,7 +155,7 @@ var DimpCore = {
     {
         alist.each(function(o) {
             var tmp,
-                a = new Element('A', { className: 'horde-button' }).store({ email: o });
+                a = new Element('A', { className: 'horde-button address' }).store({ email: o });
 
             if (o.g) {
                 a.insert(o.g.escapeHTML() + ':').addClassName('addrgroup-name');
@@ -159,21 +181,44 @@ var DimpCore = {
     updateMsgLog: function(log)
     {
         var df = document.createDocumentFragment(),
-            tmp = $('msgloglist').down('UL');
+            tmp = $('msgloglist');
 
         log.each(function(entry) {
-            df.appendChild(new Element('LI').insert(new Element('SPAN', { className: 'iconImg imp-' + entry.t })).insert(entry.m));
+            df.appendChild(new Element('LI').insert(new Element('SPAN', { className: 'iconImg imp-' + entry.t })).insert(entry.m.escapeHTML()));
         });
 
         tmp.childElements().invoke('remove');
         tmp.appendChild(df);
     },
 
-    // Abstract: define in any pages that need reloadMessage().
-    reloadMessage: function(params)
+    /* Browser-side preferences. */
+
+    getPref: function(k)
     {
+        return $.jStorage.get(
+            this.pref_prefix + k,
+            $.jStorage.get(
+                /* Fallback to non-prefixed storage. */
+                k,
+                this.prefs[k] ? this.prefs[k] : this.prefs_special(k)
+            )
+        );
     },
 
+    setPref: function(k, v)
+    {
+        if (v === null) {
+            $.jStorage.deleteKey(this.pref_prefix + k);
+            $.jStorage.deleteKey(k);
+        } else {
+            $.jStorage.set(this.pref_prefix + k, v);
+        }
+    },
+
+    // Abstract: define in any pages that need reloadMessage().
+    // One argument: params
+    reloadMessage: Prototype.emptyFunction,
+
     toggleCheck: function(elt, on)
     {
         if (on === null) {
@@ -196,6 +241,19 @@ var DimpCore = {
         elt.removeClassName(r).addClassName(a).show();
     },
 
+    baseAvailable: function()
+    {
+        var base = HordeCore.baseWindow();
+
+        if (base != window &&
+            !base.closed &&
+            !Object.isUndefined(base.DimpBase)) {
+            return base;
+        }
+
+        return null;
+    },
+
     /* Mouse click handler. */
     clickHandler: function(e)
     {
@@ -213,12 +271,12 @@ var DimpCore = {
                 [ tmp.down(), tmp.down(1), tmp.next() ].invoke('toggle');
             }
         } else if (elt.hasClassName('pgpVerifyMsg')) {
-            elt.replace(DimpCore.text.verify);
-            DimpCore.reloadMessage({ pgp_verify_msg: 1 });
+            elt.replace(this.text.verify);
+            this.reloadMessage({ pgp_verify_msg: 1 });
             e.memo.stop();
         } else if (elt.hasClassName('smimeVerifyMsg')) {
-            elt.replace(DimpCore.text.verify);
-            DimpCore.reloadMessage({ smime_verify_msg: 1 });
+            elt.replace(this.text.verify);
+            this.reloadMessage({ smime_verify_msg: 1 });
             e.memo.stop();
         }
     },
@@ -228,38 +286,67 @@ var DimpCore = {
         var baseelt = e.element();
 
         switch (e.memo.elt.readAttribute('id')) {
-        case 'ctx_contacts_new':
-            this.compose('new', {
-                to_json: Object.toJSON(baseelt.retrieve('email'))
-            });
-            break;
-
         case 'ctx_contacts_add':
             this.doAction('addContact', {
                 addr: Object.toJSON(baseelt.retrieve('email'))
             });
             break;
+
+        case 'ctx_contacts_addfilter':
+            this.doAction('newFilter', {
+                addr: Object.toJSON(baseelt.retrieve('email'))
+            });
+            break;
+
+        case 'ctx_contacts_copy':
+            window.prompt(this.text.emailcopy, baseelt.retrieve('email').b);
+            break;
+
+        case 'ctx_contacts_new':
+            this.compose('new', {
+                to_json: Object.toJSON(baseelt.retrieve('email'))
+            });
+            break;
         }
     },
 
     contextOnShow: function(e)
     {
-        var tmp;
+        var tmp, tmp2;
 
         switch (e.memo) {
         case 'ctx_contacts':
-            tmp = $(e.memo).down('DIV.contactAddr');
-            if (tmp) {
-                tmp.next().remove();
-                tmp.remove();
-            }
+            tmp = $(e.memo).down('DIV');
+            tmp2 = e.element().retrieve('email');
+            tmp.hide().childElements().invoke('remove');
 
             // Add e-mail info to context menu if personal name is shown on
             // page.
-            if ((tmp = e.element().retrieve('email')) && !tmp.g && tmp.p) {
-                $(e.memo)
-                    .insert({ top: new Element('DIV', { className: 'sep' }) })
-                    .insert({ top: new Element('DIV', { className: 'contactAddr' }).insert(tmp.b.escapeHTML()) });
+            if (tmp2) {
+                this.doAction('getContactsImage', {
+                    addr: tmp2.b
+                }, {
+                    callback: function (r) {
+                        if (r.flag) {
+                            tmp.show().insert({
+                                top: new Element('DIV')
+                                    .addClassName('flagimg')
+                                    .insert(new Element('IMG', { title: r.flagname, src: r.flag }))
+                                    .insert(r.flagname.escapeHTML())
+                            });
+                        }
+                        if (r.avatar) {
+                            tmp.show().insert({
+                                top: new Element('IMG', { src: r.avatar }).addClassName('contactimg')
+                            });
+                        }
+                    }
+                });
+
+                if (!tmp2.g && tmp2.p) {
+                    tmp.show().insert({ top: new Element('DIV', { className: 'sep' }) })
+                        .insert({ top: new Element('DIV', { className: 'contactAddr' }).insert(tmp2.b.escapeHTML()) });
+                }
             }
             break;
         }
@@ -267,14 +354,14 @@ var DimpCore = {
 
     contextOnTrigger: function(e)
     {
-        if (!DimpCore.context[e.memo]) {
+        if (!this.context[e.memo]) {
             return;
         }
 
         var div = new Element('DIV', { className: 'context', id: e.memo }).hide();
 
-        if (!Object.isArray(DimpCore.context[e.memo])) {
-            $H(DimpCore.context[e.memo]).each(function(pair) {
+        if (!Object.isArray(this.context[e.memo])) {
+            $H(this.context[e.memo]).each(function(pair) {
                 div.insert(this._contextOnTrigger(pair, e.memo));
             }, this);
         }
@@ -338,26 +425,19 @@ document.observe('ContextSensitive:click', DimpCore.contextOnClick.bindAsEventLi
 document.observe('ContextSensitive:show', DimpCore.contextOnShow.bindAsEventListener(DimpCore));
 document.observe('ContextSensitive:trigger', DimpCore.contextOnTrigger.bindAsEventListener(DimpCore));
 
+/* HTML IFRAME events. */
+document.observe('IMP_JS:htmliframe_click', function() {
+    if (this.DMenu) {
+        this.DMenu.close();
+    }
+}.bind(DimpCore));
+
 /* Dialog events. Since reloadMessage() can be extended, don't immediately
  * bind function call now. */
 document.observe('ImpPassphraseDialog:success', function() {
     DimpCore.reloadMessage({});
 });
 
-/* Notification events. */
-document.observe('HordeCore:showNotifications', function(e) {
-    switch (e.memo.type) {
-    case 'imp.reply':
-    case 'imp.forward':
-    case 'imp.redirect':
-        HordeBase.Growler.growl(m.message.escapeHTML(), {
-            className: m.type.replace('.', '-'),
-            life: 8
-        });
-        break;
-    }
-});
-
 /* Disable text selection for everything but compose/message body and FORM
  * inputs. */
 document.observe(Prototype.Browser.IE ? 'selectstart' : 'mousedown', function(e) {
@@ -369,8 +449,12 @@ document.observe(Prototype.Browser.IE ? 'selectstart' : 'mousedown', function(e)
 
         if (document.activeElement) {
             var ae = $(document.activeElement);
-            if (ae.match('TEXTAREA') || ae.match('INPUT')) {
-                ae.blur();
+            try {
+                if (ae.match('TEXTAREA') || ae.match('INPUT')) {
+                    ae.blur();
+                }
+            } catch (ex) {
+                // Chrome 32: reports that ae can be an Object - ignore.
             }
         }
     }
