@@ -1,8 +1,9 @@
 /**
- * Copyright 2005-2013 Horde LLC (http://www.horde.org/)
+ * Dynamic message view.
  *
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.horde.org/licenses/gpl.
+ * @author     Michael Slusarz <slusarz@horde.org>
+ * @copyright  2005-2014 Horde LLC
+ * @license    GPL-2 (http://www.horde.org/licenses/gpl)
  */
 
 var DimpMessage = {
@@ -54,7 +55,10 @@ var DimpMessage = {
             type: type,
             view: this.mbox
         }, {
-            callback: DimpCompose.fillForm.bind(DimpCompose),
+            callback: function(r) {
+                DimpCompose.fillForm(r);
+                $(document).fire('AutoComplete:reset');
+            },
             uids: [ this.buid ]
         });
     },
@@ -88,6 +92,8 @@ var DimpMessage = {
     /* Click handlers. */
     clickHandler: function(e)
     {
+        var base;
+
         switch (e.element().readAttribute('id')) {
         case 'windowclose':
             window.close();
@@ -107,15 +113,15 @@ var DimpMessage = {
         case 'button_delete':
         case 'button_innocent':
         case 'button_spam':
-            if (HordeCore.base.DimpBase) {
-                HordeCore.base.focus();
+            if ((base = DimpCore.baseAvailable())) {
+                base.focus();
                 if (e.element().identify() == 'button_delete') {
-                    HordeCore.base.DimpBase.deleteMsg({
+                    base.DimpBase.deleteMsg({
                         mailbox: this.mbox,
                         uid: this.buid
                     });
                 } else {
-                    HordeCore.base.DimpBase.reportSpam(e.element().identify() == 'button_spam', {
+                    base.DimpBase.reportSpam(e.element().identify() == 'button_spam', {
                         mailbox: this.mbox,
                         uid: this.buid
                     });
@@ -130,7 +136,7 @@ var DimpMessage = {
                     });
                 } else {
                     DimpCore.doAction('reportSpam', {
-                        spam: Number(e.element().identify() == 'button_spam'),
+                        spam: ~~(e.element().identify() == 'button_spam'),
                         view: this.mbox
                     }, {
                         uids: [ this.buid ],
@@ -247,6 +253,8 @@ var DimpMessage = {
 
     onDomLoad: function()
     {
+        var base;
+
         HordeCore.initHandler('click');
 
         if (DimpCore.conf.disable_compose) {
@@ -269,9 +277,9 @@ var DimpMessage = {
         }, this);
         delete this.addr_limit;
 
-        if (HordeCore.base.DimpBase) {
+        if ((base = DimpCore.baseAvailable())) {
             if (this.strip) {
-                HordeCore.base.DimpBase.poll();
+                base.DimpBase.poll();
             } else if (this.tasks) {
                 if (this.tasks['imp:maillog']) {
                     this.tasks['imp:maillog'].each(function(l) {
@@ -283,7 +291,7 @@ var DimpMessage = {
                     }, this);
                     delete this.tasks['imp:maillog'];
                 }
-                HordeCore.base.DimpBase.tasksHandler({ tasks: this.tasks });
+                base.DimpBase.tasksHandler({ tasks: this.tasks });
             }
         }
 
@@ -300,7 +308,14 @@ var DimpMessage = {
 document.observe('dom:loaded', function() {
     if (Prototype.Browser.IE && !document.addEventListener) {
         // IE 8
-        DimpMessage.onDomLoad.bind(DimpMessage).defer();
+        IMP_JS.iframeResize = IMP_JS.iframeResize.wrap(function(parentfunc, e, id) {
+            if ($('msgData').visible()) {
+                (function() { parentfunc(e, id); }).defer();
+            } else {
+                IMP_JS.iframeResize.bind(IMP_JS, e, id).defer();
+            }
+        });
+        DimpMessage.onDomLoad.bind(DimpMessage).delay(0.1);
     } else {
         DimpMessage.onDomLoad();
     }
