@@ -1,8 +1,9 @@
 /**
- * Provides the javascript for the advanced search page.
+ * Advanced search page.
  *
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.horde.org/licenses/gpl.
+ * @author     Michael Slusarz <slusarz@horde.org>
+ * @copyright  2014 Horde LLC
+ * @license    GPL-2 (http://www.horde.org/licenses/gpl)
  */
 
 var ImpSearch = {
@@ -18,7 +19,7 @@ var ImpSearch = {
         var fragment = document.createDocumentFragment(),
             node = new Element('OPTION');
 
-        $('recent_searches_div').show().next().show();
+        $('recent_searches_div').show();
 
         this.saved_searches = $H(searches);
         this.saved_searches.each(function(s) {
@@ -454,13 +455,13 @@ var ImpSearch = {
             switch (this.data.types[this.criteria[c].t]) {
             case 'header':
             case 'text':
-                this.criteria[c].n = Number(Boolean($F($(c).down('INPUT[type=checkbox]'))));
+                this.criteria[c].n = ~~(!!$F($(c).down('INPUT[type=checkbox]')));
                 this.criteria[c].v = $F($(c).down('INPUT[type=text]'));
                 data.push(this.criteria[c]);
                 break;
 
             case 'customhdr':
-                this.criteria[c].n = Number(Boolean($F($(c).down('INPUT[type=checkbox]'))));
+                this.criteria[c].n = ~~(!!$F($(c).down('INPUT[type=checkbox]')));
                 this.criteria[c].v = { h: $F($(c).down('INPUT')), s: $F($(c).down('INPUT', 1)) };
                 data.push(this.criteria[c]);
                 break;
@@ -479,7 +480,7 @@ var ImpSearch = {
                     alert(this.text.need_date);
                     return;
                 }
-                this.criteria[c].n = Number(Boolean($F($(c).down('INPUT[type=checkbox]'))));
+                this.criteria[c].n = ~~(!!$F($(c).down('INPUT[type=checkbox]')));
                 data.push(this.criteria[c]);
                 break;
 
@@ -489,12 +490,12 @@ var ImpSearch = {
                 break;
 
             case 'filter':
-                this.criteria[c].n = Number(Boolean($F($(c).down('INPUT[type=checkbox]'))));
+                this.criteria[c].n = ~~(!!$F($(c).down('INPUT[type=checkbox]')));
                 data.push(this.criteria[c]);
                 break;
 
             case 'flag':
-                this.criteria[c].n = Number(Boolean($F($(c).down('INPUT[type=checkbox]'))));
+                this.criteria[c].n = ~~(!!$F($(c).down('INPUT[type=checkbox]')));
                 data.push({
                     n: this.criteria[c].n,
                     t: 'flag',
@@ -551,11 +552,10 @@ var ImpSearch = {
             break;
 
         case 'show_unsub':
-            new Ajax.Request(this.ajaxurl + 'searchMailboxList', {
-                onSuccess: this.showUnsubCallback.bind(this),
-                parameters: {
-                    unsub: 1
-                }
+            HordeCore.doAction('searchMailboxList', {
+                unsub: 1
+            }, {
+                callback: this.showUnsubCallback.bind(this)
             });
             elt.remove();
             e.memo.stop();
@@ -656,26 +656,22 @@ var ImpSearch = {
 
     showUnsubCallback: function(r)
     {
-        var resp, sfa, vals;
-
-        if (r.responseJSON.response) {
-            resp = r.responseJSON.response;
-            this.data.mbox_list = resp.mbox_list;
-            sfa = $('search_mboxes_add');
+        var sfa = $('search_mboxes_add'),
             vals = sfa.select('[disabled]').pluck('value');
-            sfa.update(resp.tree);
-            vals.each(function(v) {
-                if (v.length) {
-                    this.disableMailbox(true, v);
-                }
-            }, this);
-        }
+
+        this.data.mbox_list = r.mbox_list;
+        sfa.update(r.tree);
+        vals.each(function(v) {
+            if (v.length) {
+                this.disableMailbox(true, v);
+            }
+        }, this);
     },
 
     onDomLoad: function()
     {
         if (!this.data) {
-            this.onDomLoad.bind(this).defer();
+            this.onDomLoad.bind(this).delay(0.1);
             return;
         }
 
