@@ -107,6 +107,10 @@
  *   Fired when rows are removed from the buffer.
  *   params: (ViewPort_Selection) The removed rows.
  *
+ * ViewPort:resize
+ *   Fired when viewport is being resized.
+ *   params: NONE
+ *
  * ViewPort:select
  *   Fired when rows are selected.
  *   params: (object) opts = (object) Boolean options [right]
@@ -219,10 +223,9 @@
  *   - slider2.js (Horde)
  *   - viewport_utils.js
  *
- * Copyright 2005-2013 Horde LLC (http://www.horde.org/)
- *
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.horde.org/licenses/gpl.
+ * @author     Michael Slusarz <slusarz@horde.org>
+ * @copyright  2005-2014 Horde LLC
+ * @license    GPL-2 (http://www.horde.org/licenses/gpl)
  */
 
 var ViewPort = Class.create({
@@ -240,7 +243,7 @@ var ViewPort = Class.create({
         this.opts.container = $(opts.container);
         this.opts.pane_data = $(opts.pane_data);
 
-        this.opts.content = new Element('DIV', { className: opts.list_class }).setStyle({ cssFloat: 'left' });
+        this.opts.content = new Element('DIV', { className: opts.list_class });
         this.opts.list_container = new Element('DIV');
         if (this.opts.list_header) {
             this.opts.list_container.insert(this.opts.list_header);
@@ -273,7 +276,7 @@ var ViewPort = Class.create({
         // Init empty string now.
         this.empty_msg = new Element('SPAN', { className: 'vpEmpty' });
 
-        Event.observe(window, 'resize', this.onResize.bind(this));
+        Event.observe(window, 'resize', function() { this.onResize(); }.bind(this));
         document.observe('DragDrop2:start', this._onDragStart.bindAsEventListener(this));
         document.observe('DragDrop2:end', this._onDragEnd.bindAsEventListener(this));
         document.observe('dblclick', this._onDragDblClick.bindAsEventListener(this));
@@ -295,7 +298,7 @@ var ViewPort = Class.create({
         if (this.page_size === null) {
             ps = this.getPageSize(this.pane_mode ? 'default' : 'max');
             if (isNaN(ps)) {
-                return this.loadView.bind(this, view, opts).defer();
+                return this.loadView.bind(this, view, opts).delay(0.1);
             }
             this.page_size = ps;
         }
@@ -421,7 +424,7 @@ var ViewPort = Class.create({
     {
         if (vs.size()) {
             if (this.isbusy) {
-                this.remove.bind(this, vs).defer();
+                this.remove.bind(this, vs).delay(0.1);
             } else {
                 this.isbusy = true;
                 try {
@@ -466,15 +469,17 @@ var ViewPort = Class.create({
         if (nowait) {
             this._onResize(size);
         } else {
-            this.resizefunc = this._onResize.bind(this, size).defer();
+            this.resizefunc = this._onResize.bind(this, size).delay(0.1);
         }
     },
 
     // size = (integer) The page size to use instead of auto-determining.
     _onResize: function(size)
     {
-        var c_opts = {},
-            h = this.opts.list_header ?  this.opts.list_header.getHeight() : 0,
+        this.opts.container.fire('ViewPort:resize');
+
+        var c_opts = {}, w,
+            h = this.opts.list_header ? this.opts.list_header.getHeight() : 0,
             lh = this._getLineHeight(),
             sp = this.split_pane;
 
@@ -500,14 +505,15 @@ var ViewPort = Class.create({
 
             h += lh * this.page_size;
             this.opts.list_container.setStyle({
-                height: h + 'px',
                 cssFloat: 'none',
-                overflow: 'hidden',
+                height: h + 'px',
                 width: 'auto'
             });
-            this.opts.content.setStyle({ width: '100%' });
+            this.opts.content.setStyle({ width: 'auto' });
             sp.currbar.show();
-            this.opts.pane_data.show().setStyle({ height: Math.max(document.viewport.getHeight() - this.opts.pane_data.viewportOffset()[1], 0) + 'px' });
+            this.opts.pane_data.show().setStyle({
+                height: Math.max(document.viewport.getHeight() - this.opts.pane_data.viewportOffset()[1], 0) + 'px'
+            });
             break;
 
         case 'vert':
@@ -517,18 +523,26 @@ var ViewPort = Class.create({
                 this.page_size = this.getPageSize('max');
             }
 
-            if (!sp.vert.width) {
-                sp.vert.width = parseInt(this.opts.container.clientWidth * 0.35, 10);
-            }
+            w = this.opts.container.getWidth();
+
+            /* Adapt splitbar width to current screen size. */
+            sp.vert.width = sp.vert.width
+                ? Math.max(15, Math.min(w - 15, sp.vert.width))
+                : parseInt(w * 0.45, 10);
 
             h += lh * this.page_size - this.opts.container.getLayout().get('border-bottom');
             this.opts.list_container.setStyle({
                 cssFloat: 'left',
-                height: h + 'px'
+                height: h + 'px',
+                width: sp.vert.width + 'px'
             });
             this.opts.content.setStyle({ width: sp.vert.width + 'px' });
-            sp.currbar.setStyle({ height: h - sp.currbar.getLayout().get('border-bottom') + 'px' }).show();
-            this.opts.pane_data.setStyle({ height: h - this.opts.pane_data.getLayout().get('border-bottom') + 'px' }).show();
+            sp.currbar.setStyle({
+                height: h - sp.currbar.getLayout().get('border-bottom') + 'px'
+            }).show();
+            this.opts.pane_data.setStyle({
+                height: h - this.opts.pane_data.getLayout().get('border-bottom') + 'px'
+            }).show();
             break;
 
         default:
@@ -548,10 +562,9 @@ var ViewPort = Class.create({
             this.opts.list_container.setStyle({
                 cssFloat: 'none',
                 height: (h + (lh * this.page_size)) + 'px',
-                overflow: 'hidden',
                 width: 'auto'
             });
-            this.opts.content.setStyle({ width: '100%' });
+            this.opts.content.setStyle({ width: 'auto' });
             break;
         }
 
@@ -599,7 +612,7 @@ var ViewPort = Class.create({
     _fetchBuffer: function(opts)
     {
         if (this.isbusy) {
-            this._fetchBuffer.bind(this, opts).defer();
+            this._fetchBuffer.bind(this, opts).delay(0.1);
         } else {
             this.isbusy = true;
             try {
@@ -805,30 +818,18 @@ var ViewPort = Class.create({
             this.opts.container.fire('ViewPort:endRangeFetch', r.view);
         }
 
-        if (!Object.isUndefined(r.cacheid)) {
-            this._ajaxResponse(r);
-        }
+        this._ajaxResponse(r);
     },
 
-    // r = (Object) viewport response object
     _ajaxResponse: function(r)
     {
         if (this.isbusy) {
-            this._ajaxResponse.bind(this, r).defer();
-        } else {
-            this.isbusy = true;
-            try {
-                this._ajaxResponseDo(r);
-            } catch (e) {
-                this.isbusy = false;
-                throw e;
-            }
-            this.isbusy = false;
+            this._ajaxResponse.bind(this, r).delay(0.1);
+            return;
         }
-    },
 
-    _ajaxResponseDo: function(r)
-    {
+        this.isbusy = true;
+
         this._clearWait();
 
         var callback, offset, tmp,
@@ -886,6 +887,8 @@ var ViewPort = Class.create({
             // viewport incorrectly.
             buffer.setMetaData({ offset: Number(r.rownum) - 1 }, true);
         }
+
+        this.isbusy = false;
     },
 
     // offset = (integer) Offset of row to display
@@ -1027,7 +1030,12 @@ var ViewPort = Class.create({
 
     bufferLoaded: function(view)
     {
-        return Boolean(this.views[view]);
+        return !!this.views[view];
+    },
+
+    bufferCount: function()
+    {
+        return Object.keys(this.views).size();
     },
 
     currentOffset: function()
@@ -1067,6 +1075,8 @@ var ViewPort = Class.create({
     // return: (integer) Number of rows in current view.
     getPageSize: function(type)
     {
+        var h, lh;
+
         switch (type) {
         case 'current':
             return Math.min(this.page_size, this.getMetaData('total_rows'));
@@ -1077,7 +1087,12 @@ var ViewPort = Class.create({
                 : Math.max(parseInt(this.getPageSize('max') * 0.45, 10), 5);
 
         case 'max':
-            return parseInt((document.viewport.getHeight() - this.opts.content.viewportOffset()[1]) / this._getLineHeight(), 10);
+            h = document.viewport.getHeight() - this.opts.content.viewportOffset()[1];
+            lh = this._getLineHeight();
+            if (this.split_pane.currbar && this.pane_mode == 'horiz') {
+                h -= this.split_pane.currbar.getHeight() + lh;
+            }
+            return parseInt(h / lh, 10);
 
         default:
             return this.page_size;
@@ -1141,7 +1156,7 @@ var ViewPort = Class.create({
                 constraint: 'vertical',
                 ghosting: true,
                 nodrop: true,
-                snap: function(x, y, elt) {
+                snap: function(x, y) {
                     var sp = this.split_pane,
                         l = parseInt((y - sp.pos) / sp.lh, 10);
                     if (l < 1) {
@@ -1208,7 +1223,7 @@ var ViewPort = Class.create({
         case 'vert':
             drag = DragDrop.Drags.getDrag(e.element());
             sp.vert.width = drag.lastCoord[0] - this.opts.list_container.viewportOffset()[0];
-            this.opts.content.setStyle({ width: sp.vert.width + 'px' });
+            this.onResize(true);
             change = drag.wasDragged;
             break;
         }
@@ -1227,21 +1242,22 @@ var ViewPort = Class.create({
             return;
         }
 
-        var change, old_size = this.page_size;
+        var old_size;
 
         switch (this.pane_mode) {
         case 'horiz':
+            old_size = this.page_size;
             this.onResize(true, this.getPageSize('default'));
-            change = (old_size != this.page_size);
+            if (old_size != this.page_size) {
+                this.opts.container.fire('ViewPort:splitBarChange', 'horiz');
+            }
             break;
 
         case 'vert':
-            this.opts.content.setStyle({ width: parseInt(this.opts.container.clientWidth * 0.45, 10) + 'px' });
-            change = true;
-        }
-
-        if (change) {
-            this.opts.container.fire('ViewPort:splitBarChange', this.pane_mode);
+            delete this.split_pane.vert.width;
+            this.onResize(true);
+            this.opts.container.fire('ViewPort:splitBarChange', 'vert');
+            break;
         }
     },
 
@@ -1354,11 +1370,12 @@ ViewPort_Scroller = Class.create({
             return;
         }
 
-        var c = this.vp.opts.content;
+        var c = this.vp.opts.content,
+            mw = this.mousewheelHandler.bindAsEventListener(this);
 
         // Create the outer div.
-        this.scrollDiv = new Element('DIV', { className: 'vpScroll' }).setStyle({ cssFloat: 'left', overflow: 'hidden' }).hide();
-        c.insert({ after: this.scrollDiv });
+        this.scrollDiv = new Element('DIV', { className: 'vpScroll' }).setStyle({ cssFloat: 'right', overflow: 'hidden' }).hide();
+        c.insert({ before: this.scrollDiv });
 
         this.scrollDiv.observe('Slider2:change', function() {
             if (!this.noupdate) {
@@ -1384,17 +1401,30 @@ ViewPort_Scroller = Class.create({
        });
 
         // Mouse wheel handler.
-        c.observe(Prototype.Browser.Gecko ? 'DOMMouseScroll' : 'mousewheel', function(e) {
-            var move_num = Math.min(this.vp.getPageSize(), 3);
-            this.moveScroll(this.currentOffset() + ((e.wheelDelta >= 0 || e.detail < 0) ? (-1 * move_num) : move_num));
-            /* Mozilla bug https://bugzilla.mozilla.org/show_bug.cgi?id=502818
-             * Need to stop or else multiple scroll events may be fired. We
-             * lose the ability to have the mousescroll bubble up, but that is
-             * more desirable than having the wrong scrolling behavior. */
-            if (Prototype.Browser.Gecko && !e.stop) {
-                Event.stop(e);
-            }
-        }.bindAsEventListener(this));
+        if ('onwheel' in document || (document.documentMode >= 9)) {
+            c.observe('wheel', mw);
+        } else {
+            c.observe('mousewheel', mw).observe('DomMouseScroll', mw);
+        }
+    },
+
+    mousewheelHandler: function(e)
+    {
+        var delta = e.wheelDelta || 0;
+
+        if (e.detail) {
+            delta = e.detail * -1;
+        }
+        if (e.deltaY) {
+            delta = e.deltaY * -1;
+        }
+        if (!Object.isUndefined(e.wheelDeltaY)) {
+            delta = e.wheelDeltaY;
+        }
+
+        if (delta) {
+            this.moveScroll(this.currentOffset() + (Math.min(this.vp.getPageSize(), 3) * (delta > 0 ? -1 : 1)));
+        }
     },
 
     setSize: function(viewsize, totalsize)
@@ -1411,17 +1441,11 @@ ViewPort_Scroller = Class.create({
         if (this.scrollbar.needScroll()) {
             switch (this.vp.pane_mode) {
             case 'vert':
-                this.scrollDiv.setStyle({ marginLeft: 0 });
                 if (!this.vertscroll) {
                     c.setStyle({ width: (c.clientWidth - this.scrollDiv.getWidth()) + 'px' });
                 }
                 vs = true;
                 break;
-
-            case 'horiz':
-            default:
-                this.scrollDiv.setStyle({ marginLeft: '-' + this.scrollDiv.getWidth() + 'px' });
-                break;
             }
 
             this.scrollDiv.setStyle({ height: c.clientHeight + 'px' });
@@ -1607,7 +1631,7 @@ ViewPort_Buffer = Class.create({
 
     UIDsToRows: function(uids)
     {
-        return rows.collect(this.rowlist.index.bind(this.rowlist)).compact();
+        return uids.collect(this.rowlist.index.bind(this.rowlist)).compact();
     },
 
     // vs = (ViewPort_Selection) TODO
@@ -1735,7 +1759,9 @@ ViewPort_Selection = Class.create({
     add: function(format, d)
     {
         var c = this._convert(format, d);
-        this.data = this.data.size() ? this.data.concat(c).uniq() : c;
+        this.data = this.data.size()
+            ? this.data.concat(c.reject(this.data.include.bind(this.data)))
+            : c;
     },
 
     remove: function(format, d)
