@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -26,7 +26,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -52,11 +52,16 @@ class IMP_Ajax_Application extends Horde_Core_Ajax_Application
     {
         global $injector, $registry;
 
+        $this->queue = $injector->getInstance('IMP_Ajax_Queue');
+
         switch ($registry->getView()) {
         case $registry::VIEW_BASIC:
             $this->addHandler('IMP_Ajax_Application_Handler_Mboxtoggle');
             $this->addHandler('IMP_Ajax_Application_Handler_Passphrase');
             $this->addHandler('IMP_Ajax_Application_Handler_Search');
+            if ($injector->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_REMOTE)) {
+                $this->addHandler('IMP_Ajax_Application_Handler_RemotePrefs');
+            }
             break;
 
         case $registry::VIEW_DYNAMIC:
@@ -67,6 +72,10 @@ class IMP_Ajax_Application extends Horde_Core_Ajax_Application
             $this->addHandler('IMP_Ajax_Application_Handler_Mboxtoggle');
             $this->addHandler('IMP_Ajax_Application_Handler_Passphrase');
             $this->addHandler('IMP_Ajax_Application_Handler_Search');
+            if ($injector->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_REMOTE)) {
+                $this->addHandler('IMP_Ajax_Application_Handler_Remote');
+                $this->addHandler('IMP_Ajax_Application_Handler_RemotePrefs');
+            }
             break;
 
         case $registry::VIEW_SMARTMOBILE:
@@ -82,8 +91,7 @@ class IMP_Ajax_Application extends Horde_Core_Ajax_Application
 
         $this->addHandler('IMP_Ajax_Application_Handler_ImageUnblock');
         $this->addHandler('Horde_Core_Ajax_Application_Handler_Imple');
-
-        $this->queue = $injector->getInstance('IMP_Ajax_Queue');
+        $this->addHandler('Horde_Core_Ajax_Application_Handler_Prefs');
 
         /* Copy 'view' paramter to 'mailbox', because this is what
          * IMP_Indices_Mailbox expects. */
@@ -95,7 +103,7 @@ class IMP_Ajax_Application extends Horde_Core_Ajax_Application
 
         /* Make sure the viewport entry is initialized. */
         $vp = isset($this->_vars->viewport)
-            ? Horde_Serialize::unserialize($this->_vars->viewport, Horde_Serialize::JSON)
+            ? json_decode($this->_vars->viewport)
             : new stdClass;
         $this->_vars->viewport = new Horde_Support_ObjectStub($vp);
 
@@ -108,11 +116,12 @@ class IMP_Ajax_Application extends Horde_Core_Ajax_Application
 
         /* Check for global poll task. */
         if (isset($this->_vars->poll)) {
-            $poll = Horde_Serialize::unserialize($this->_vars->poll, Horde_Serialize::JSON);
+            $poll = json_decode($this->_vars->poll);
             $this->queue->poll(
                 empty($poll)
-                    ? $injector->getInstance('IMP_Imap_Tree')->getPollList()
-                    : IMP_Mailbox::formFrom($poll)
+                    ? $injector->getInstance('IMP_Ftree')->poll->getPollList()
+                    : IMP_Mailbox::formFrom($poll),
+                true
             );
         }
     }
@@ -237,7 +246,7 @@ class IMP_Ajax_Application extends Horde_Core_Ajax_Application
         }
 
         if ($vp->search) {
-            $search = Horde_Serialize::unserialize($vp->search, Horde_Serialize::JSON);
+            $search = json_decode($vp->search);
             $args += array(
                 'search_buid' => isset($search->buid) ? $search->buid : null,
                 'search_unseen' => isset($search->unseen) ? $search->unseen : null
@@ -285,7 +294,7 @@ class IMP_Ajax_Application extends Horde_Core_Ajax_Application
          * on the IMAP server (saves some STATUS calls). */
         if (!is_null($rw)) {
             try {
-                $GLOBALS['injector']->getInstance('IMP_Imap')->openMailbox($this->indices->mailbox, $rw ? Horde_Imap_Client::OPEN_READWRITE : Horde_Imap_Client::OPEN_AUTO);
+                $this->indices->mailbox->imp_imap->openMailbox($this->indices->mailbox, $rw ? Horde_Imap_Client::OPEN_READWRITE : Horde_Imap_Client::OPEN_AUTO);
             } catch (IMP_Imap_Exception $e) {
                 $e->notify();
                 return null;
@@ -303,7 +312,7 @@ class IMP_Ajax_Application extends Horde_Core_Ajax_Application
      *   - from: (string) From address to use.
      *   - identity: (integer) The identity to use
      *
-     @param string $action  AJAX action.
+     * @param string $action  AJAX action.
      *
      * @return array  An array with the following values:
      *   - (object) AJAX base return object (with action and success
