@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,7 +17,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -37,7 +37,7 @@ class IMP_Ajax_Application_Handler_Common extends Horde_Core_Ajax_Application_Ha
          * separate poll action because there are other tasks done when
          * specifically requesting a poll. */
 
-        $this->_base->queue->quota($this->_base->indices->mailbox);
+        $this->_base->queue->quota($this->_base->indices->mailbox, false);
 
         if ($this->_base->changed()) {
             $this->_base->addTask('viewport', $this->_base->viewPortData(true));
@@ -62,7 +62,11 @@ class IMP_Ajax_Application_Handler_Common extends Horde_Core_Ajax_Application_Ha
      */
     public function viewPort()
     {
+        global $notification, $session;
+
         if (!$this->_base->indices->mailbox) {
+            /* Sanity checking only - this would only happen by direct
+             * access, so don't worry about clean error handling. */
             return false;
         }
 
@@ -90,26 +94,37 @@ class IMP_Ajax_Application_Handler_Common extends Horde_Core_Ajax_Application_Ha
 
         $this->_base->queue->poll($this->_base->indices->mailbox);
 
+        $result = false;
         if ($changed || $vp_vars->rangeslice || !$vp_vars->checkcache) {
             /* Ticket #7422: Listing messages may be a long-running operation
              * so close the session while we are doing it to prevent
              * deadlocks. */
-            $GLOBALS['session']->close();
+            $session->close();
 
-            $vp = $this->_base->viewPortData($changed);
+            try {
+                $vp = $this->_base->viewPortData($changed);
+                $result = true;
+
+                if (isset($vp_vars->delhide)) {
+                    $vp->metadata_reset = 1;
+                }
+            } catch (Exception $e) {
+                $vp = new IMP_Ajax_Application_Viewport_Error($this->_base->indices->mailbox);
+            }
 
             /* Reopen the session. */
-            $GLOBALS['session']->start();
+            $session->start();
 
-            if (isset($vp_vars->delhide)) {
-                $vp->metadata_reset = 1;
+            if ($result === false) {
+                $notification->push($e, 'horde.error');
             }
 
             $this->_base->addTask('viewport', $vp);
-            return true;
         }
 
-        return false;
+        $this->_base->queue->quota($this->_base->indices->mailbox, $vp_vars->checkcache);
+
+        return $result;
     }
 
     /**
@@ -440,6 +455,7 @@ class IMP_Ajax_Application_Handler_Common extends Horde_Core_Ajax_Application_Ha
             }
 
             $result = $compose->ajax->getResponse($resume);
+            $this->_base->queue->attachment($compose->compose, $this->vars->type);
         } catch (Horde_Exception $e) {
             $GLOBALS['notification']->push($e);
             $this->_base->checkUidvalidity();
@@ -469,6 +485,7 @@ class IMP_Ajax_Application_Handler_Common extends Horde_Core_Ajax_Application_Ha
      *
      * See the list of variables needed for
      * IMP_Ajax_Application#composeSetup(). Additional variables used:
+     *   - addr_ac: (string) TODO
      *   - encrypt: (integer) The encryption method to use (IMP ENCRYPT
      *              constants).
      *   - html: (integer) In HTML compose mode?
@@ -526,22 +543,30 @@ class IMP_Ajax_Application_Handler_Common extends Horde_Core_Ajax_Application_Ha
                     'pgp_attach_pubkey' => $this->vars->pgp_attach_pubkey,
                     'priority' => $this->vars->priority,
                     'readreceipt' => $this->vars->request_read_receipt,
-                    'save_attachments' => $this->vars->save_attachments_select,
                     'save_sent' => ($sm_displayed
                                     ? (bool)$this->vars->save_sent_mail
                                     : $identity->getValue('save_sent_mail')),
                     'sent_mail' => ($sm_displayed
                                       ? (isset($this->vars->save_sent_mail_mbox) ? IMP_Mailbox::formFrom($this->vars->save_sent_mail_mbox) : $identity->getValue(IMP_Mailbox::MBOX_SENT))
                                       : $identity->getValue(IMP_Mailbox::MBOX_SENT)),
+                    'signature' => $this->vars->signature,
+                    'strip_attachments' => (isset($this->vars->save_attachments_select) && !$this->vars->save_attachments_select),
                     'vcard_attach' => ($this->vars->vcard_attach ? $identity->getValue('fullname') : null)
                 )
             );
             $notification->push(empty($headers['subject']) ? _("Message sent successfully.") : sprintf(_("Message \"%s\" sent successfully."), Horde_String::truncate($headers['subject'])), 'horde.success');
+        } catch (IMP_Compose_Exception_Address $e) {
+            $this->_handleBadComposeAddr($e);
+            $result->success = 0;
+            return $result;
         } catch (IMP_Compose_Exception $e) {
             $result->success = 0;
 
-            if (!is_null($e->tied_identity)) {
+            if (is_null($e->tied_identity)) {
+                $notify_level = 'horde.error';
+            } else {
                 $result->identity = $e->tied_identity;
+                $notify_level = 'horde.warning';
             }
 
             if ($e->encrypt) {
@@ -581,7 +606,7 @@ class IMP_Ajax_Application_Handler_Common extends Horde_Core_Ajax_Application_Ha
 
         if ($indices = $imp_compose->getMetadata('indices')) {
             /* Update maillog information. */
-            $this->_base->queue->maillog($indices, $imp_compose->getMetadata('in_reply_to'));
+            $this->_base->queue->maillog($indices);
         }
 
         $imp_compose->destroy('send');
@@ -614,7 +639,9 @@ class IMP_Ajax_Application_Handler_Common extends Horde_Core_Ajax_Application_Ha
                 $subject = $val->headers->getValue('subject');
                 $GLOBALS['notification']->push(empty($subject) ? _("Message redirected successfully.") : sprintf(_("Message \"%s\" redirected successfully."), Horde_String::truncate($subject)), 'horde.success');
 
-                $this->_base->queue->maillog(new IMP_Indices($val->mbox, $val->uid), $val->headers->getValue('message-id'));
+                $this->_base->queue->maillog(
+                    new IMP_Indices($val->mbox, $val->uid)
+                );
             }
         } catch (Horde_Exception $e) {
             $GLOBALS['notification']->push($e);
@@ -683,4 +710,35 @@ class IMP_Ajax_Application_Handler_Common extends Horde_Core_Ajax_Application_Ha
         return $result;
     }
 
+    /* Internal methods. */
+
+    /**
+     * Handle bad addresses entered during a compose.
+     *
+     * @param IMP_Compose_Exception_Address $e  The address exception.
+     */
+    protected function _handleBadComposeAddr(IMP_Compose_Exception_Address $e)
+    {
+        global $notification;
+
+        $addr_ac = $this->vars->addr_ac
+            ? json_decode($this->vars->addr_ac, true)
+            : array();
+
+        foreach ($e as $val) {
+            $addr = strval($val->address);
+            $notification->push($val->error, 'horde.warning');
+
+            foreach ($addr_ac as $val2) {
+                if ($addr == $val2['addr']) {
+                    $this->_base->queue->compose_addr(
+                        $val2['id'],
+                        $val2['itemid'],
+                        ($val->level == $e::BAD) ? 'impACListItemBad' : 'impACListItemWarn'
+                    );
+                }
+            }
+        }
+    }
+
 }
