@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,11 +16,12 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
-class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_Handler
+class IMP_Ajax_Application_Handler_Dynamic
+extends Horde_Core_Ajax_Application_Handler
 {
     /**
      * The list of actions that require readonly access to the session.
@@ -38,18 +39,21 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      *   - mbox: (string) The name of the mailbox to check (base64url
      *           encoded).
      *
-     * @return boolean  True if submailboxes can be created.
+     * @return object  Object with the following properties:
+     *   - result: (boolean) True if submailboxes can be created.
      */
     public function createMailboxPrepare()
     {
         $mbox = IMP_Mailbox::formFrom($this->vars->mbox);
+        $ret = new stdClass;
+        $ret->result = true;
 
-        if ($mbox->access_creatembox) {
-            return true;
+        if (!$mbox->access_creatembox) {
+            $GLOBALS['notification']->push(sprintf(_("You may not create child mailboxes in \"%s\"."), $mbox->display), 'horde.error');
+            $ret->result = false;
         }
 
-        $GLOBALS['notification']->push(sprintf(_("You may not create child mailboxes in \"%s\"."), $mbox->display), 'horde.error');
-        return false;
+        return $ret;
     }
 
     /**
@@ -57,7 +61,6 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      *
      * Variables used:
      *   - mbox: (string) The name of the new mailbox.
-     *   - noexpand: (integer) Submailbox is not yet expanded.
      *   - parent: (string) The parent mailbox (base64url encoded).
      *
      * @return boolean  True on success, false on failure.
@@ -70,22 +73,15 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
 
         $result = false;
 
-        try {
-            $new_mbox = $GLOBALS['injector']->getInstance('IMP_Imap_Tree')->createMailboxName(
-                isset($this->vars->parent) ? IMP_Mailbox::formFrom($this->vars->parent) : '',
-                $this->vars->mbox
-            );
+        $parent = isset($this->vars->parent)
+            ? IMP_Mailbox::formFrom($this->vars->parent)
+            : IMP_Mailbox::get(IMP_Ftree::BASE_ELT);
+        $new_mbox = $parent->createMailboxName($this->vars->mbox);
 
-            if ($new_mbox->exists) {
-                $GLOBALS['notification']->push(sprintf(_("Mailbox \"%s\" already exists."), $new_mbox->display), 'horde.warning');
-            } elseif ($new_mbox->create()) {
-                $result = true;
-                if (isset($this->vars->parent) && $this->vars->noexpand) {
-                    $this->_base->queue->setMailboxOpt('noexpand', 1);
-                }
-            }
-        } catch (Horde_Exception $e) {
-            $GLOBALS['notification']->push($e);
+        if ($new_mbox->exists) {
+            $GLOBALS['notification']->push(sprintf(_("Mailbox \"%s\" already exists."), $new_mbox->display), 'horde.warning');
+        } elseif ($new_mbox->create()) {
+            $result = true;
         }
 
         return $result;
@@ -99,14 +95,17 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      *           encoded).
      *   - type: (string) Either 'delete' or 'rename'.
      *
-     * @return boolean  True if mailbox can be deleted/renamed.
+     * @return object  Object with the following properties:
+     *   - result: (boolean) True if mailbox can be deleted/renamed.
      */
     public function deleteMailboxPrepare()
     {
         $mbox = IMP_Mailbox::formFrom($this->vars->mbox);
+        $ret = new stdClass;
 
         if ($mbox->access_deletembox) {
-            return true;
+            $ret->result = true;
+            return $ret;
         }
 
         switch ($this->vars->type) {
@@ -119,7 +118,8 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
             break;
         }
 
-        return false;
+        $ret->result = false;
+        return $ret;
     }
 
     /**
@@ -146,7 +146,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      * Variables used:
      *   - new_name: (string) New mailbox name (child node) (UTF-8).
      *   - new_parent: (string) New parent name (UTF-8; base64url encoded).
-     *                 If not set, uses old parent.
+     *                 If not present, uses old parent.
      *   - old_name: (string) Full name of old mailbox (base64url encoded).
      *
      * @return boolean  True on success, false on failure.
@@ -157,20 +157,22 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
             return false;
         }
 
-        try {
-            $old_name = IMP_Mailbox::formFrom($this->vars->old_name);
+        $old_name = IMP_Mailbox::formFrom($this->vars->old_name);
+        if (isset($this->vars->new_parent)) {
+            $parent = strlen($this->vars->new_parent)
+                ? IMP_Mailbox::formFrom($this->vars->new_parent)
+                : IMP_Mailbox::get(IMP_Ftree::BASE_ELT);
+        } else {
+            $parent = IMP_Mailbox::get($old_name->parent);
+        }
 
-            $new_name = $GLOBALS['injector']->getInstance('IMP_Imap_Tree')->createMailboxName(
-                isset($this->vars->new_parent) ? IMP_Mailbox::formFrom($this->vars->new_parent) : $old_name->parent_imap,
-                $this->vars->new_name
-            );
+        if ($parent) {
+            $new_name = $parent->createMailboxName($this->vars->new_name);
 
             if (($old_name != $new_name) && $old_name->rename($new_name)) {
                 $this->_base->queue->setMailboxOpt('switch', $new_name->form_to);
                 return true;
             }
-        } catch (Horde_Exception $e) {
-            $GLOBALS['notification']->push($e);
         }
 
         return false;
@@ -184,24 +186,25 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      *   - mbox: (string) The name of the mailbox to check (base64url
      *           encoded).
      *
-     * @return integer  The number of messages to be deleted.
+     * @return object  Object with the following properties:
+     *   - result: (integer) The number of messages to be deleted.
      */
     public function emptyMailboxPrepare()
     {
         $mbox = IMP_Mailbox::formFrom($this->vars->mbox);
+        $res = new stdClass;
+        $res->result = 0;
 
         if (!$mbox->access_empty) {
             $GLOBALS['notification']->push(sprintf(_("The mailbox \"%s\" may not be emptied."), $mbox->display), 'horde.error');
-            return 0;
+        } else {
+            $poll_info = $mbox->poll_info;
+            if (!($res->result = $poll_info->msgs)) {
+                $GLOBALS['notification']->push(sprintf(_("The mailbox \"%s\" is already empty."), $mbox->display), 'horde.message');
+            }
         }
 
-        $poll_info = $mbox->poll_info;
-        if (empty($poll_info->msgs)) {
-            $GLOBALS['notification']->push(sprintf(_("The mailbox \"%s\" is already empty."), $mbox->display), 'horde.message');
-            return 0;
-        }
-
-        return $poll_info->msgs;
+        return $res;
     }
 
     /**
@@ -245,7 +248,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      */
     public function flagAll()
     {
-        $flags = Horde_Serialize::unserialize($this->vars->flags, Horde_Serialize::JSON);
+        $flags = json_decode($this->vars->flags);
         if (!$this->vars->mbox || empty($flags)) {
             return false;
         }
@@ -267,6 +270,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      * Variables used:
      *   - all: (integer) 1 to show all mailboxes.
      *   - base: (string) The base mailbox.
+     *   - expall: (boolean) 1 to expand all (requires 'all').
      *   - initial: (string) 1 to indicate the initial request for mailbox
      *              list.
      *   - mboxes: (string) The list of mailboxes to process (JSON encoded
@@ -278,17 +282,30 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      */
     public function listMailboxes()
     {
+        global $injector, $prefs, $session;
+
+        $ftree = $injector->getInstance('IMP_Ftree');
+        $iterator = new AppendIterator();
+
         /* This might be a long running operation. */
         if ($this->vars->initial) {
-            $GLOBALS['session']->close();
+            $session->close();
+            $ftree->eltdiff->clear();
+
+            /* @todo: Correctly handle unsubscribed mailboxes in ftree. */
+            if ($ftree->unsubscribed_loaded && !$this->vars->reload) {
+                $ftree->init();
+            }
         }
 
-        $imptree = $GLOBALS['injector']->getInstance('IMP_Imap_Tree');
-        $initreload = ($this->vars->initial || $this->vars->reload);
+        if ($this->vars->reload) {
+            $ftree->init();
+        }
 
-        $mask = IMP_Imap_Tree::FLIST_VFOLDER;
+        $filter = new IMP_Ftree_IteratorFilter($ftree);
         if ($this->vars->unsub) {
-            $mask |= IMP_Imap_Tree::FLIST_UNSUB;
+            $ftree->loadUnsubscribed();
+            $filter->remove($filter::UNSUB);
         }
 
         if (isset($this->vars->base)) {
@@ -297,65 +314,116 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
 
         if ($this->vars->all) {
             $this->_base->queue->setMailboxOpt('all', 1);
-        } else {
-            if ($initreload) {
-                $mask |= IMP_Imap_Tree::FLIST_ANCESTORS | IMP_Imap_Tree::FLIST_SAMELEVEL;
-
-                if ($GLOBALS['prefs']->getValue('nav_expanded')) {
-                    $this->_base->queue->setMailboxOpt('expand', 1);
-                    $mask |= IMP_Imap_Tree::FLIST_EXPANDED;
-                } else {
-                    $mask |= IMP_Imap_Tree::FLIST_NOCHILDREN;
-                }
-            } else {
-                $this->_base->queue->setMailboxOpt('expand', 1);
-                $mask |= IMP_Imap_Tree::FLIST_EXPANDED | IMP_Imap_Tree::FLIST_NOBASE;
+            $iterator->append($filter);
+            if ($this->vars->expall) {
+                $this->vars->action = 'expand';
+                $this->_base->callAction('toggleMailboxes');
             }
-        }
+        } elseif ($this->vars->initial || $this->vars->reload) {
+            $special = new ArrayIterator();
+            $special->append($ftree['INBOX']);
 
-        if ($this->vars->reload) {
-            $imptree->init();
-        }
-
-        $imptree->showUnsubscribed($this->vars->unsub);
-
-        if (!empty($this->vars->mboxes)) {
-            $mboxes = IMP_Mailbox::formFrom(Horde_Serialize::unserialize($this->vars->mboxes, Horde_Serialize::JSON));
-            if ($initreload) {
-                $mboxes = array_merge(array('INBOX'), array_diff($mboxes, array('INBOX')));
-            }
-
-            foreach ($mboxes as $val) {
-                $imptree->setIteratorFilter($mask, $val);
-                foreach ($imptree as $val2) {
-                    $imptree->addEltDiff($val2);
-                    $this->_base->queue->poll($val2);
-                }
-
-                if (!$initreload) {
-                    $imptree->expand($val);
-                }
-            }
-        }
-
-        /* Add special mailboxes explicitly to the initial folder list, since
-         * they are ALWAYS displayed, may appear outside of the folder
-         * slice requested, and need to be sorted logically. */
-        if ($initreload) {
+            /* Add special mailboxes explicitly to the initial folder list,
+             * since they are ALWAYS displayed, may appear outside of the
+             * folder slice requested, and need to be sorted logically. */
+            $s_elts = array();
             foreach (IMP_Mailbox::getSpecialMailboxesSort() as $val) {
-                if ($imptree[$val]) {
-                    $imptree->addEltDiff($val);
+                if (isset($ftree[$val])) {
+                    $special->append($val);
+                    $s_elts[] = $ftree[$val];
+                }
+            }
+            $iterator->append($special);
+
+            /* Go through and find any parent elements that contain only
+             * special mailbox children - this need to be suppressed in
+             * display. */
+            $filter2 = clone $filter;
+            $filter2->add(array($filter2::CONTAINERS, $filter2::SPECIALMBOXES));
+            $no_children = array();
+
+            foreach (array_unique($s_elts) as $val) {
+                while (($val = $val->parent) && !$val->base_elt) {
+                    $filter2->iterator = new IMP_Ftree_Iterator($val);
+                    foreach ($filter2 as $val) {
+                        /* If we found at least one viewable mailbox, this
+                         * element needs its children to be displayed. */
+                        break 2;
+                    }
+                    $no_children[] = strval($val);
                 }
             }
 
-            /* Poll all mailboxes on initial display. */
-            $this->_base->queue->poll($imptree->getPollList());
+            if (!empty($no_children)) {
+                $this->_base->queue->ftreeCallback = function($id, $ob) use ($no_children) {
+                    if (in_array($id, $no_children)) {
+                        unset($ob->ch);
+                    }
+                };
+            }
+
+            /* Add regular mailboxes. */
+            $no_mbox = false;
+
+            switch ($prefs->getValue('nav_expanded')) {
+            case IMP_Ftree_Prefs_Expanded::NO:
+                $filter->add($filter::CHILDREN);
+                break;
+
+            case IMP_Ftree_Prefs_Expanded::YES:
+                $this->_base->queue->setMailboxOpt('expand', 1);
+                $no_mbox = true;
+                break;
+
+            case IMP_Ftree_Prefs_Expanded::LAST:
+                $filter->add($filter::EXPANDED);
+                $this->_base->queue->setMailboxOpt('expand', 1);
+                break;
+            }
+
+            $filter->mboxes = array('INBOX');
+            $iterator->append($filter);
+
+            if (!$no_mbox) {
+                $mboxes = IMP_Mailbox::formFrom(json_decode($this->vars->mboxes));
+                foreach ($mboxes as $val) {
+                    if (!$val->inbox) {
+                        $ancestors = new IMP_Ftree_IteratorFilter(
+                            new IMP_Ftree_Iterator_Ancestors($val->tree_elt)
+                        );
+                        if ($this->vars->unsub) {
+                            $ancestors->remove($ancestors::UNSUB);
+                        }
+                        $iterator->append($ancestors);
+                    }
+                }
+            }
+        } else {
+            $filter->add($filter::EXPANDED);
+            $this->_base->queue->setMailboxOpt('expand', 1);
+
+            foreach (array_filter(IMP_Mailbox::formFrom(json_decode($this->vars->mboxes))) as $val) {
+                $filter->iterator = new IMP_Ftree_Iterator($val->tree_elt);
+                $iterator->append($filter);
+                $ftree->expand($val);
+            }
         }
 
-        $this->_base->queue->quota($this->_base->indices->mailbox);
+        array_map(
+            array($ftree->eltdiff, 'add'),
+            array_unique(iterator_to_array($iterator, false))
+        );
 
         if ($this->vars->initial) {
-            $GLOBALS['session']->start();
+            $session->start();
+
+            /* We need at least 1 changed mailbox. If not, something went
+             * wrong and we should reinitialize the folder list. */
+            if (!$ftree->eltdiff->changed_elts) {
+                $this->vars->reload = true;
+                $this->listMailboxes();
+                $this->vars->reload = false;
+            }
         }
 
         return true;
@@ -374,10 +442,10 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
         $this->_base->callAction('viewPort');
 
         $this->vars->initial = 1;
-        $this->vars->mboxes = Horde_Serialize::serialize(array($this->vars->mbox), Horde_Serialize::JSON);
+        $this->vars->mboxes = json_encode(array($this->vars->mailbox));
         $this->listMailboxes();
 
-        $this->_base->queue->flagConfig();
+        $this->_base->queue->flagConfig(Horde_Registry::VIEW_DYNAMIC);
 
         return true;
     }
@@ -396,24 +464,24 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      */
     public function modifyPoll()
     {
+        global $injector;
+
         if (!$this->vars->mbox) {
             return false;
         }
 
         $mbox = IMP_Mailbox::formFrom($this->vars->mbox);
 
-        $imptree = $GLOBALS['injector']->getInstance('IMP_Imap_Tree');
-
         $result = new stdClass;
         $result->add = intval($this->vars->add);
         $result->mbox = $this->vars->mbox;
 
         if ($this->vars->add) {
-            $imptree->addPollList($mbox);
+            $injector->getInstance('IMP_Ftree')->poll->addPollList($mbox);
             $this->_base->queue->poll($mbox);
             $GLOBALS['notification']->push(sprintf(_("\"%s\" mailbox now polled for new mail."), $mbox->display), 'horde.success');
         } else {
-            $imptree->removePollList($mbox);
+            $injector->getInstance('IMP_Ftree')->poll->removePollList($mbox);
             $GLOBALS['notification']->push(sprintf(_("\"%s\" mailbox no longer polled for new mail."), $mbox->display), 'horde.success');
         }
 
@@ -495,7 +563,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
             return false;
         }
 
-        $flags = Horde_Serialize::unserialize($this->vars->flags, Horde_Serialize::JSON);
+        $flags = json_decode($this->vars->flags);
 
         /* Check for non-system flags. If we find any, and the server supports
          * CONDSTORE, we should make sure that these flags are only updated if
@@ -513,7 +581,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
         if (!$this->_base->indices->mailbox->search &&
             $this->vars->viewport->cacheid &&
             array_diff($flags, $system_flags)) {
-            $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
+            $imp_imap = $this->_base->indices->mailbox->imp_imap;
             $parsed = $imp_imap->parseCacheId($this->vars->viewport->cacheid);
 
             try {
@@ -686,6 +754,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      * Variables used:
      *   - atc_indices: (string) [JSON array] Attachment IDs to delete.
      *   - imp_compose: (string) The IMP_Compose cache identifier.
+     *   - quiet: (boolean) If true, don't output notifications.
      *
      * @return array  The list of attchment IDs that were deleted.
      */
@@ -697,9 +766,11 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
 
         if (isset($this->vars->atc_indices)) {
             $imp_compose = $injector->getInstance('IMP_Factory_Compose')->create($this->vars->imp_compose);
-            foreach (Horde_Serialize::unserialize($this->vars->atc_indices, Horde_Serialize::JSON) as $val) {
+            foreach (json_decode($this->vars->atc_indices) as $val) {
                 if (isset($imp_compose[$val])) {
-                    $notification->push(sprintf(_("Deleted attachment \"%s\"."), Horde_Mime::decode($imp_compose[$val]->getPart()->getName(true))), 'horde.success');
+                    if (empty($this->vars->quiet)) {
+                        $notification->push(sprintf(_("Deleted attachment \"%s\"."), Horde_Mime::decode($imp_compose[$val]->getPart()->getName(true))), 'horde.success');
+                    }
                     unset($imp_compose[$val]);
                     $result[] = $val;
                     $this->_base->queue->compose($imp_compose);
@@ -707,7 +778,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
             }
         }
 
-        if (empty($result)) {
+        if (empty($result) && empty($this->vars->quiet)) {
             $notification->push(_("At least one attachment could not be deleted."), 'horde.error');
         }
 
@@ -765,21 +836,27 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      */
     public function sendMDN()
     {
+        global $injector, $notification;
+
         if (count($this->_base->indices) != 1) {
             return false;
         }
 
         try {
-            $contents = $GLOBALS['injector']->getInstance('IMP_Factory_Contents')->create($this->_base->indices);
+            $contents = $injector->getInstance('IMP_Factory_Contents')->create($this->_base->indices);
         } catch (IMP_Imap_Exception $e) {
             $e->notify(_("The Message Disposition Notification was not sent. This is what the server said") . ': ' . $e->getMessage());
             return false;
         }
 
         list($mbox, $uid) = $this->_base->indices->getSingle();
-        $GLOBALS['injector']->getInstance('IMP_Message_Ui')->MDNCheck($mbox, $uid, $contents->getHeaderAndMarkAsSeen(), true);
+        $injector->getInstance('IMP_Message_Ui')->MDNCheck(
+            new IMP_Indices($mbox, $uid),
+            $contents->getHeaderAndMarkAsSeen(),
+            true
+        );
 
-        $GLOBALS['notification']->push(_("The Message Disposition Notification was sent successfully."), 'horde.success');
+        $notification->push(_("The Message Disposition Notification was sent successfully."), 'horde.success');
 
         $result = new stdClass;
         $result->buid = $this->_base->vars->buid;
@@ -814,7 +891,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
 
         try {
             $this->_base->indices = new IMP_Indices_Mailbox(
-                $this->_base->mailbox,
+                $this->_base->indices->mailbox,
                 $injector->getInstance('IMP_Message')->stripPart($this->_base->indices, $this->vars->id)
             );
         } catch (IMP_Exception $e) {
@@ -838,41 +915,88 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
      * AJAX action: Convert HTML to text (compose data).
      *
      * Variables used:
-     *   - changed: (integer) Has the text changed from the original?
+     *   - data: (string) [JSON array] List of data to convert. Keys are UIDs
+     *           used to idetify the return values. Values are arrays with
+     *           these keys:
+     *     - changed: (integer) Has the text changed from the original?
+     *     - text: (string) The text to convert.
      *   - imp_compose: (string) The IMP_Compose cache identifier.
-     *   - text: (string) The text to convert.
      *
      * @return object  An object with the following entries:
-     *   - text: (string) The converted text.
+     *   - text: (array) Array with keys as UIDs and values as the converted
+     *           text string.
      */
     public function html2Text()
     {
+        return $this->_convertText('text');
+    }
+
+    /**
+     * AJAX action: Convert text to HTML (compose data).
+     *
+     * Variables used:
+     *   - data: (string) [JSON array] List of data to convert. Keys are UIDs
+     *           used to idetify the return values. Values are arrays with
+     *           these keys:
+     *     - changed: (integer) Has the text changed from the original?
+     *     - text: (string) The text to convert.
+     *   - imp_compose: (string) The IMP_Compose cache identifier.
+     *
+     * @return object  An object with the following entries:
+     *   - text: (array) Array with keys as UIDs and values as the converted
+     *           text string.
+     */
+    public function text2Html()
+    {
+        return $this->_convertText('html');
+    }
+
+    /**
+     * Helper for html2Text() and text2Html().
+     *
+     * @internal
+     */
+    protected function _convertText($mode)
+    {
+        global $injector;
+
+        $compose = null;
+
         $result = new stdClass;
+        $result->text = array();
 
-        if (!$this->vars->changed) {
-            $compose = $this->_base->initCompose();
+        foreach (json_decode($this->vars->data, true) as $key => $val) {
+            $tmp = null;
 
-            switch ($compose->compose->replyType()) {
-            case IMP_Compose::FORWARD_BODY:
-            case IMP_Compose::FORWARD_BOTH:
-                $data = $compose->compose->forwardMessageText($compose->contents, array(
-                    'format' => 'text'
-                ));
-                $result->text = $data['body'];
-                return $result;
+            if (empty($val['changed'])) {
+                if (!$compose) {
+                    $compose = $this->_base->initCompose();
+                }
 
-            case IMP_Compose::REPLY_ALL:
-            case IMP_Compose::REPLY_LIST:
-            case IMP_Compose::REPLY_SENDER:
-                $data = $compose->compose->replyMessageText($compose->contents, array(
-                    'format' => 'text'
-                ));
-                $result->text = $data['body'];
-                return $result;
+                switch ($compose->compose->replyType()) {
+                case IMP_Compose::FORWARD_BODY:
+                case IMP_Compose::FORWARD_BOTH:
+                    $data = $compose->compose->forwardMessageText($compose->contents, array(
+                        'format' => $mode
+                    ));
+                    $tmp = $data['body'];
+                    break;
+
+                case IMP_Compose::REPLY_ALL:
+                case IMP_Compose::REPLY_LIST:
+                case IMP_Compose::REPLY_SENDER:
+                    $data = $compose->compose->replyMessageText($compose->contents, array(
+                        'format' => $mode
+                    ));
+                    $tmp = $data['body'];
+                    break;
+                }
             }
-        }
 
-        $result->text = $GLOBALS['injector']->getInstance('IMP_Compose_Ui')->convertComposeText($this->vars->text, 'text');
+            $result->text[$key] = is_null($tmp)
+                ? $injector->getInstance('IMP_Compose_Ui')->convertComposeText($val['text'], $mode)
+                : $tmp;
+        }
 
         return $result;
     }
@@ -902,12 +1026,16 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
             if ($imp_compose->canUploadAttachment()) {
                 try {
                     $atc_ob = $imp_compose->addAttachmentFromUpload('upload');
-                    $atc_ob->related = true;
+                    if ($atc_ob[0] instanceof IMP_Compose_Exception) {
+                        throw $atc_ob[0];
+                    }
+
+                    $atc_ob[0]->related = true;
 
                     $data = array(
-                        IMP_Compose::RELATED_ATTR => 'src;' . $atc_ob->id
+                        IMP_Compose::RELATED_ATTR => 'src;' . $atc_ob[0]->id
                     );
-                    $url = strval($atc_ob->viewUrl());
+                    $url = strval($atc_ob[0]->viewUrl());
                 } catch (IMP_Compose_Exception $e) {
                     $data = $e->getMessage();
                 }
@@ -921,59 +1049,16 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
         return new Horde_Core_Ajax_Response_Raw(
             '<html>' .
                 Horde::wrapInlineScript(array(
-                    'window.parent.CKEDITOR.tools.callFunction(' . $this->vars->CKEditorFuncNum . ',' . Horde_Serialize::serialize($url, Horde_Serialize::JSON) . ',' . Horde_Serialize::serialize($data, Horde_Serialize::JSON) . ')'
+                    'window.parent.CKEDITOR.tools.callFunction(' . $this->vars->CKEditorFuncNum . ',' . json_encode($url) . ',' . json_encode($data) . ')'
                 )) .
             '</html>',
             'text/html'
         );
     }
 
-    /**
-     * AJAX action: Convert text to HTML (compose data).
-     *
-     * Variables used:
-     *   - changed: (integer) Has the text changed from the original?
-     *   - imp_compose: (string) The IMP_Compose cache identifier.
-     *   - text: (string) The text to convert.
-     *
-     * @return object  An object with the following entries:
-     *   - text: (string) The converted text.
-     */
-    public function text2Html()
-    {
-        $result = new stdClass;
-
-        if (!$this->vars->changed) {
-            $compose = $this->_base->initCompose();
-
-            switch ($compose->compose->replyType()) {
-            case IMP_Compose::FORWARD_BODY:
-            case IMP_Compose::FORWARD_BOTH:
-                $data = $compose->compose->forwardMessageText($compose->contents, array(
-                    'format' => 'html'
-                ));
-                $result->text = $data['body'];
-                return $result;
-
-            case IMP_Compose::REPLY_ALL:
-            case IMP_Compose::REPLY_LIST:
-            case IMP_Compose::REPLY_SENDER:
-                $data = $compose->compose->replyMessageText($compose->contents, array(
-                    'format' => 'html'
-                ));
-                $result->text = $data['body'];
-                return $result;
-            }
-        }
-
-        $result->text = $GLOBALS['injector']->getInstance('IMP_Compose_Ui')->convertComposeText($this->vars->text, 'html');
-
-        return $result;
-    }
-
     /**
      * AJAX action: Is the given mailbox fixed? Called dynamically to delay
-     * retrieveal of ACLs of all visible mailboxes at initialization.
+     * retrieval of ACLs of all visible mailboxes at initialization.
      *
      * Variables used:
      *   - mbox: (integer) The mailbox name.
@@ -1011,7 +1096,7 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
         $imp_flags = $injector->getInstance('IMP_Flags');
 
         try {
-            $imp_flags->addFlag($this->vars->flagname);
+            $imapflag = $imp_flags->addFlag($this->vars->flagname);
         } catch (IMP_Exception $e) {
             $notification->push($e, 'horde.error');
             $ret->success = false;
@@ -1023,10 +1108,10 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
         }
 
         $this->vars->add = true;
-        $this->vars->flags = Horde_Serialize::serialize(array($this->vars->flagname), Horde_Serialize::JSON);
+        $this->vars->flags = json_encode(array($imapflag));
         $this->flagMessages();
 
-        $this->_base->queue->flagConfig();
+        $this->_base->queue->flagConfig(Horde_Registry::VIEW_DYNAMIC);
 
         $name = 'imp:viewport';
         if ($this->_base->tasks->$name) {
@@ -1036,4 +1121,160 @@ class IMP_Ajax_Application_Handler_Dynamic extends Horde_Core_Ajax_Application_H
         return $ret;
     }
 
+    /**
+     * AJAX action: Generate the sent-mail select list.
+     *
+     * Variables used: NONE
+     *
+     * @return object  An object with the following properties:
+     * <pre>
+     *   - flist: (array) TODO
+     * </pre>
+     */
+    public function sentMailList()
+    {
+        global $injector;
+
+        /* Check to make sure the sent-mail mailboxes are created; they need
+         * to exist to show up in drop-down list. */
+        $identity = $injector->getInstance('IMP_Identity');
+        foreach (array_keys($identity->getAll('id')) as $ident) {
+            $mbox = $identity->getValue(IMP_Mailbox::MBOX_SENT, $ident);
+            if ($mbox instanceof IMP_Mailbox) {
+                $mbox->create();
+            }
+        }
+
+        $flist = array();
+        $iterator = new IMP_Ftree_IteratorFilter($injector->getInstance('IMP_Ftree'));
+        $iterator->add($iterator::NONIMAP);
+
+        foreach ($iterator as $val) {
+            $mbox_ob = $val->mbox_ob;
+            $tmp = array(
+                'f' => $mbox_ob->display,
+                'l' => Horde_String::abbreviate(str_repeat(' ', 2 * $val->level) . $mbox_ob->basename, 30),
+                'v' => $val->container ? '' : $mbox_ob->form_to
+            );
+            if ($tmp['f'] == $tmp['v']) {
+                unset($tmp['f']);
+            }
+            $flist[] = $tmp;
+        }
+
+        $ret = new stdClass;
+        $ret->flist = $flist;
+
+        return $ret;
+    }
+
+    /**
+     * AJAX action: Redirect to the filter edit page and pre-populate with
+     * an e-mail address.
+     *
+     * Requires EITHER 'addr' -or- mailbox/indices from form params.
+     *
+     * Variables used:
+     * <pre>
+     *   - addr: (string) The e-mail address to use.
+     * </pre>
+     *
+     * @return Horde_Core_Ajax_Response_HordeCore_Reload  Object with URL to
+     *                                                    redirect to.
+     */
+    public function newFilter()
+    {
+        global $injector, $notification, $registry;
+
+        if (isset($this->vars->addr)) {
+            $addr_ob = $injector->getInstance('IMP_Dynamic_AddressList')->parseAddressList($this->vars->addr);
+        } else {
+            $query = new Horde_Imap_Client_Fetch_Query();
+            $query->envelope();
+
+            $imp_imap = $this->_base->indices->mailbox->imp_imap;
+            list($mbox, $uid) = $this->_base->indices->getSingle();
+            $ret = $imp_imap->fetch($mbox, $query, array(
+                'ids' => $imp_imap->getIdsOb($uid)
+            ));
+
+            $addr_ob = $ret[$uid]->getEnvelope()->from;
+        }
+
+        // TODO: Currently supports only a single, non-group contact.
+        $ob = $addr_ob[0];
+        if (!$ob) {
+            return false;
+        } elseif ($ob instanceof Horde_Mail_Rfc822_Group) {
+            $notification->push(_("Editing group lists not currently supported."), 'horde.warning');
+            return false;
+        }
+
+        try {
+            return new Horde_Core_Ajax_Response_HordeCore_Reload(
+                $registry->link('mail/newEmailFilter', array(
+                    'email' => $ob->bare_address
+                ))
+            );
+        } catch (Horde_Exception $e) {
+            return false;
+        }
+    }
+
+    /**
+     * AJAX action: Return the contacts images for a given e-mail address.
+     *
+     * Variables used:
+     * <pre>
+     *   - addr: (string) The e-mail address.
+     * </pre>
+     *
+     * @return object  An object with the following properties:
+     * <pre>
+     *   - avatar: (string) The URL of the avatar image.
+     *   - flag: (string) The URL of the sender's country flag image.
+     *   - flagname: (string) The name of the country of the sender.
+     * </pre>
+     */
+    public function getContactsImage()
+    {
+        $contacts_img = new IMP_Contacts_Image($this->vars->addr);
+        $out = new stdClass;
+
+        try {
+            $res = $contacts_img->getImage($contacts_img::AVATAR);
+            $out->avatar = strval($res['url']);
+        } catch (IMP_Exception $e) {}
+
+        try {
+            $res = $contacts_img->getImage($contacts_img::FLAG);
+            $out->flag = strval($res['url']);
+            $out->flagname = $res['desc'];
+        } catch (IMP_Exception $e) {}
+
+        return $out;
+    }
+
+    /**
+     * AJAX action: Determine the size of a mailbox.
+     *
+     * Variables used:
+     *   - mbox: (string) The name of the mailbox to check (base64url
+     *           encoded).
+     *
+     * @return object  An object with the following properties:
+     * <pre>
+     *   - size: (string) Formatted size string.
+     * </pre>
+     */
+    public function mailboxSize()
+    {
+        $mbox = IMP_Mailbox::formFrom($this->vars->mbox);
+
+        $ret = new stdClass;
+        $ret->size = $mbox->size;
+
+        return $ret;
+    }
+
 }
