@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2005-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2005-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2005-2013 Horde LLC
+ * @copyright 2005-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2005-2013 Horde LLC
+ * @copyright 2005-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -122,7 +122,7 @@ class IMP_Ajax_Application_ListMessages
 
         /* Optimization: saves at least a STATUS and an EXAMINE call since
          * we will eventually open mailbox READ-WRITE. */
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $mbox->imp_imap;
         $imp_imap->openMailbox($mbox, Horde_Imap_Client::OPEN_READWRITE);
 
         /* Create the base object. */
@@ -131,17 +131,20 @@ class IMP_Ajax_Application_ListMessages
 
         if ($is_search) {
             /* For search mailboxes, we need to invalidate all browser data
-             * and repopulate, since BUIDs may have changed. */
-             $args['cache'] = array();
-             $args['change'] = true;
-             $result->data_reset = $result->rowlist_reset = true;
+             * and repopulate on force update, since BUIDs may have
+             * changed (TODO: only do this if search mailbox has changed?). */
+            if (!empty($args['change'])) {
+                 $args['cache'] = array();
+                 $args['change'] = true;
+                 $result->data_reset = $result->rowlist_reset = true;
+            }
         } elseif (!$args['initial'] && $args['cacheid'] && $args['cache']) {
             /* Check for UIDVALIDITY expiration. If it has changed, we need to
              * purge the cached items on the browser. */
             $parsed = $imp_imap->parseCacheId($args['cacheid']);
             $uid_expire = true;
 
-            if (!$parsed['date'] != date('z')) {
+            if ($parsed['date'] == date('z')) {
                 try {
                     $imp_imap->sync($mbox, $parsed['token'], array(
                         'criteria' => Horde_Imap_Client::SYNC_UIDVALIDITY
@@ -175,6 +178,9 @@ class IMP_Ajax_Application_ListMessages
 
         /* Actions only done on 'initial' request. */
         if ($args['initial']) {
+            /* Load queue information on original request. */
+            $injector->getInstance('IMP_Ajax_Queue')->quota($mbox, true);
+
             if (!$mbox->is_imap) {
                 $result->setMetadata('pop3', 1);
             }
@@ -447,14 +453,17 @@ class IMP_Ajax_Application_ListMessages
             $msg['size'] = IMP::sizeFormat($ob['size']);
 
             /* Format the Date: Header. */
-            $msg['date'] = $imp_ui->getDate($ob['envelope']->date);
+            $msg['date'] = $imp_ui->getDate(isset($ob['envelope']->date) ? $ob['envelope']->date : null);
 
             /* Format the From: Header. */
             $getfrom = $imp_ui->getFrom($ob['envelope']);
             $msg['from'] = $getfrom['from'];
+            if ($getfrom['from'] !== $getfrom['from_addr']) {
+                $msg['fromaddr'] = $getfrom['from_addr'];
+            }
 
             /* Format the Subject: Header. */
-            $msg['subject'] = $imp_ui->getSubject($ob['envelope']->subject, true);
+            $msg['subject'] = $imp_ui->getSubject($ob['envelope']->subject);
 
             /* Check to see if this is a list message. Namely, we want to
              * check for 'List-Post' information because that is the header
@@ -469,7 +478,11 @@ class IMP_Ajax_Application_ListMessages
 
         /* Allow user to alter template array. */
         try {
-            $msgs = Horde::callHook('mailboxarray', array($msgs), 'imp');
+            $msgs = $injector->getInstance('Horde_Core_Hooks')->callHook(
+                'mailboxarray',
+                'imp',
+                array($msgs)
+            );
         } catch (Horde_Exception_HookNotSet $e) {}
 
         return $msgs;
