@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2005-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2005-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2005-2013 Horde LLC
+ * @copyright 2005-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2005-2013 Horde LLC
+ * @copyright 2005-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -71,7 +71,7 @@ class IMP_Ajax_Application_ShowMessage
             $query = new Horde_Imap_Client_Fetch_Query();
             $query->envelope();
 
-            $imp_imap = $injector->getInstance('IMP_Imap');
+            $imp_imap = $mbox->imp_imap;
             $imp_imap->openMailbox($mbox, Horde_Imap_Client::OPEN_READWRITE);
 
             $ret = $imp_imap->fetch($mbox, $query, array(
@@ -126,7 +126,7 @@ class IMP_Ajax_Application_ShowMessage
      */
     public function showMessage($args)
     {
-        global $injector, $page_output, $prefs, $registry;
+        global $injector, $page_output, $prefs, $registry, $session;
 
         $preview = !empty($args['preview']);
 
@@ -200,7 +200,7 @@ class IMP_Ajax_Application_ShowMessage
 
         /* Maillog information. */
         $ajax_queue = $injector->getInstance('IMP_Ajax_Queue');
-        $ajax_queue->maillog($this->_indices, $this->_envelope->message_id);
+        $ajax_queue->maillog($this->_indices);
 
         if (!$preview) {
             /* Display the user-specified headers for the current identity. */
@@ -208,7 +208,15 @@ class IMP_Ajax_Application_ShowMessage
             foreach ($user_hdrs as $user_hdr) {
                 $user_val = $mime_headers->getValue($user_hdr);
                 if (!empty($user_val)) {
-                    $headers[] = array('name' => $user_hdr, 'value' => htmlspecialchars($user_val));
+                    if (!is_array($user_val)) {
+                        $user_val = array($user_val);
+                    }
+                    foreach ($user_val as $val) {
+                        $headers[] = array(
+                            'name' => $user_hdr,
+                            'value' => htmlspecialchars($val)
+                        );
+                    }
                 }
             }
             $result['headers'] = array_values($headers);
@@ -268,7 +276,7 @@ class IMP_Ajax_Application_ShowMessage
         /* Do MDN processing now. */
         switch ($registry->getView()) {
         case $registry::VIEW_DYNAMIC:
-            if ($imp_ui->MDNCheck($mbox, $uid, $mime_headers)) {
+            if ($imp_ui->MDNCheck(new IMP_Indices($mbox, $uid), $mime_headers)) {
                 $status = new IMP_Mime_Status(array(
                     _("The sender of this message is requesting notification from you when you have read this message."),
                     sprintf(_("Click %s to send the notification message."), Horde::link('#', '', '', '', '', '', '', array('id' => 'send_mdn_link')) . _("HERE") . '</a>')
@@ -280,13 +288,13 @@ class IMP_Ajax_Application_ShowMessage
 
         /* Build body text. This needs to be done before we build the
          * attachment list. */
-        $this->_contents->fetchCloseSession = true;
+        $session->close();
         $inlineout = $this->_contents->getInlineOutput(array(
             'mask' => $contents_mask,
             'part_info_display' => $part_info_display,
             'show_parts' => $show_parts
         ));
-        $this->_contents->fetchCloseSession = false;
+        $session->start();
 
         $result['js'] = array_merge($result['js'], $inlineout['js_onload']);
         $result['msgtext'] .= $inlineout['msgtext'];
@@ -323,13 +331,13 @@ class IMP_Ajax_Application_ShowMessage
                 $summary = $this->_contents->getSummary($id, $contents_mask);
                 $tmp = array();
                 foreach ($part_info as $val) {
-                    if (strlen($summary[$val])) {
+                    if (isset($summary[$val])) {
                         $tmp[$val] = ($summary[$val] instanceof Horde_Url)
                             ? strval($summary[$val]->setRaw(true))
                             : $summary[$val];
                     }
                 }
-                $partlist[] = $tmp;
+                $partlist[] = array_filter($tmp);
             }
 
             $result['atc_list'] = $partlist;
@@ -365,7 +373,7 @@ class IMP_Ajax_Application_ShowMessage
 
         /* Add changed flag information. */
         if (!$this->_peek && $mbox->is_imap) {
-            $status = $injector->getInstance('IMP_Imap')->status($mbox, Horde_Imap_Client::STATUS_PERMFLAGS);
+            $status = $mbox->imp_imap->status($mbox, Horde_Imap_Client::STATUS_PERMFLAGS);
             if (in_array(Horde_Imap_Client::FLAG_SEEN, $status['permflags'])) {
                 $ajax_queue->flag(array(Horde_Imap_Client::FLAG_SEEN), true, $this->_indices);
             }
