@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2009-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2009-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2009-2013 Horde LLC
+ * @copyright 2009-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -19,7 +19,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2009-2013 Horde LLC
+ * @copyright 2009-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -85,26 +85,38 @@ class IMP_Api extends Horde_Registry_Api
      *
      * @return array  The list of IMAP mailboxes. A list of arrays with the
      *                following keys:
-     *   - a: (integer) The mailbox attributes.
      *   - d: (string) The namespace delimiter.
      *   - label: (string) Human readable label (UTF-8).
      *   - level: (integer) The child level of this element.
      *   - ob: (Horde_Imap_Client_Mailbox) A mailbox object.
+     *   - subscribed: (boolean) True if mailbox is subscribed (@since 6.2.0).
      */
     public function mailboxList()
     {
+        global $injector;
+
+        $iterator = new IMP_Ftree_IteratorFilter(
+            $injector->getInstance('IMP_Ftree')
+        );
+        $iterator->add(array(
+            $iterator::CONTAINERS,
+            $iterator::REMOTE,
+            $iterator::VFOLDER
+        ));
         $mboxes = array();
-        $imap_tree = $GLOBALS['injector']->getInstance('IMP_Imap_Tree');
+
+        foreach ($iterator as $val) {
+            $mbox_ob = $val->mbox_ob;
+            $sub = $mbox_ob->sub;
 
-        $imap_tree->setIteratorFilter(IMP_Imap_Tree::FLIST_NOCONTAINER);
-        foreach ($imap_tree as $val) {
-            $e = $imap_tree->getElement($val->value);
             $mboxes[] = array(
-                'a' => $e['a'],
-                'd' => $val->namespace_delimiter,
-                'label' => $val->label,
+                // TODO: Remove for IMP 7.
+                'a' => $sub ? 8 : 0,
+                'd' => $mbox_ob->namespace_delimiter,
+                'label' => $mbox_ob->label,
                 'level' => $val->level,
-                'ob' => $val->imap_mbox_ob
+                'ob' => $mbox_ob->imap_mbox_ob,
+                'subscribed' => $sub
             );
         }
 
@@ -237,7 +249,7 @@ class IMP_Api extends Horde_Registry_Api
      */
     public function server()
     {
-        $imap_ob = $GLOBALS['injector']->getInstance('IMP_Imap');
+        $imap_ob = $GLOBALS['injector']->getInstance('IMP_Factory_Imap')->create();
 
         return array(
             'hostspec' => $imap_ob->getParam('hostspec'),
@@ -294,6 +306,33 @@ class IMP_Api extends Horde_Registry_Api
         return $GLOBALS['injector']->getInstance('IMP_Sentmail')->favouriteRecipients($limit, $filter);
     }
 
+    /**
+     * Log an entry to the IMP_Sentmail system.
+     *
+     * @param string $action            The performed action. One of:
+     * <pre>
+     *   - forward
+     *   - mdn
+     *   - new
+     *   - redirect
+     *   - reply
+     * </pre>
+     * @param string|array $recipients  The message recipients.
+     * @param string $message_id        The Message-ID.
+     * @param boolean $success          Was the message successfully sent?
+     */
+    public function logRecipient(
+        $reason, $recipients, $message_id, $success = true
+    )
+    {
+        $GLOBALS['injector']->getInstance('IMP_Sentmail')->log(
+            $reason,
+            $message_id,
+            $recipients,
+            $success
+        );
+    }
+
     /**
      * Returns the Horde_Imap_Client object created using the IMP credentials.
      *
@@ -301,7 +340,7 @@ class IMP_Api extends Horde_Registry_Api
      */
     public function imapOb()
     {
-        return $GLOBALS['injector']->getInstance('IMP_Imap')->getOb();
+        return $GLOBALS['injector']->getInstance('IMP_Factory_Imap')->create()->client_ob;
     }
 
     /**
@@ -334,6 +373,7 @@ class IMP_Api extends Horde_Registry_Api
      * Obtain the Maillog for a given message.
      *
      * @since 6.1.0
+     * @todo  This should not be returning a Horde_History_Log object.
      *
      * @param string $mid  The Message-ID to obtain the log for.
      *
@@ -341,15 +381,34 @@ class IMP_Api extends Horde_Registry_Api
      */
     public function getMaillog($mid)
     {
-        return ($log = $GLOBALS['injector']->getInstance('IMP_Maillog'))
-            ? $log->getLog($mid)
-            : new Horde_History_Log($mid);
+        global $injector, $registry;
+
+        $log = $injector->getInstance('IMP_Maillog')->getLog(
+            new IMP_Maillog_Message($mid)
+        );
+
+        $history = array();
+        foreach ($log as $val) {
+            $history[] = array(
+                'history_action' => $val->action,
+                'history_desc' => '',
+                'history_id' => 0,
+                'history_modseq' => 0,
+                'history_ts' => $val->timestamp,
+                'history_who' => $registry->getAuth()
+            );
+        }
+
+        return new Horde_History_Log($mid, $history);
     }
 
     /**
      * Log an entry in the Maillog.
      *
      * @since 6.1.0
+     * @todo  Rewrite this. $action and $data are both IMP specific, so they
+     *        aren't intended to be set from outside of IMP. And $mid should
+     *        be replaced by mailbox/UID.
      *
      * @param string $action  The action to log.
      * @param string $mid     The Message-ID.
@@ -357,9 +416,36 @@ class IMP_Api extends Horde_Registry_Api
      */
     public function logMaillog($action, $mid, $data = null)
     {
-        if ($log = $GLOBALS['injector']->getInstance('IMP_Maillog')) {
-            $log->log($action, $mid, $data);
+        switch ($action) {
+        case 'forward':
+            $log = new IMP_Maillog_Log_Forward($data['recipients']);
+            break;
+
+        case 'mdn':
+            $log = new IMP_Maillog_Log_Mdn();
+            break;
+
+        case 'redirect':
+            $log = new IMP_Maillog_Log_Redirect($data['recipients']);
+            break;
+
+        case 'reply':
+            $log = new IMP_Maillog_Log_Reply();
+            break;
+
+        case 'reply_all':
+            $log = new IMP_Maillog_Log_Replyall();
+            break;
+
+        case 'reply_list':
+            $log = new IMP_Maillog_Log_Replylist();
+            break;
         }
+
+        $GLOBALS['injector']->getInstance('IMP_Maillog')->log(
+            new IMP_Maillog_Message($mid),
+            $log
+        );
     }
 
     /**
@@ -367,6 +453,7 @@ class IMP_Api extends Horde_Registry_Api
      * the specified timestamp.
      *
      * @since 6.1.0
+     * @todo  This should not be returning Message-IDs.
      *
      * @param integer $ts  The timestamp to start searching from. Only entries
      *                     after this timestamp will be returned.
@@ -375,9 +462,10 @@ class IMP_Api extends Horde_Registry_Api
      */
     public function getMaillogChanges($ts)
     {
-        return ($log = $GLOBALS['injector']->getInstance('IMP_Maillog'))
-            ? preg_replace('/^([^:]*:){2}/', '', array_keys($log->getChanges($ts)))
-            : array();
+        return array_map(
+            'strval',
+            $GLOBALS['injector']->getInstance('IMP_Maillog')->getChanges($ts)
+        );
     }
 
 }
