@@ -66,7 +66,7 @@ class IMP_Application extends Horde_Registry_Application
 
     /**
      */
-    public $version = 'H5 (6.2.1)';
+    public $version = 'H5 (6.2.2)';
 
     /**
      * Server key used in logged out session.
@@ -505,7 +505,7 @@ class IMP_Application extends Horde_Registry_Application
             try {
                 $val = $func();
                 if ($val instanceof $key) {
-                    $drivers[] = $val;
+                    $out[] = $val;
                 }
             } catch (Horde_Exception $e) {
             }
