@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -15,8 +15,7 @@
  * This class provides authentication for IMP.
  *
  * The following is the list of IMP session variables:
- *   - compose_cache: (array) List of compose objects that have not yet been
- *                    garbage collected.
+ *   - compose_ob: (array) Active compose objects.
  *   - file_upload: (integer) If file uploads are allowed, the max size.
  *   - filteravail: (boolean) Can we apply filters manually?
  *   - imap_ob: (IMP_Imap) The IMAP client object.
@@ -31,7 +30,7 @@
  * @author    Jon Parise <jon@horde.org>
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -66,7 +65,7 @@ class IMP_Auth
             $credentials['server'] = self::getAutoLoginServer();
         }
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $injector->getInstance('IMP_Factory_Imap')->create();
 
         // Check for valid IMAP Client object.
         if (!$imp_imap->init) {
@@ -76,7 +75,7 @@ class IMP_Auth
             }
 
             try {
-                $imp_imap->createImapObject($credentials['userId'], $credentials['password'], $credentials['server']);
+                $imp_imap->createBaseImapObject($credentials['userId'], $credentials['password'], $credentials['server']);
             } catch (IMP_Imap_Exception $e) {
                 self::_log(false, $imp_imap);
                 throw $e->authException();
@@ -114,6 +113,11 @@ class IMP_Auth
             $credentials['userId'] = $auth_ob->getCredential('userId');
         }
 
+        if (!isset($credentials['password']) ||
+            !strlen($credentials['password'])) {
+            return false;
+        }
+
         try {
             self::authenticate($credentials);
         } catch (Horde_Auth_Exception $e) {
@@ -131,30 +135,26 @@ class IMP_Auth
      */
     static protected function _log($status, $imap_ob)
     {
-        if ($status) {
-            $msg = 'Login success';
-            $level = 'NOTICE';
-        } else {
-            $msg = 'FAILED LOGIN';
-            $level = 'INFO';
-        }
-
+        $msg = $status
+            ? 'Login success'
+            : 'FAILED LOGIN';
         $user = $imap_ob->getParam('username');
-        if (($auth_id = $GLOBALS['registry']->getAuth()) !== false) {
+
+        if (($auth_id = $GLOBALS['registry']->getAuth()) &&
+            ($user != $auth_id)) {
             $user .= ' (Horde user ' . $auth_id . ')';
         }
 
-        $msg = sprintf(
-            $msg . ' for %s [%s]%s to {%s:%s%s}',
-            $user,
-            $_SERVER['REMOTE_ADDR'],
-            empty($_SERVER['HTTP_X_FORWARDED_FOR']) ? '' : ' (forwarded for [' . $_SERVER['HTTP_X_FORWARDED_FOR'] . '])',
-            $imap_ob->init ? $imap_ob->getParam('hostspec') : '',
-            $imap_ob->init ? $imap_ob->getParam('port') : '',
-            ' [' . ($imap_ob->isImap() ? 'imap' : 'pop') . ']'
+        Horde::log(
+            sprintf(
+                $msg . ' for %s (%s)%s to {%s}',
+                $user,
+                $_SERVER['REMOTE_ADDR'],
+                empty($_SERVER['HTTP_X_FORWARDED_FOR']) ? '' : ' (forwarded for [' . $_SERVER['HTTP_X_FORWARDED_FOR'] . '])',
+                $imap_ob->url
+            ),
+            $status ? 'NOTICE' : 'INFO'
         );
-
-        Horde::log($msg, $level);
     }
 
     /**
@@ -203,7 +203,7 @@ class IMP_Auth
     {
         global $injector, $registry;
 
-        if (($servers = $injector->getInstance('IMP_Imap')->loadServerConfig()) === false) {
+        if (($servers = $injector->getInstance('IMP_Factory_Imap')->create()->loadServerConfig()) === false) {
             return false;
         }
 
@@ -227,62 +227,6 @@ class IMP_Auth
         return false;
     }
 
-    /**
-     * Returns the initial page.
-     *
-     * @return object  Object with the following properties:
-     *   - mbox (IMP_Mailbox)
-     *   - url (Horde_Url)
-     */
-    static public function getInitialPage()
-    {
-        $init_url = $GLOBALS['prefs']->getValue('initial_page');
-        if (!$init_url ||
-            !$GLOBALS['injector']->getInstance('IMP_Imap')->access(IMP_Imap::ACCESS_FOLDERS)) {
-            $init_url = 'INBOX';
-        }
-
-        if ($init_url == IMP::INITIAL_FOLDERS) {
-            $mbox = null;
-        } else {
-            $mbox = IMP_Mailbox::get($init_url);
-            $GLOBALS['injector']->getInstance('Horde_Variables')->mailbox = $mbox->exists
-                ? $mbox->form_to
-                : IMP_Mailbox::get('INBOX')->form_to;
-        }
-
-        $result = new stdClass;
-        $result->mbox = $mbox;
-
-        switch ($GLOBALS['registry']->getView()) {
-        case Horde_Registry::VIEW_BASIC:
-            $result->url = is_null($mbox)
-                ? IMP_Basic_Folders::url()
-                : $mbox->url('mailbox');
-            break;
-
-        case Horde_Registry::VIEW_DYNAMIC:
-            $result->url = IMP_Dynamic_Mailbox::url(array(
-                'mailbox' => is_null($mbox) ? 'INBOX' : $mbox
-            ));
-            break;
-
-        case Horde_Registry::VIEW_MINIMAL:
-            $result->url = is_null($mbox)
-                ? IMP_Minimal_Folders::url()
-                : IMP_Minimal_Mailbox::url(array('mailbox' => $mbox));
-            break;
-
-        case Horde_Registry::VIEW_SMARTMOBILE:
-            $result->url = is_null($mbox)
-                ? Horde::url('smartmobile.php', true)
-                : $mbox->url('mailbox');
-            break;
-        }
-
-        return $result;
-    }
-
     /**
      * Perform post-login tasks. Session creation requires the full IMP
      * environment, which is not available until this callback.
@@ -293,7 +237,7 @@ class IMP_Auth
     {
         global $browser, $injector, $session;
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $injector->getInstance('IMP_Factory_Imap')->create();
 
         /* Perform post-login tasks for IMAP object. */
         $imp_imap->doPostLoginTasks();
