@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2000-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2000-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2000-2013 Horde LLC
+ * @copyright 2000-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -18,7 +18,7 @@
  * @author    Chuck Hagenbuch <chuck@horde.org>
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2000-2013 Horde LLC
+ * @copyright 2000-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -31,7 +31,7 @@ class IMP_Basic_Folders extends IMP_Basic_Base
         global $injector, $notification, $page_output, $prefs, $registry, $session;
 
         /* Redirect back to the mailbox if folder use is not allowed. */
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $injector->getInstance('IMP_Factory_Imap')->create();
         if (!$imp_imap->access(IMP_Imap::ACCESS_FOLDERS)) {
             $notification->push(_("The folder view is not enabled."), 'horde.error');
             Horde::url('mailbox', true)->redirect();
@@ -45,7 +45,7 @@ class IMP_Basic_Folders extends IMP_Basic_Base
         $page_output->addScriptFile('folders.js');
 
         /* Get the base URL for this page. */
-        $folders_url = self::url();
+        $folders_url = self::url()->setRaw(true);
 
         /* These JS defines are required by all sub-pages. */
         $page_output->addInlineJsVars(array(
@@ -63,18 +63,14 @@ class IMP_Basic_Folders extends IMP_Basic_Base
             )
         ));
 
-        /* Initialize the IMP_Imap_Tree object. */
-        $imaptree = $injector->getInstance('IMP_Imap_Tree');
+        /* Initialize the IMP_Ftree object. */
+        $ftree = $injector->getInstance('IMP_Ftree');
 
         /* $mbox_list entries are urlencoded. */
         $mbox_list = isset($this->vars->mbox_list)
             ? IMP_Mailbox::formFrom($this->vars->mbox_list)
             : array();
 
-        /* Token to use in requests */
-        $token = $injector->getInstance('Horde_Token');
-        $folders_token = $token->get('imp.folders');
-
         /* META refresh time (might be altered by actionID). */
         $refresh_time = $prefs->getValue('refresh_time');
 
@@ -85,13 +81,14 @@ class IMP_Basic_Folders extends IMP_Basic_Base
         $view->addHelper('FormTag');
         $view->addHelper('Tag');
 
-        $view->folders_token = $folders_token;
+        $token = $session->getToken();
+        $view->token = $token;
 
         /* Run through the action handlers. */
         if ($this->vars->actionID) {
             try {
-                $token->validate($this->vars->folders_token, 'imp.folders');
-            } catch (Horde_Token_Exception $e) {
+                $session->checkToken($this->vars->token);
+            } catch (Horde_Exception $e) {
                 $notification->push($e);
                 $this->vars->actionID = null;
             }
@@ -99,15 +96,15 @@ class IMP_Basic_Folders extends IMP_Basic_Base
 
         switch ($this->vars->actionID) {
         case 'expand_all_folders':
-            $imaptree->expandAll();
+            $ftree->expandAll();
             break;
 
         case 'collapse_all_folders':
-            $imaptree->collapseAll();
+            $ftree->collapseAll();
             break;
 
         case 'rebuild_tree':
-            $imaptree->init();
+            $ftree->init();
             break;
 
         case 'expunge_mbox':
@@ -147,10 +144,10 @@ class IMP_Basic_Folders extends IMP_Basic_Base
         case 'create_mbox':
             if (isset($this->vars->new_mailbox)) {
                 try {
-                    $new_mbox = $imaptree->createMailboxName(
-                        empty($mbox_list) ? null : $mbox_list[0],
-                        $this->vars->new_mailbox
-                    );
+                    $parent = empty($mbox_list)
+                        ? IMP_Mailbox::get(IMP_Ftree::BASE_ELT)
+                        : $mbox_list[0];
+                    $new_mbox = $parent->createMailboxName($this->vars->new_mailbox);
                     if ($new_mbox->exists) {
                         $notification->push(sprintf(_("Mailbox \"%s\" already exists."), $new_mbox->display), 'horde.warning');
                     } else {
@@ -174,15 +171,15 @@ class IMP_Basic_Folders extends IMP_Basic_Base
                 for ($i = 0; $i < $iMax; ++$i) {
                     $old_name = IMP_Mailbox::formFrom($old_names[$i]);
                     $old_ns = $old_name->namespace_info;
-                    $new = trim($new_names[$i], $old_ns['delimiter']);
+                    $new = trim($new_names[$i], $old_ns->delimiter);
 
                     /* If this is a personal namespace, then anything goes as
                      * far as the input. Just append the personal namespace to
                      * it. */
-                    if (($old_ns['type'] == Horde_Imap_Client::NS_PERSONAL) ||
-                        ($old_ns['name'] &&
-                         (stripos($new_names[$i], $old_ns['name']) !== 0))) {
-                        $new = $old_ns['name'] . $new;
+                    if (($old_ns->type === $old_ns::NS_PERSONAL) ||
+                        (strlen($old_ns->name) &&
+                         (stripos($new_names[$i], $old_ns->name) !== 0))) {
+                        $new = $old_ns->name . $new;
                     }
 
                     $old_name->rename($new);
@@ -205,19 +202,18 @@ class IMP_Basic_Folders extends IMP_Basic_Base
             if ($subscribe) {
                 $showAll = !$showAll;
                 $session->set('imp', 'showunsub', $showAll);
-                $imaptree->showUnsubscribed($showAll);
             }
             break;
 
         case 'poll_mbox':
             if (!empty($mbox_list)) {
-                $imaptree->addPollList($mbox_list);
+                $ftree->poll->addPollList($mbox_list);
             }
             break;
 
         case 'nopoll_mbox':
             if (!empty($mbox_list)) {
-                $imaptree->removePollList($mbox_list);
+                $ftree->poll->removePollList($mbox_list);
             }
             break;
 
@@ -299,10 +295,8 @@ class IMP_Basic_Folders extends IMP_Basic_Base
                 $loop = array();
                 $sum = 0;
 
-                $imp_message = $injector->getInstance('IMP_Message');
-
                 foreach ($mbox_list as $val) {
-                    $size = $imp_message->sizeMailbox($val, false);
+                    $size = $val->size;
                     $data = array(
                         'name' => $val->display,
                         'size' => sprintf(_("%.2fMB"), $size / (1024 * 1024)),
@@ -343,7 +337,7 @@ class IMP_Basic_Folders extends IMP_Basic_Base
 
         $this->title = _("Folder Navigator");
 
-        $folders_url->add('folders_token', $folders_token);
+        $folders_url->add('token', $token);
 
         /* Prepare the topbar. */
         $injector->getInstance('Horde_View_Topbar')->subinfo =
@@ -385,7 +379,7 @@ class IMP_Basic_Folders extends IMP_Basic_Base
             $actions->toggle_subscribe = Horde::widget(array(
                 'url' => $folders_url->copy()->add(array(
                     'actionID' => 'toggle_subscribed_view',
-                    'folders_token' => $folders_token
+                    'token' => $token
                 )),
                 'title' => $subToggleText,
                 'nocheck' => true
@@ -397,7 +391,7 @@ class IMP_Basic_Folders extends IMP_Basic_Base
         $actions->expand_all = Horde::widget(array(
             'url' => $folders_url->copy()->add(array(
                 'actionID' => 'expand_all_folders',
-                'folders_token' => $folders_token
+                'token' => $token
             )),
             'title' => _("Expand All"),
             'nocheck' => true
@@ -405,28 +399,35 @@ class IMP_Basic_Folders extends IMP_Basic_Base
         $actions->collapse_all = Horde::widget(array(
             'url' => $folders_url->copy()->add(array(
                 'actionID' => 'collapse_all_folders',
-                'folders_token' => $folders_token
+                'token' => $token
             )),
             'title' => _("Collapse All"),
             'nocheck' => true
         ));
 
         /* Build the folder tree. */
-        $imaptree->setIteratorFilter(IMP_Imap_Tree::FLIST_VFOLDER);
-        $tree = $imaptree->createTree('imp_folders', array(
+        $iterator = new IMP_Ftree_IteratorFilter($ftree);
+        $iterator->add(array($iterator::REMOTE, $iterator::VFOLDER));
+        if ($showAll) {
+            $ftree->loadUnsubscribed();
+            $iterator->remove($iterator::UNSUB);
+        }
+        $tree = $ftree->createTree('imp_folders', array(
             'checkbox' => true,
             'editvfolder' => true,
+            'iterator' => $iterator,
             'poll_info' => true
         ));
 
         $displayNames = $fullNames = array();
 
-        foreach ($imaptree as $key => $val) {
-            $tmp = $displayNames[] = $val->display;
+        foreach ($ftree as $val) {
+            $mbox_ob = $val->mbox_ob;
+            $tmp = $displayNames[] = $mbox_ob->display;
 
-            $tmp2 = $val->display_notranslate;
+            $tmp2 = $mbox_ob->display_notranslate;
             if ($tmp != $tmp2) {
-                $fullNames[$key] = $tmp2;
+                $fullNames[strval($val)] = $tmp2;
             }
         }
 
