@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,7 +17,7 @@
  * @author    Chuck Hagenbuch <chuck@horde.org>
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -27,10 +27,10 @@ class IMP_Basic_Message extends IMP_Basic_Base
      */
     protected function _init()
     {
-        global $conf, $injector, $notification, $page_output, $prefs, $registry;
+        global $conf, $injector, $notification, $page_output, $prefs, $registry, $session;
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
         $mailbox = $this->indices->mailbox;
+        $imp_imap = $mailbox->imp_imap;
 
         /* We know we are going to be exclusively dealing with this mailbox,
          * so select it on the IMAP server (saves some STATUS calls). Open R/W
@@ -45,7 +45,6 @@ class IMP_Basic_Message extends IMP_Basic_Base
             return;
         }
 
-        $horde_token = $injector->getInstance('Horde_Token');
         $imp_flags = $injector->getInstance('IMP_Flags');
         $imp_identity = $injector->getInstance('IMP_Identity');
         $imp_message = $injector->getInstance('IMP_Message');
@@ -53,19 +52,9 @@ class IMP_Basic_Message extends IMP_Basic_Base
 
         /* Run through action handlers. */
         if ($this->vars->actionID) {
-            switch ($this->vars->actionID) {
-            case 'strip_attachment':
-                $token_name = 'imp.impcontents';
-                break;
-
-            default:
-                $token_name = 'imp.message';
-                break;
-            }
-
             try {
-                $horde_token->validate($this->vars->message_token, $token_name);
-            } catch (Horde_Token_Exception $e) {
+                $session->getToken($this->vars->token);
+            } catch (Horde_Exception $e) {
                 $notification->push($e);
                 $this->vars->actionID = null;
             }
@@ -250,12 +239,12 @@ class IMP_Basic_Message extends IMP_Basic_Base
         $buid = $imp_mailbox->getBuid($msg_index['m'], $msg_index['u']);
         $msgindex = $imp_mailbox->getIndex();
         $message_url = Horde::url('basic.php')->add('page', 'message');
-        $message_token = $horde_token->get('imp.message');
+        $token = $session->getToken();
         $self_link = self::url(array(
             'buid' => $buid,
             'mailbox' => $mailbox
         ))->add(array(
-            'message_token' => $message_token,
+            'token' => $token,
             'start' => $msgindex
         ));
 
@@ -271,26 +260,19 @@ class IMP_Basic_Message extends IMP_Basic_Base
         /* Build From address links. */
         $display_headers['from'] = $imp_ui->buildAddressLinks($envelope->from, $self_link);
 
-        /* Add country/flag image. Try X-Originating-IP first, then fall back
-         * to the sender's domain name. */
-        $from_img = '';
-        $origin_host = str_replace(array('[', ']'), '', $mime_headers->getValue('X-Originating-IP'));
-        if ($origin_host) {
-            if (!is_array($origin_host)) {
-                $origin_host = array($origin_host);
-            }
-            foreach ($origin_host as $host) {
-                $from_img .= Horde_Core_Ui_FlagImage::generateFlagImageByHost($host) . ' ';
-            }
-            trim($from_img);
-        }
-
-        if (empty($from_img) && !empty($envelope->from)) {
-            $from_img .= Horde_Core_Ui_FlagImage::generateFlagImageByHost($envelope->from[0]->host) . ' ';
-        }
-
-        if (!empty($from_img)) {
-            $display_headers['from'] .= '&nbsp;' . $from_img;
+        /* Add country/flag image. */
+        if (!empty($envelope->from)) {
+            $contacts_img = new IMP_Contacts_Image($envelope->from[0]);
+            try {
+                $res = $contacts_img->getImage($contacts_img::FLAG);
+                $display_headers['from'] .= '&nbsp;' . Horde_Themes_Image::tag(
+                    $res['url'],
+                    array(
+                        'alt' => $res['desc'],
+                        'fullsrc' => true
+                    )
+                );
+            } catch (IMP_Exception $e) {}
         }
 
         /* Look for Face information. */
@@ -339,21 +321,11 @@ class IMP_Basic_Message extends IMP_Basic_Base
 
         /* Determine if all/list/user-requested headers needed. */
         $all_headers = $this->vars->show_all_headers;
-        $list_headers = $this->vars->show_list_headers;
         $user_hdrs = $imp_ui->getUserHeaders();
 
         /* Check for the presence of mailing list information. */
         $list_info = $imp_ui->getListInformation($mime_headers);
 
-        /* See if the mailing list information has been requested to be
-         * displayed. */
-        if ($list_info['exists'] && ($list_headers || $all_headers)) {
-            $all_list_headers = $this->_parseAllListHeaders($mime_headers);
-            $list_headers_lookup = $mime_headers->listHeaders();
-        } else {
-            $all_list_headers = array();
-        }
-
         /* Display all headers or, optionally, the user-specified headers for
          * the current identity. */
         $full_headers = array();
@@ -364,7 +336,6 @@ class IMP_Basic_Message extends IMP_Basic_Base
 
                 /* Skip the header if we have already dealt with it. */
                 if (!isset($display_headers[$lc_head]) &&
-                    !isset($all_list_headers[$lc_head]) &&
                     (!in_array($lc_head, array('importance', 'x-priority')) ||
                     !isset($display_headers['priority']))) {
                     $full_headers[$lc_head] = $val;
@@ -384,8 +355,8 @@ class IMP_Basic_Message extends IMP_Basic_Base
          * as it may have changed if we deleted/copied/moved messages. We may
          * need other stuff in the query string, so we need to do an
          * add/remove of uid info. */
-        $selfURL = $mailbox->url(Horde::selfUrlParams()->remove(array('actionID')), $buid)->add('message_token', $message_token);
-        $headersURL = $selfURL->copy()->remove(array('show_all_headers', 'show_list_headers'));
+        $selfURL = $mailbox->url(Horde::selfUrlParams()->remove(array('actionID')), $buid)->add('token', $token);
+        $headersURL = $selfURL->copy()->remove(array('show_all_headers'));
 
         /* Generate previous/next links. */
         $prev_msg = $imp_mailbox[$imp_mailbox->getIndex() - 1];
@@ -476,7 +447,7 @@ class IMP_Basic_Message extends IMP_Basic_Base
         $t_view->message_url = $message_url;
         $t_view->mailbox = $mailbox->form_to;
         $t_view->start = $msgindex;
-        $t_view->message_token = $message_token;
+        $t_view->token = $token;
 
         /* Prepare the navbar navigate template. */
         $n_view = clone $view;
@@ -522,10 +493,17 @@ class IMP_Basic_Message extends IMP_Basic_Base
                 'title' => _("Copy"),
                 'nocheck' => true
             ));
-            $n_view->options = IMP::flistSelect(array(
+
+            $iterator = new IMP_Ftree_IteratorFilter(
+                $injector->getInstance('IMP_Ftree')
+            );
+            $iterator->add($iterator::NONIMAP);
+
+            $n_view->options = new IMP_Ftree_Select(array(
                 'heading' => _("This message to"),
                 'inc_tasklists' => true,
                 'inc_notepads' => true,
+                'iterator' => $iterator,
                 'new_mbox' => true
             ));
         }
@@ -736,7 +714,7 @@ class IMP_Basic_Message extends IMP_Basic_Base
             'title' => _("Headers"),
             'nocheck' => true
         ));
-        if ($all_headers || $list_headers) {
+        if ($all_headers) {
             $a_view->common_headers = Horde::widget(array(
                 'url' => $headersURL,
                 'title' => _("Show Common Headers"),
@@ -750,9 +728,14 @@ class IMP_Basic_Message extends IMP_Basic_Base
                 'nocheck' => true
             ));
         }
-        if ($list_info['exists'] && !$list_headers) {
+        if ($list_info['exists']) {
             $a_view->list_headers = Horde::widget(array(
-                'url' => $headersURL->copy()->add('show_list_headers', 1),
+                'onclick' => Horde::popupJs(IMP_Basic_Listinfo::url(array(
+                                 'buid' => $buid,
+                                 'mailbox' => $mailbox
+                             )), array(
+                                 'urlencode' => true
+                             )),
                 'title' => _("Show Mailing List Information"),
                 'nocheck' => true
             ));
@@ -782,12 +765,6 @@ class IMP_Basic_Message extends IMP_Basic_Base
                 );
             }
         }
-        foreach ($all_list_headers as $head => $val) {
-            $hdrs[] = array(
-                'name' => $list_headers_lookup[$head],
-                'val' => $val
-            );
-        }
 
         /* Determine the fields that will appear in the MIME info entries. */
         $part_info = $part_info_display = array('icon', 'description', 'size');
@@ -809,7 +786,7 @@ class IMP_Basic_Message extends IMP_Basic_Base
             IMP_Contents::SUMMARY_PRINT;
 
         /* Do MDN processing now. */
-        $mdntext = $imp_ui->MDNCheck($msg_index['m'], $buid, $mime_headers, $this->vars->mdn_confirm)
+        $mdntext = $imp_ui->MDNCheck(new IMP_Indices($msg_index['m'], $buid), $mime_headers, $this->vars->mdn_confirm)
             ? strval(new IMP_Mime_Status(array(
                 _("The sender of this message is requesting a notification from you when you have read this message."),
                 sprintf(_("Click %s to send the notification message."), Horde::link($selfURL->copy()->add('mdn_confirm', 1)) . _("HERE") . '</a>')
@@ -853,7 +830,7 @@ class IMP_Basic_Message extends IMP_Basic_Base
                 $a_view->strip_all = Horde::widget(array(
                     'url' => Horde::selfUrlParams()->add(array(
                         'actionID' => 'strip_all',
-                        'message_token' => $message_token
+                        'token' => $token
                     )),
                     'class' => 'stripAllAtc',
                     'title' => _("Strip All Attachments"),
@@ -927,16 +904,16 @@ class IMP_Basic_Message extends IMP_Basic_Base
 
         if (!empty($conf['tasklist']['use_notepad']) ||
             !empty($conf['tasklist']['use_tasklist'])) {
-            $page_output->addScriptPackage('Dialog');
+            $page_output->addScriptPackage('Horde_Core_Script_Package_Dialog');
         }
 
         $page_output->noDnsPrefetch();
 
-        if ($maillog = $injector->getInstance('IMP_Maillog')) {
-            Horde::startBuffer();
-            $maillog->displayLog($envelope->message_id);
-            $this->output = Horde::endBuffer();
+        Horde::startBuffer();
+        foreach ($injector->getInstance('IMP_Maillog')->getLog(new IMP_Maillog_Message($this->indices, array('mdn'))) as $val) {
+            $notification->push($val->message, 'imp.' . $val->action);
         }
+        $this->output = Horde::endBuffer();
 
         $this->output .= $t_view->render('navbar_top') .
             $n_view->render('navbar_navigate') .
@@ -973,64 +950,4 @@ class IMP_Basic_Message extends IMP_Basic_Base
         $this->output = $ob->output;
     }
 
-    /**
-     * Parses all of the available mailing list headers.
-     *
-     * @param Horde_Mime_Headers $headers  A Horde_Mime_Headers object.
-     *
-     * @return array  Keys are the list header names, values are the
-     *                parsed list header values.
-     */
-    protected function _parseAllListHeaders($headers)
-    {
-        $ret = array();
-
-        foreach (array_keys($headers->listHeaders()) as $val) {
-            if ($data = $headers->getValue($val)) {
-                $ret[$val] = $this->_parseListHeaders($val, $data);
-            }
-        }
-
-        return $ret;
-    }
-
-    /**
-     * Parse the information in mailing list headers.
-     *
-     * @param string $id    The header ID.
-     * @param string $data  The header text to process.
-     *
-     * @return string  The header value.
-     */
-    protected function _parseListHeaders($id, $data)
-    {
-        $output = '';
-        $parser = $GLOBALS['injector']->getInstance('Horde_ListHeaders');
-        $text_filter = $GLOBALS['injector']->getInstance('Horde_Core_Factory_TextFilter');
-
-        foreach ($parser->parse($id, $data) as $val) {
-            /* RFC 2369 [2] states that we should only show the *FIRST* URL
-             * that appears in a header that we can adequately handle. */
-            if (stripos($val->url, 'mailto:') === 0) {
-                $url = substr($val->url, 7);
-                $clink = new IMP_Compose_Link($url);
-                $output = Horde::link($clink->link()) . $url . '</a>';
-                foreach ($val->comments as $val2) {
-                    $output .= '&nbsp;(' . $val2 . ')';
-                }
-                break;
-            } elseif ($url = $text_filter->filter($val->url, 'linkurls')) {
-                $output = $url;
-                foreach ($val->comments as $val2) {
-                    $output .= '&nbsp;(' . $val2 . ')';
-                }
-                break;
-            }
-        }
-
-        return strlen($output)
-            ? $output
-            : htmlspecialchars($data);
-    }
-
 }
