@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -19,7 +19,7 @@
  * @author    Chuck Hagenbuch <chuck@horde.org>
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -27,6 +27,7 @@ class IMP_Basic_Search extends IMP_Basic_Base
 {
     /**
      * URL Parameters:
+     * <pre>
      *   - criteria_form: (string) JSON representation of the search query.
      *   - edit_query: (integer) If true, edit a search query (contained in
      *                 'mailbox' parameter).
@@ -44,6 +45,7 @@ class IMP_Basic_Search extends IMP_Basic_Base
      *                 'vfolder'). If empty, the search should not be saved.
      *   - subfolder: (boolean) If set, search mailbox will default to
      *                subfolder search.
+     * </pre>
      */
     protected function _init()
     {
@@ -156,7 +158,7 @@ class IMP_Basic_Search extends IMP_Basic_Base
         }
 
         /* Check that searching is allowed. */
-        if (!$injector->getInstance('IMP_Imap')->access(IMP_Imap::ACCESS_SEARCH)) {
+        if (!$injector->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_SEARCH)) {
             $default_mailbox->url('mailbox')->redirect();
         }
 
@@ -166,18 +168,12 @@ class IMP_Basic_Search extends IMP_Basic_Base
         $dynamic_view = ($registry->getView() == Horde_Registry::VIEW_DYNAMIC);
         $js_vars = array();
 
-        $flist = $imp_flags->getList(array(
-            'imap' => true,
-            'mailbox' => $default_mailbox
-        ));
-
         /* Generate the search query if 'criteria_form' is present in the form
          * data. */
         if ($this->vars->criteria_form) {
-            $c_data = Horde_Serialize::unserialize($this->vars->criteria_form, Horde_Serialize::JSON);
             $c_list = array();
 
-            foreach ($c_data as $val) {
+            foreach (json_decode($this->vars->criteria_form) as $val) {
                 switch ($val->t) {
                 case 'from':
                 case 'to':
@@ -300,7 +296,7 @@ class IMP_Basic_Search extends IMP_Basic_Base
                 break;
 
             case 'vfolder':
-                $form = Horde_Serialize::unserialize($this->vars->mboxes_form, Horde_Serialize::JSON);
+                $form = json_decode($this->vars->mboxes_form);
                 $q_ob = $imp_search->createQuery($c_list, array(
                     'id' => IMP_Mailbox::formFrom($this->vars->edit_query_vfolder),
                     'label' => $this->vars->search_label,
@@ -319,7 +315,7 @@ class IMP_Basic_Search extends IMP_Basic_Base
                 break;
 
             default:
-                $form = Horde_Serialize::unserialize($this->vars->mboxes_form, Horde_Serialize::JSON);
+                $form = json_decode($this->vars->mboxes_form);
                 $q_ob = $imp_search->createQuery($c_list, array(
                     'mboxes' => IMP_Mailbox::formFrom($form->mbox),
                     'subfolders' => IMP_Mailbox::formFrom($form->subfolder)
@@ -385,9 +381,12 @@ class IMP_Basic_Search extends IMP_Basic_Base
             }
         } else {
             /* Process list of recent searches. */
+            $iterator = IMP_Search_IteratorFilter::create(
+                IMP_Search_IteratorFilter::QUERY
+            );
             $rs = array();
-            $imp_search->setIteratorFilter(IMP_Search::LIST_QUERY);
-            foreach ($imp_search as $val) {
+
+            foreach ($iterator as $val) {
                 $rs[$val->formid] = array(
                     'c' => $val->criteria,
                     'f' => array(
@@ -440,7 +439,12 @@ class IMP_Basic_Search extends IMP_Basic_Base
         $view->filterlist = $f_list;
 
         /* Create the flag list. */
+        $flist = $imp_flags->getList(array(
+            'imap' => true,
+            'mailbox' => $default_mailbox
+        ));
         $flag_set = array();
+
         foreach ($flist as $val) {
             $flag_set[] = array(
                 'l' => $val->label,
@@ -456,12 +460,8 @@ class IMP_Basic_Search extends IMP_Basic_Base
             $js_vars['ImpSearch.allsearch'] = IMP_Mailbox::formTo(IMP_Search_Query::ALLSEARCH);
             $ob = $injector->getInstance('IMP_Search_Ui')->getSearchMboxList();
             $mbox_list = $ob->mbox_list;
+            $view->subscribe = $prefs->getValue('subscribe');
             $view->tree = $ob->tree->getTree();
-
-            if ($prefs->getValue('subscribe')) {
-                $view->subscribe = true;
-                $js_vars['ImpSearch.ajaxurl'] = $registry->getServiceLink('ajax', 'imp')->url;
-            }
         }
 
         /* Prepare the topbar. */
