@@ -1,13 +1,13 @@
 <?php
 /**
- * Copyright 2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2013-2014 Horde LLC (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.horde.org/licenses/lgpl21.
+ * See the enclosed file COPYING for license information (GPL). If you
+ * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2013 Horde LLC
- * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
+ * @copyright 2013-2014 Horde LLC
+ * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
 
@@ -16,8 +16,8 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2013 Horde LLC
- * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
+ * @copyright 2013-2014 Horde LLC
+ * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
 class IMP_Compose_LinkedAttachment
@@ -56,7 +56,7 @@ class IMP_Compose_LinkedAttachment
     {
         global $conf, $injector;
 
-        /* Sanity checking. */
+        /* Sanity checking - no gettext needed. */
         if (empty($conf['compose']['link_attachments'])) {
             throw new IMP_Exception('Linked attachments are forbidden.');
         }
@@ -80,17 +80,14 @@ class IMP_Compose_LinkedAttachment
         }
 
         $data = $this->_atc->read();
-        fseek($data, 0, SEEK_END);
-        $size = ftell($data);
-        rewind($data);
 
         $md = $this->_atc->getMetadata();
-        $browser->downloadHeaders($md->filename, $md->type, false, $size);
+        $browser->downloadHeaders($md->filename, $md->type, false, $data->length());
 
-        while (!feof($data)) {
-            echo fread($data, 8192);
+        while (!$data->eof()) {
+            echo $data->substring(0, 8192);
         }
-        fclose($data);
+        $data->close();
     }
 
     /**
@@ -135,7 +132,7 @@ class IMP_Compose_LinkedAttachment
         $vfs = $injector->getInstance('Horde_Core_Factory_Vfs')->create();
 
         /* Build reproducible ID value from old data. */
-        $id = hash('md5', $ts . '|' . $file);
+        $id = hash('sha1', $ts . '|' . $file);
 
         /* Create new attachment object. */
         $atc = $injector->getInstance('IMP_Factory_ComposeAtc')->create($this->_user, $id);
