@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2006-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2006-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2006-2013 Horde LLC
+ * @copyright 2006-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2006-2013 Horde LLC
+ * @copyright 2006-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -123,13 +123,17 @@ class IMP_Compose_Ui
      */
     public function addIdentityJs()
     {
+        global $injector, $page_output;
+
         $identities = array();
-        $identity = $GLOBALS['injector']->getInstance('IMP_Identity');
+        $identity = $injector->getInstance('IMP_Identity');
+
+        $sigs = $identity->hasSignature(true);
 
         foreach (array_keys($identity->getAll('id')) as $ident) {
             $sm = $identity->getValue(IMP_Mailbox::MBOX_SENT, $ident);
 
-            $identities[] = array(
+            $entry = array(
                 // Sent mail mailbox name
                 'sm_name' => $sm ? $sm->form_to : '',
                 // Save in sent mail mailbox by default?
@@ -139,9 +143,27 @@ class IMP_Compose_Ui
                 // Bcc addresses to add
                 'bcc' => strval($identity->getBccAddresses($ident))
             );
+
+            if ($sigs) {
+                $sig = $identity->getSignature('text', $ident);
+                $html_sig = $identity->getSignature('html', $ident);
+                if (!strlen($html_sig) && strlen($sig)) {
+                    $html_sig = IMP_Compose::text2html($sig);
+                }
+                $sig_dom = new Horde_Domhtml($html_sig, 'UTF-8');
+                $html_sig = '';
+                foreach ($sig_dom->getBody()->childNodes as $child) {
+                    $html_sig .= $sig_dom->dom->saveXml($child);
+                }
+
+                $entry['sig'] = trim($sig);
+                $entry['hsig'] = $html_sig;
+            }
+
+            $identities[] = $entry;
         }
 
-        $GLOBALS['page_output']->addInlineJsVars(array(
+        $page_output->addInlineJsVars(array(
             'ImpComposeBase.identities' => $identities
         ));
     }
