@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,7 +17,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -51,21 +51,6 @@ class IMP_Contents
     const HEADER_TEXT = 2;
     const HEADER_STREAM = 3;
 
-    /**
-     * Flag to indicate whether the last call to getBodypart() returned
-     * decoded data.
-     *
-     * @var string
-     */
-    public $lastBodyPartDecode = null;
-
-    /**
-     * Close session when fetching data from IMAP server?
-     *
-     * @var boolean
-     */
-    public $fetchCloseSession = false;
-
     /**
      * Have we scanned for embedded parts?
      *
@@ -101,6 +86,13 @@ class IMP_Contents
      */
     protected $_message;
 
+    /**
+     * Cached data for the MIME Viewer objects.
+     *
+     * @var object
+     */
+    protected $_viewcache;
+
     /**
      * Constructor.
      *
@@ -129,6 +121,16 @@ class IMP_Contents
         }
     }
 
+    /**
+     * String representation of object.
+     *
+     * @return string  The indices string.
+     */
+    public function __toString()
+    {
+        return strval($this->getIndicesOb());
+    }
+
     /**
      * Returns the IMAP UID for the current message.
      *
@@ -196,8 +198,7 @@ class IMP_Contents
      * @param integer $id     The ID of the MIME part.
      * @param array $options  Additional options:
      *   - decode: (boolean) Attempt to decode the bodypart on the remote
-     *             server. If successful, sets self::$lastBodyPartDecode to
-     *             the content-type of the decoded data.
+     *             server.
      *             DEFAULT: No
      *   - length: (integer) If set, only download this many bytes of the
      *             bodypart from the server.
@@ -207,15 +208,22 @@ class IMP_Contents
      *   - stream: (boolean) If true, return a stream.
      *             DEFAULT: No
      *
-     * @return mixed  The text of the part or a stream resource if 'stream'
-     *                is true.
+     * @return object  Object with the following properties:
+     * <pre>
+     *   - data: (mixed) The text of the part or a stream resource if 'stream'
+     *           option is true.
+     *   - decode: (string) If 'decode' option is true, and bodypart decoded
+     *             on server, the content-type of the decoded data.
+     * </pre>
      */
     public function getBodyPart($id, $options = array())
     {
-        $this->lastBodyPartDecode = null;
+        $ret = new stdClass;
+        $ret->data = '';
+        $ret->decode = null;
 
         if (empty($id)) {
-            return '';
+            return $ret;
         }
 
         if (!$this->_indices || $this->isEmbedded($id)) {
@@ -224,21 +232,23 @@ class IMP_Contents
                 $ob = $this->getMIMEPart($id, array('nocontents' => true));
 
                 if (empty($options['stream'])) {
-                    return is_null($ob)
-                        ? ''
-                        : $ob->getContents();
+                    if (!is_null($ob)) {
+                        $ret->data = $ob->getContents();
+                    }
+                } else {
+                    $ret->data = is_null($ob)
+                        ? fopen('php://temp', 'r+')
+                        : $ob->getContents(array('stream' => true));
                 }
 
-                return is_null($ob)
-                    ? fopen('php://temp', 'r+')
-                    : $ob->getContents(array('stream' => true));
+                return $ret;
             }
 
             $base_id = $id;
             while (!in_array($base_id, $this->_embedded, true)) {
                 $base_id = Horde_Mime::mimeIdArithmetic($base_id, 'up');
                 if (is_null($base_id)) {
-                    return '';
+                    return $ret;
                 }
             }
 
@@ -256,18 +266,26 @@ class IMP_Contents
             }
 
             if (empty($options['stream'])) {
-                return $body;
+                $ret->data = $body;
+                return $ret;
             }
 
-            $fp = fopen('php://temp', 'r+');
+            $ret->data = fopen('php://temp', 'r+');
             if (strlen($body)) {
-                fwrite($fp, $body);
-                fseek($fp, 0);
+                fwrite($ret->data, $body);
+                fseek($ret->data, 0);
             }
-            return $fp;
+            return $ret;
         }
 
         $query = new Horde_Imap_Client_Fetch_Query();
+        if (substr($id, -2) === '.0') {
+            $rfc822 = true;
+            $id = substr($id, 0, -2);
+        } else {
+            $rfc822 = false;
+        }
+
         if (!isset($options['length']) || !empty($options['length'])) {
             $bodypart_params = array(
                 'decode' => !empty($options['decode']),
@@ -279,32 +297,64 @@ class IMP_Contents
                 $bodypart_params['length'] = $options['length'];
             }
 
-            $query->bodyPart($id, $bodypart_params);
+            if ($rfc822) {
+                $bodypart_params['id'] = $id;
+                $query->bodyText($bodypart_params);
+            } else {
+                $query->bodyPart($id, $bodypart_params);
+            }
         }
 
         if (!empty($options['mimeheaders'])) {
-            $query->mimeHeader($id, array(
-                'peek' => true
-            ));
+            if ($rfc822) {
+                $query->headerText(array(
+                    'id' => $id,
+                    'peek' => true
+                ));
+            } else {
+                $query->mimeHeader($id, array(
+                    'peek' => true
+                ));
+            }
         }
 
         if ($res = $this->_fetchData($query)) {
             try {
                 if (empty($options['mimeheaders'])) {
-                    $this->lastBodyPartDecode = $res->getBodyPartDecode($id);
-                    return $res->getBodyPart($id, !empty($options['stream']));
+                    $ret->decode = $res->getBodyPartDecode($id);
+                    $ret->data = $rfc822
+                        ? $res->getBodyText($id, !empty($options['stream']))
+                        : $res->getBodyPart($id, !empty($options['stream']));
+                    return $ret;
                 } elseif (empty($options['stream'])) {
-                    return $res->getMimeHeader($id) . $res->getBodyPart($id);
+                    $ret->data = $rfc822
+                        ? ($res->getHeaderText($id) . $res->getBodyText($id))
+                        : ($res->getMimeHeader($id) . $res->getBodyPart($id));
+                    return $ret;
                 }
 
-                $swrapper = new Horde_Support_CombineStream(array($res->getMimeHeader($id, Horde_Imap_Client_Data_Fetch::HEADER_STREAM), $res->getBodyPart($id, true)));
-                return $swrapper->fopen();
+                if ($rfc822) {
+                    $data = array(
+                        $res->getHeaderText($id, Horde_Imap_Client_Data_Fetch::HEADER_STREAM),
+                        $res->getBodyText($id, true)
+                    );
+                } else {
+                    $data = array(
+                        $res->getMimeHeader($id, Horde_Imap_Client_Data_Fetch::HEADER_STREAM),
+                        $res->getBodyPart($id, true)
+                    );
+                }
+
+                $ret->data = Horde_Stream_Wrapper_Combine::getStream($data);
+                return $ret;
             } catch (Horde_Exception $e) {}
         }
 
-        return empty($options['stream'])
-            ? ''
-            : fopen('php://temp', 'r+');
+        if (!empty($options['stream'])) {
+            $ret->data = fopen('php://temp', 'r+');
+        }
+
+        return $ret;
     }
 
     /**
@@ -334,8 +384,10 @@ class IMP_Contents
                     return $this->getHeader(self::HEADER_TEXT) . $res->getBodyText(0);
                 }
 
-                $swrapper = new Horde_Support_CombineStream(array($this->getHeader(self::HEADER_STREAM), $res->getBodyText(0, true)));
-                return $swrapper->fopen();
+                return Horde_Stream_Wrapper_Combine::getStream(array(
+                    $this->getHeader(self::HEADER_STREAM),
+                    $res->getBodyText(0, true)
+                ));
             } catch (Horde_Exception $e) {}
         }
 
@@ -366,15 +418,17 @@ class IMP_Contents
      */
     public function getHeaderAndMarkAsSeen($type = self::HEADER_OB)
     {
-        if ($this->getMailbox()->readonly) {
+        $mbox = $this->getMailbox();
+
+        if ($mbox->readonly) {
             $seen = false;
         } else {
             $seen = true;
 
             if (isset($this->_header)) {
                 try {
-                    $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
-                    $imp_imap->store($this->getMailbox(), array(
+                    $imp_imap = $mbox->imp_imap;
+                    $imp_imap->store($mbox, array(
                         'add' => array(
                             Horde_Imap_Client::FLAG_SEEN
                         ),
@@ -486,8 +540,8 @@ class IMP_Contents
                 'length' => empty($options['length']) ? null : $options['length'],
                 'stream' => true
             ));
-            $part->setContents($body, array(
-                'encoding' => $this->lastBodyPartDecode,
+            $part->setContents($body->data, array(
+                'encoding' => $body->decode,
                 'usestream' => true
             ));
         }
@@ -523,6 +577,9 @@ class IMP_Contents
         $mime_part = empty($options['mime_part'])
             ? $this->getMIMEPart($mime_id)
             : $options['mime_part'];
+        if (!$mime_part) {
+            return array($mime_id => null);
+        }
 
         if (!empty($options['autodetect']) &&
             ($tempfile = Horde::getTempFile()) &&
@@ -541,7 +598,7 @@ class IMP_Contents
             ? null
             : $options['type'];
 
-        $viewer = $GLOBALS['injector']->getInstance('IMP_Factory_MimeViewer')->create($mime_part, $this, $type);
+        $viewer = $GLOBALS['injector']->getInstance('IMP_Factory_MimeViewer')->create($mime_part, array('contents' => $this, 'type' => $type));
 
         switch ($mode) {
         case self::RENDER_INLINE:
@@ -768,7 +825,7 @@ class IMP_Contents
         if (($mask & self::SUMMARY_BYTES) ||
             $download_zip ||
             ($mask & self::SUMMARY_SIZE)) {
-            $part['bytes'] = $size = $mime_part->getBytes(true);
+            $part['bytes'] = $size = $mime_part->getBytes();
             $part['size'] = ($size > 1048576)
                 ? sprintf(_("%s MB"), IMP::numberFormat($size / 1048576, 1))
                 : sprintf(_("%s KB"), max(round($size / 1024), 1));
@@ -777,9 +834,13 @@ class IMP_Contents
         /* Get part's icon. */
         if (($mask & self::SUMMARY_ICON) ||
             ($mask & self::SUMMARY_ICON_RAW)) {
-            $part['icon'] = $GLOBALS['injector']->getInstance('Horde_Core_Factory_MimeViewer')->getIcon($mime_type);
+            $part['icon'] = $GLOBALS['injector']->getInstance('IMP_Factory_MimeViewer')->getIcon($mime_type);
             if ($mask & self::SUMMARY_ICON) {
-                $part['icon'] = Horde::img($part['icon'], '', array('title' => $mime_type), '');
+                $part['icon'] = Horde_Themes_Image::tag($part['icon'], array(
+                    'attr' => array(
+                        'title' => $mime_type
+                    )
+                ));
             }
         } else {
             $part['icon'] = null;
@@ -815,7 +876,7 @@ class IMP_Contents
         if ($is_atc &&
             $download_zip &&
             ($part['bytes'] > 204800)) {
-            $viewer = $GLOBALS['injector']->getInstance('IMP_Factory_MimeViewer')->create($mime_part, $this, $mime_type);
+            $viewer = $GLOBALS['injector']->getInstance('IMP_Factory_MimeViewer')->create($mime_part, array('contents' => $this, 'type' => $mime_type));
             if (!$viewer->getMetadata('compressed')) {
                 $part['download_zip'] = $this->linkView($mime_part, 'download_attach', null, array('class' => 'iconImg downloadZipAtc', 'jstext' => sprintf(_("Download %s in .zip Format"), $description), 'params' => array('zip' => 1)));
             }
@@ -847,9 +908,9 @@ class IMP_Contents
             $part['strip'] = Horde::link(
                 Horde::selfUrlParams()->add(array(
                     'actionID' => 'strip_attachment',
-                    'message_token' => $GLOBALS['injector']->getInstance('Horde_Token')->get('imp.impcontents'),
+                    'imapid' => $id,
                     'muid' => strval($this->getIndicesOb()),
-                    'imapid' => $id
+                    'token' => $GLOBALS['session']->getToken()
                 )),
                 _("Strip Attachment"),
                 'iconImg deleteImg stripAtc',
@@ -931,7 +992,12 @@ class IMP_Contents
             'params' => array()
         ), $options);
 
-        return Horde::link($this->urlView($mime_part, $actionID, $options), $options['jstext'], $options['class'], strval(new Horde_Support_Randomid())) . $text . '</a>';
+        return Horde::link(
+            $this->urlView($mime_part, $actionID, $options),
+            $options['jstext'],
+            $options['class'],
+            ($actionID == 'download_attach') ? null : strval(new Horde_Support_Randomid())
+        ) . $text . '</a>';
     }
 
     /**
@@ -965,7 +1031,7 @@ class IMP_Contents
 
         $url = Horde::popupJs(Horde::url('view.php'), array(
             'menu' => true,
-            'onload' => empty($options['onload']) ? '' : $options['onload'],
+            'onload' => empty($options['onload']) ? 'IMP_JS.resizePopup' : $options['onload'],
             'params' => $this->_urlViewParams($mime_part, $actionID, isset($options['params']) ? $options['params'] : array()),
             'urlencode' => true
         ));
@@ -1014,6 +1080,8 @@ class IMP_Contents
      */
     protected function _buildMessage($parts = null)
     {
+        global $injector;
+
         if (is_null($parts)) {
             if ($this->_build) {
                 return;
@@ -1028,6 +1096,8 @@ class IMP_Contents
         $last_id = null;
         $to_process = array();
 
+        $mv_factory = $injector->getInstance('IMP_Factory_MimeViewer');
+
         foreach ($parts as $id) {
             if (!is_null($last_id) &&
                 (strpos($id, $last_id) === 0)) {
@@ -1037,7 +1107,7 @@ class IMP_Contents
             $last_id = null;
 
             $mime_part = $this->getMIMEPart($id, array('nocontents' => true));
-            $viewer = $GLOBALS['injector']->getInstance('IMP_Factory_MimeViewer')->create($mime_part, $this);
+            $viewer = $mv_factory->create($mime_part, array('contents' => $this));
             if ($viewer->embeddedMimeParts()) {
                 $mime_part = $this->getMIMEPart($id);
                 $viewer->setMIMEPart($mime_part);
@@ -1072,7 +1142,7 @@ class IMP_Contents
         if (!is_object($part)) {
             $part = $this->getMIMEPart($part, array('nocontents' => true));
         }
-        $viewer = $GLOBALS['injector']->getInstance('IMP_Factory_MimeViewer')->create($part, $this, $type);
+        $viewer = $GLOBALS['injector']->getInstance('IMP_Factory_MimeViewer')->create($part, array('contents' => $this, 'type' => $type));
 
         if ($mask & self::RENDER_INLINE_AUTO) {
             $mask |= self::RENDER_INLINE | self::RENDER_INFO;
@@ -1540,24 +1610,29 @@ class IMP_Contents
      */
     protected function _fetchData(Horde_Imap_Client_Fetch_Query $query)
     {
-        if ($this->fetchCloseSession) {
-            $GLOBALS['session']->close();
-        }
-
         try {
-            $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
-            $res = $imp_imap->fetch($this->getMailbox(), $query, array(
+            $mbox = $this->getMailbox();
+            $imp_imap = $mbox->imp_imap;
+            return $imp_imap->fetch($mbox, $query, array(
                 'ids' => $imp_imap->getIdsOb($this->getUid())
             ))->first();
         } catch (Horde_Imap_Client_Exception $e) {
-            $res = new Horde_Imap_Client_Data_Fetch();
+            return new Horde_Imap_Client_Data_Fetch();
+        }
+    }
+
+    /**
+     * Return the view cache object for this message.
+     *
+     * @return object  View object.
+     */
+    public function getViewCache()
+    {
+        if (!isset($this->_viewcache)) {
+            $this->_viewcache = new stdClass;
         }
 
-        if ($this->fetchCloseSession) {
-            $GLOBALS['session']->start();
-        }
-
-        return $res;
+        return $this->_viewcache;
     }
 
 }
