@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -45,6 +45,8 @@ class IMP_Contents_View
      */
     public function downloadAll()
     {
+        global $page_output, $session;
+
         $headers = $this->_contents->getHeader();
         $zipfile = trim(preg_replace('/[^\pL\pN-+_. ]/u', '_', $headers->getValue('subject')), ' _');
         if (empty($zipfile)) {
@@ -53,14 +55,13 @@ class IMP_Contents_View
             $zipfile .= '.zip';
         }
 
-        $GLOBALS['page_output']->disableCompression();
-        $this->_contents->fetchCloseSession = true;
+        $page_output->disableCompression();
+        $session->close();
 
         $tosave = array();
         foreach ($this->_contents->downloadAllList() as $val) {
-            $mime = $this->_contents->getMIMEPart($val);
-            $name = $mime->getName(true);
-            if (!$name) {
+            $mime = $this->_getRawDownloadPart($val);
+            if (!($name = $mime->getName(true))) {
                 $name = sprintf(_("part %s"), $val);
             }
             $tosave[] = array(
@@ -86,16 +87,11 @@ class IMP_Contents_View
      */
     public function downloadAttach($id, $zip = false)
     {
-        $mime = $this->_contents->getMIMEPart($id);
-        if ($this->_contents->canDisplay($id, IMP_Contents::RENDER_RAW)) {
-            $this->_contents->fetchCloseSession = true;
-            $render = $this->_contents->renderMIMEPart($id, IMP_Contents::RENDER_RAW);
-            $part = reset($render);
-            $mime->setContents($part['data'], array(
-                'encoding' => 'binary'
-            ));
-        }
+        global $page_output, $session;
 
+        $session->close();
+
+        $mime = $this->_getRawDownloadPart($id);
         $name = $this->_contents->getPartName($mime);
 
         /* Compress output? */
@@ -111,7 +107,7 @@ class IMP_Contents_View
             $name .= '.zip';
             $type = 'application/zip';
 
-            $GLOBALS['page_output']->disableCompression();
+            $page_output->disableCompression();
         } else {
             $data = $mime->getContents(array('stream' => true));
             $type = $mime->getType(true);
@@ -128,7 +124,9 @@ class IMP_Contents_View
      */
     public function downloadRender($id, $mode, $ctype = null)
     {
-        $this->_contents->fetchCloseSession = true;
+        global $session;
+
+        $session->close();
 
         $render = $this->_contents->renderMIMEPart(
             $id,
@@ -145,7 +143,9 @@ class IMP_Contents_View
      */
     public function viewAttach($id, $mode, $autodetect = false, $ctype = null)
     {
-        $this->_contents->fetchCloseSession = true;
+        global $session;
+
+        $session->close();
 
         $render = $this->_contents->renderMIMEPart(
             $id,
@@ -171,14 +171,18 @@ class IMP_Contents_View
      */
     public function viewSource()
     {
-        $this->_contents->fetchCloseSession = true;
+        global $session;
+
+        $session->close();
+
+        $charset = $this->_contents->getMIMEMessage()->getCharset();
 
         return array(
             'data' => $this->_contents->fullMessageText(array(
                 'stream' => true
             )),
             'name' => _("Message Source"),
-            'type' => 'text/plain'
+            'type' => 'text/plain; charset=' . (is_null($charset) ? 'UTF-8' : $charset)
         );
     }
 
@@ -186,12 +190,14 @@ class IMP_Contents_View
      */
     public function saveMessage()
     {
+        global $session;
+
+        $session->close();
+
         $name = ($subject = $this->_contents->getHeader()->getValue('subject'))
             ? trim(preg_replace('/[^\pL\pN-+_. ]/u', '_', $subject), ' _')
             : 'saved_message';
 
-        $this->_contents->fetchCloseSession = true;
-
         return array(
             'data' => $this->_contents->fullMessageText(array(
                 'stream' => true
@@ -214,8 +220,8 @@ class IMP_Contents_View
 
         $part = reset($render);
 
-        /* Directly render part if this is not an HTML part. */
-        if (stripos($part['type'], 'text/html') !== 0) {
+        /* Directly render part if this is not an HTML part or it is empty. */
+        if (stripos($part['type'], 'text/html') !== 0 || !strlen($part['data'])) {
             return $part;
         }
 
@@ -231,6 +237,7 @@ class IMP_Contents_View
             if ($hdr_val = $headerob->getValue($key)) {
                 /* Format date string. */
                 if ($key == 'date') {
+                    $registry->setTimeZone();
                     $hdr_val = $imp_ui_mbox->getDate($hdr_val, $imp_ui_mbox::DATE_FORCE | $imp_ui_mbox::DATE_FULL);
                 }
 
@@ -268,24 +275,45 @@ class IMP_Contents_View
                 }
             }
 
-            $css = $page_output->css;
-            $style = '';
+            /* Cache CSS. */
+            $cache_list = array();
+            $cache_ob = $injector->getInstance('Horde_Cache');
 
-            try {
-                $css_parser = new Horde_Css_Parser($css->loadCssFiles($css->getStylesheets()));
+            $css_list = $page_output->css->getStylesheets();
+            foreach ($css_list as $val) {
+                $cache_list[] = $val['fs'];
+                $cache_list[] = filemtime($val['fs']);
+            }
+            $cache_id = 'imp_printcss_' . hash(
+                (PHP_MINOR_VERSION >= 4) ? 'fnv132' : 'sha1',
+                implode('|', $cache_list)
+            );
 
-                foreach ($css_parser->doc->getContents() as $val) {
-                    if (($val instanceof Sabberworm\CSS\RuleSet\DeclarationBlock) &&
-                        array_intersect($selectors, array_map('strval', $val->getSelectors()))) {
-                        $style .= implode('', array_map('strval', $val->getRules()));
+            if (($style = $cache_ob->get($cache_id, 0)) === false) {
+                try {
+                    $css_parser = new Horde_Css_Parser(
+                        $page_output->css->loadCssFiles(
+                            $page_output->css->getStylesheets()
+                        )
+                    );
+
+                    $style = '';
+
+                    foreach ($css_parser->doc->getContents() as $val) {
+                        if (($val instanceof Sabberworm\CSS\RuleSet\DeclarationBlock) &&
+                            array_intersect($selectors, array_map('strval', $val->getSelectors()))) {
+                            $style .= implode('', array_map('strval', $val->getRules()));
+                        }
                     }
-                }
 
-                if (strlen($style)) {
-                    $elt->setAttribute('style', ($elt->hasAttribute('style') ? rtrim($elt->getAttribute('style'), ' ;') . ';' : '') . $style);
+                    $cache_ob->set($cache_id, $style, 86400);
+                } catch (Exception $e) {
+                    // Ignore CSS if it can't be parsed.
                 }
-            } catch (Exception $e) {
-                // Ignore CSS if it can't be parsed.
+            }
+
+            if (strlen($style)) {
+                $elt->setAttribute('style', ($elt->hasAttribute('style') ? rtrim($elt->getAttribute('style'), ' ;') . ';' : '') . $style);
             }
         }
 
@@ -366,4 +394,28 @@ class IMP_Contents_View
         return $params;
     }
 
+    /* Internal methods. */
+
+    /**
+     * Get a MIME Part for use in creating download.
+     *
+     * @param string $id  MIME ID.
+     *
+     * @return Horde_Mime_Part  MIME part.
+     */
+    protected function _getRawDownloadPart($id)
+    {
+        $mime = $this->_contents->getMIMEPart($id);
+
+        if ($this->_contents->canDisplay($id, IMP_Contents::RENDER_RAW)) {
+            $render = $this->_contents->renderMIMEPart($id, IMP_Contents::RENDER_RAW);
+            $part = reset($render);
+            $mime->setContents($part['data'], array(
+                'encoding' => 'binary'
+            ));
+        }
+
+        return $mime;
+    }
+
 }
