@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -126,7 +126,7 @@ abstract class IMP_Dynamic_Base
      */
     protected function _addBaseVars()
     {
-        global $prefs, $session;
+        global $prefs, $registry, $session;
 
         /* Variables used in core javascript files. */
         $this->js_conf = array_filter(array(
@@ -136,7 +136,11 @@ abstract class IMP_Dynamic_Base
             'URI_VIEW' => strval(Horde::url('view.php')->add(IMP_Contents_View::addToken())),
 
             // Other variables
-            'disable_compose' => !IMP_Compose::canCompose()
+            'disable_compose' => !IMP_Compose::canCompose(),
+            'pref_prefix' => hash(
+                (PHP_MINOR_VERSION >= 4) ? 'fnv132' : 'sha1',
+                $registry->getAuth() . '|' . $_SERVER['SERVER_NAME']
+            )
         ));
 
         /* Context menu definitions.
@@ -148,8 +152,11 @@ abstract class IMP_Dynamic_Base
          */
         $context = array(
             'ctx_contacts' => array(
+                // Empty sub item needs to be a javascript object
+                '_sub1' => new stdClass,
                 'new' => _("New Message"),
-                'add' => _("Add to Address Book")
+                'add' => _("Add to Address Book"),
+                'copy' => _("Copy to Clipboard")
             ),
             'ctx_reply' => array(
                 'reply' => _("To Sender"),
@@ -158,6 +165,10 @@ abstract class IMP_Dynamic_Base
             )
         );
 
+        if ($registry->hasLink('mail/newEmailFilter')) {
+            $context['ctx_contacts']['addfilter'] = _("Create Filter");
+        }
+
         /* Forward context menu. */
         $context['ctx_forward'] = array(
             'attach' => _("As Attachment"),
@@ -182,6 +193,7 @@ abstract class IMP_Dynamic_Base
         /* Gettext strings used in core javascript files. */
         $this->js_text = array(
             'allparts_label' => _("Parts"),
+            'emailcopy' => _("Your browser security settings don't permit direct access to the clipboard.") . "\n" . _("You need to either use the keyboard (Ctrl/Cmd + C) or right click on the selected address to access the Copy command."),
             'loading' => _("Loading..."),
             'strip_warn' => _("Are you sure you wish to PERMANENTLY delete this attachment?"),
             'verify' => _("Verifying...")
