@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2005-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2005-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2005-2013 Horde LLC
+ * @copyright 2005-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2005-2013 Horde LLC
+ * @copyright 2005-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -27,7 +27,9 @@ class IMP_Dynamic_Compose_Common
      *
      * @param IMP_Dynamic_Base $base  Base dynamic view object.
      * @param array $args             Configuration parameters:
-     *   - redirect: (string) Display the redirect interface?
+     *   - redirect: (boolean) Display the redirect interface? If true,
+     *               display only redirect. If false, don't add redirect
+     *               interface to page.
      *   - resume: (boolean) Are we resuming a saved draft?
      *   - show_editor: (boolean) Show the HTML editor?
      *   - template: (string) Display the edit template interface?
@@ -38,15 +40,16 @@ class IMP_Dynamic_Compose_Common
     {
         global $injector, $page_output, $prefs;
 
+        $page_output->addScriptPackage('Horde_Core_Script_Package_Keynavlist');
         $page_output->addScriptPackage('IMP_Script_Package_ComposeBase');
         $page_output->addScriptFile('compose-dimp.js');
         $page_output->addScriptFile('draghandler.js');
-        $page_output->addScriptFile('murmurhash3.js');
-        $page_output->addScriptFile('textarearesize.js', 'horde');
+        $page_output->addScriptFile('editor.js');
+        $page_output->addScriptFile('imp.js');
 
         if (!$prefs->isLocked('default_encrypt') &&
             ($prefs->getValue('use_pgp') || $prefs->getValue('use_smime'))) {
-            $page_output->addScriptPackage('Dialog');
+            $page_output->addScriptPackage('Horde_Core_Script_Package_Dialog');
             $page_output->addScriptFile('passphrase.js');
         }
 
@@ -67,18 +70,23 @@ class IMP_Dynamic_Compose_Common
             return $view->render('redirect');
         }
 
-        $imp_ui->attachAutoCompleter(array('to', 'cc', 'bcc', 'redirect_to'));
+        $ac = array('to', 'cc', 'bcc');
+        if (!isset($args['redirect'])) {
+            $ac[] = 'redirect_to';
+        }
+
+        $imp_ui->attachAutoCompleter($ac);
         $view->spellcheck = $imp_ui->attachSpellChecker();
 
         $this->_compose($base, $view, $args);
-        return $view->render('compose') . $view->render('redirect');
+        return $view->render('compose') . (isset($args['redirect']) ? '' : $view->render('redirect'));
     }
 
     /**
      */
     protected function _compose($base, $view, $args)
     {
-        global $injector, $registry, $prefs, $session;
+        global $injector, $registry, $page_output, $prefs, $session;
 
         $view->title = $args['title'];
 
@@ -93,45 +101,16 @@ class IMP_Dynamic_Compose_Common
             $view->compose_html = !empty($args['show_editor']);
             $view->rte = true;
 
-            $injector->getInstance('IMP_Editor')->init();
+            $page_output->addScriptPackage('IMP_Script_Package_Editor');
+            $page_output->addScriptFile('external/base64.js');
         }
 
         /* Create list for sent-mail selection. */
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $injector->getInstance('IMP_Factory_Imap')->create();
         if ($imp_imap->access(IMP_Imap::ACCESS_FOLDERS) &&
             !$prefs->isLocked('save_sent_mail')) {
             $view->save_sent_mail = true;
-
-            if (!$prefs->isLocked(IMP_Mailbox::MBOX_SENT)) {
-                $view->save_sent_mail_select = true;
-
-                /* Check to make sure the sent-mail mailboxes are created;
-                 * they need to exist to show up in drop-down list. */
-                foreach (array_keys($identity->getAll('id')) as $ident) {
-                    $mbox = $identity->getValue(IMP_Mailbox::MBOX_SENT, $ident);
-                    if ($mbox instanceof IMP_Mailbox) {
-                        $mbox->create();
-                    }
-                }
-
-                $flist = array();
-                $imaptree = $injector->getInstance('IMP_Imap_Tree');
-                $imaptree->setIteratorFilter();
-
-                foreach ($imaptree as $val) {
-                    $tmp = array(
-                        'f' => $val->display,
-                        'l' => Horde_String::abbreviate(str_repeat(' ', 2 * $val->level) . $val->basename, 30),
-                        'v' => $val->container ? '' : $val->form_to
-                    );
-                    if ($tmp['f'] == $tmp['v']) {
-                        unset($tmp['f']);
-                    }
-                    $flist[] = $tmp;
-                }
-
-                $base->js_conf['flist'] = $flist;
-            }
+            $view->save_sent_mail_select = !$prefs->isLocked(IMP_Mailbox::MBOX_SENT);
         }
 
         $view->drafts = ($imp_imap->access(IMP_Imap::ACCESS_DRAFTS) &&
@@ -167,15 +146,18 @@ class IMP_Dynamic_Compose_Common
             $view->encrypt = $prefs->getValue('default_encrypt');
         }
 
-        $select_list = array();
+        $from_list = array();
         foreach ($identity->getSelectList() as $id => $from) {
-            $select_list[] = array(
+            $from_list[] = array(
                 'label' => $from,
                 'sel' => ($id == $selected_identity),
                 'val' => $id
             );
         }
-        $view->select_list = $select_list;
+        $view->from_list = $from_list;
+
+        $view->signature = $identity->hasSignature(true);
+        $view->sigExpanded = $prefs->getValue('signature_expanded');
     }
 
     /**
@@ -219,15 +201,13 @@ class IMP_Dynamic_Compose_Common
         $templates_mbox = IMP_Mailbox::getPref(IMP_Mailbox::MBOX_TEMPLATES);
 
         $base->js_conf += array_filter(array(
-            'URI_MAILBOX' => strval(IMP_Dynamic_Mailbox::url()),
-
             'compose_cursor' => ($compose_cursor ? $compose_cursor : 'top'),
             'rte_avail' => intval($browser->hasFeature('rte')),
             'spellcheck' => intval($prefs->getValue('compose_spellcheck')),
             'templates_mbox' => $templates_mbox ? $templates_mbox->form_to : null
         ));
 
-        if ($injector->getInstance('IMP_Imap')->access(IMP_Imap::ACCESS_DRAFTS) &&
+        if ($injector->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_DRAFTS) &&
             ($drafts_mbox = IMP_Mailbox::getPref(IMP_Mailbox::MBOX_DRAFTS)) &&
             !$drafts_mbox->readonly) {
             $base->js_conf += array_filter(array(
@@ -238,7 +218,7 @@ class IMP_Dynamic_Compose_Common
         }
 
         if ($registry->hasMethod('contacts/search')) {
-            $base->js_conf['URI_ABOOK'] = strval(IMP_Basic_Contacts::url(array('full' => true)));
+            $base->js_conf['URI_ABOOK'] = strval(IMP_Basic_Contacts::url()->setRaw(true));
         }
 
         if ($prefs->getValue('set_priority')) {
@@ -275,8 +255,13 @@ class IMP_Dynamic_Compose_Common
 
         /* Gettext strings used in compose page. */
         $base->js_text += array(
+            'change_identity' => _("You have edited your signature. Change the identity and lose your changes?"),
             'compose_cancel' => _("Cancelling this message will permanently discard its contents and will delete auto-saved drafts.\nAre you sure you want to do this?"),
+            'compose_close' => _("Compose action completed. You may now safely close this window."),
+            'dragdropimg_error' => _("Could not add %d file(s) to message: only images are supported."),
+            'multiple_atc' => _("%d Attachments"),
             'nosubject' => _("The message does not have a subject entered.") . "\n" . _("Send message without a subject?"),
+            'paste_error' => _("Could not paste image as the clipboard data is invalid."),
             'replyall' => _("%d recipients"),
             'spell_noerror' => _("No spelling errors found."),
             'toggle_html' => _("Discard all text formatting information (by converting from HTML to plain text)? This conversion cannot be reversed."),
