@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -31,19 +31,24 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
         $page_output->addScriptFile('dimpbase.js');
         $page_output->addScriptFile('passphrase.js');
         $page_output->addScriptFile('viewport.js');
+        $page_output->addScriptFile('external/tinycon.js');
         $page_output->addScriptFile('dragdrop2.js', 'horde');
         $page_output->addScriptFile('form_ghost.js', 'horde');
         $page_output->addScriptFile('jstorage.js', 'horde');
         $page_output->addScriptFile('slider2.js', 'horde');
         $page_output->addScriptFile('toggle_quotes.js', 'horde');
-        $page_output->addScriptPackage('Dialog');
+        $page_output->addScriptPackage('Horde_Core_Script_Package_Dialog');
         $page_output->addScriptPackage('IMP_Script_Package_Imp');
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $injector->getInstance('IMP_Factory_Imap')->create();
 
         if ($imp_imap->access(IMP_Imap::ACCESS_FLAGS)) {
             $page_output->addScriptFile('colorpicker.js', 'horde');
-            $this->view->picker_img = Horde::img('colorpicker.png', _("Color Picker"));
+            $this->view->picker_img = Horde_Themes_Image::tag('colorpicker.png', array('alt' => _("Color Picker")));
+        }
+
+        if ($imp_imap->access(IMP_Imap::ACCESS_REMOTE)) {
+            $page_output->addScriptFile('external/base64.js');
         }
 
         $this->_addMailboxVars();
@@ -125,16 +130,17 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
         global $conf, $injector, $prefs, $registry;
 
         /* Does server support ACLs? */
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $injector->getInstance('IMP_Factory_Imap')->create();
         $acl = $imp_imap->access(IMP_Imap::ACCESS_ACL);
 
         $this->js_conf += array_filter(array(
             // URLs
+            'URI_LISTINFO' => strval(IMP_Basic_Listinfo::url()->setRaw(true)),
             'URI_MESSAGE' => strval(IMP_Dynamic_Message::url()->setRaw(true)),
             'URI_PORTAL' => strval($registry->getServiceLink('portal')->setRaw(true)),
             'URI_PREFS_IMP' => strval($registry->getServiceLink('prefs', 'imp')->setRaw(true)),
-            'URI_SEARCH' => strval(IMP_Basic_Search::url(array('full' => true))),
-            'URI_THREAD' => strval(IMP_Basic_Thread::url(array('full' => true))),
+            'URI_SEARCH' => strval(IMP_Basic_Search::url()->setRaw(true)),
+            'URI_THREAD' => strval(IMP_Basic_Thread::url()->setRaw(true)),
 
             // IMAP Flags
             'FLAG_DELETED' => Horde_Imap_Client::FLAG_DELETED,
@@ -154,8 +160,9 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
                 'mboxzip' => _("Download into a MBOX file (ZIP compressed)")
             ),
             'filter_any' => intval($prefs->getValue('filter_any_mailbox')),
+            'from_link' => intval($prefs->getValue('from_link') == 0),
             'fsearchid' => IMP_Mailbox::formTo(IMP_Search::MBOX_PREFIX . IMP_Search::DIMP_FILTERSEARCH),
-            'initial_page' => is_null($initial_page = IMP_Auth::getInitialPage()->mbox) ? null : $initial_page->form_to,
+            'initial_page' => is_null($initial_page = IMP::getInitialPage()->mbox) ? null : $initial_page->form_to,
             'mbox_expand' => intval($prefs->getValue('nav_expanded') == 2),
             'name' => $registry->get('name', 'imp'),
             'poll_alter' => intval(!$prefs->isLocked('nav_poll') && !$prefs->getValue('nav_poll_all')),
@@ -236,13 +243,20 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
                 '_sep1' => null,
                 'noaction' => _("No actions available")
             ),
+            'ctx_remoteauth' => array(
+                '_mbox' => '',
+                '_sep1' => null,
+                'create' => _("Create Mailbox"),
+                'logout' => _("Log Out")
+            ),
             'ctx_sortopts' => array(
                 'from' => _("From"),
                 'to' => _("To"),
                 'subject' => _("Subject"),
                 'thread' => _("Thread"),
-                'date' => _("Date"),
-                'size' => ("Size"),
+                'msgarrival' => _("Date (Arrival)"),
+                'msgdate' => _("Date (Message)"),
+                'size' => _("Size"),
                 '_sep1' => null,
                 'sequence' => _("No Sort")
             ),
@@ -307,6 +321,7 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
             'innocent' => _("Report as Innocent"),
             'blacklist' => _("Blacklist"),
             'whitelist' => _("Whitelist"),
+            'addfilter' => _("Create Filter"),
             'delete' => _("Delete"),
             'undelete' => _("Undelete"),
             '_sub3' => array(
@@ -321,6 +336,9 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
         if (empty($imp_imap->config->innocent_params)) {
             unset($context['ctx_message']['innocent']);
         }
+        if (!$registry->hasLink('mail/newEmailFilter')) {
+            unset($context['ctx_message']['addfilter']);
+        }
         if ($prefs->getValue('use_trash')) {
             unset($context['ctx_message']['undelete']);
         }
@@ -353,14 +371,43 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
             '_sep6' => null,
             'export' => _("Export"),
             'import' => _("Import"),
+            '_sep7' => null,
+            'size' => _("Mailbox Size"),
             '_sub2' => array(
-                '_sep7' => null,
+                '_sep8' => null,
                 'acl' => _("Edit ACL")
-            )
+            ),
         );
 
         if (!$subscribe) {
-            unset($context['ctx_mbox']['sub'], $context['ctx_mbox']['unsub']);
+            unset(
+                $context['ctx_mbox']['sub'],
+                $context['ctx_mbox']['unsub']
+            );
+        }
+        if (!$imp_imap->access(IMP_Imap::ACCESS_IMPORT)) {
+            unset($context['ctx_mbox']['import']);
+        }
+        if (!$imp_imap->access(IMP_Imap::ACCESS_FLAGS)) {
+            unset(
+                $context['ctx_mbox']['_sep2'],
+                $context['ctx_mbox']['setflag'],
+                $context['ctx_mbox']['_sep3'],
+                $context['ctx_mbox']['poll'],
+                $context['ctx_mbox']['nopoll']
+            );
+        }
+        if (!$imp_imap->access(IMP_Imap::ACCESS_FOLDERS)) {
+            unset(
+                $context['ctx_mbox']['_sep7'],
+                $context['ctx_mbox']['size']
+            );
+        }
+        if (!$imp_imap->access(IMP_Imap::ACCESS_SEARCH)) {
+            unset(
+                $context['ctx_mbox']['_sep4'],
+                $context['ctx_mbox']['search']
+            );
         }
 
         /* Other Actions context menu. */
@@ -414,7 +461,8 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
             'save' => _("Save"),
             'viewsource' => _("View Source"),
             'allparts' => _("All Parts"),
-            'thread' => _("View Thread")
+            'thread' => _("View Thread"),
+            'listinfo' => _("List Info")
         );
 
         if (empty($conf['user']['allow_view_source'])) {
@@ -445,17 +493,25 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
                 '*advanced' => _("Advanced Search...")
             );
             /* Generate filter array. */
-            $imp_search = $injector->getInstance('IMP_Search');
-            $imp_search->setIteratorFilter(IMP_Search::LIST_FILTER);
+            $iterator = IMP_Search_IteratorFilter::create(
+                IMP_Search_IteratorFilter::FILTER
+            );
 
             $context['ctx_filter'] = array();
-            foreach (iterator_to_array($imp_search) as $key => $val) {
+            foreach ($iterator as $val) {
                 if ($val->enabled) {
-                    $context['ctx_filter']['*' . $key] = $val->label;
+                    $context['ctx_filter']['*' . $val->id] = $val->label;
                 }
             }
         }
 
+        /* Remote accounts context menu. */
+        if ($imp_imap->access(IMP_Imap::ACCESS_REMOTE)) {
+            $context['ctx_rcontainer'] = array(
+                '*prefs' => _("Manage Remote Accounts")
+            );
+        }
+
         $this->js_context = array_merge($context, $this->js_context);
 
         $this->js_text += array(
@@ -478,6 +534,7 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
             'message_0' => _("No messages"),
             'message_1' => _("1 message"),
             'message_2' => _("%d messages"),
+            'mboxsize' => _("%s is: %s."),
             'moveto' => _("Move %s to %s"),
             'newflag' => _("Create New Flag..."),
             'newflag_name' => _("Flag Name:"),
@@ -485,6 +542,7 @@ class IMP_Dynamic_Mailbox extends IMP_Dynamic_Base
             'onlogout' => _("Logging Out..."),
             'portal' => _("Portal"),
             'prefs' => _("User Options"),
+            'remote_password' => _("Password for %s:"),
             'rename_prompt' => _("Rename %s to:"),
             'search' => _("Search"),
             'search_input' => _("Search (%s)"),
