@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,7 +17,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -34,7 +34,7 @@ class IMP_Factory_Mail extends Horde_Core_Factory_Mail
         if ($transport == 'smtp') {
             $params = array_merge(
                 $params,
-                $injector->getInstance('IMP_Imap')->config->smtp
+                $injector->getInstance('IMP_Factory_Imap')->create()->config->smtp
             );
         }
 
