@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2009-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2009-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2009-2013 Horde LLC
+ * @copyright 2009-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,7 +17,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2009-2013 Horde LLC
+ * @copyright 2009-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -111,7 +111,7 @@ class IMP_Flags implements ArrayAccess, Serializable
      */
     public function getList(array $opts = array())
     {
-        if (!$GLOBALS['injector']->getInstance('IMP_Imap')->access(IMP_Imap::ACCESS_FLAGS)) {
+        if (!$GLOBALS['injector']->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_FLAGS)) {
             return array();
         }
 
@@ -238,6 +238,8 @@ class IMP_Flags implements ArrayAccess, Serializable
      */
     public function parse(array $opts = array())
     {
+        global $injector;
+
         $opts = array_merge(array(
             'flags' => array(),
             'headers' => null,
@@ -246,7 +248,14 @@ class IMP_Flags implements ArrayAccess, Serializable
 
         if (!empty($opts['runhook']) && $this->_flaghook) {
             try {
-                $opts['flags'] = array_merge($opts['flags'], Horde::callHook('msglist_flags', array($opts['runhook']), 'imp'));
+                $opts['flags'] = array_merge(
+                    $opts['flags'],
+                    $injector->getInstance('Horde_Core_Hooks')->callHook(
+                        'msglist_flags',
+                        'imp',
+                        array($opts['runhook'])
+                    )
+                );
             } catch (Horde_Exception_HookNotSet $e) {
                 $this->_flaghook = false;
             }
@@ -382,23 +391,25 @@ class IMP_Flags implements ArrayAccess, Serializable
      */
     public function serialize()
     {
-        return serialize(array(
-            $this->_flags,
-            $this->_userflags
-        ));
+        return $GLOBALS['injector']->getInstance('Horde_Pack')->pack(
+            array(
+                $this->_flags,
+                $this->_userflags
+            ), array(
+                'compression' => false,
+                'phpob' => true
+            )
+        );
     }
 
     /**
      */
     public function unserialize($data)
     {
-        $data = @unserialize($data);
-        if (!is_array($data)) {
-            throw new Exception('Cache invalidation.');
-        }
-
-        $this->_flags = $data[0];
-        $this->_userflags = $data[1];
+        list(
+            $this->_flags,
+            $this->_userflags
+        ) = $GLOBALS['injector']->getInstance('Horde_Pack')->unpack($data);
     }
 
 }
