@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -29,6 +29,13 @@ class IMP_Images
      */
     public $alwaysShow = false;
 
+    /**
+     * Results cache.
+     *
+     * @var array
+     */
+    protected $_cache = array();
+
     /**
      * Show inline images in messages?
      *
@@ -38,6 +45,20 @@ class IMP_Images
      * @return boolean  True if inline image should be shown.
      */
     public function showInlineImage(IMP_Contents $contents)
+    {
+        $cid = strval($contents);
+
+        if (!isset($this->_cache[$cid])) {
+            $this->_cache[$cid] = $this->_showInlineImage($contents);
+        }
+
+        return $this->_cache[$cid];
+    }
+
+    /**
+     * @see showInlineImage
+     */
+    protected function _showInlineImage(IMP_Contents $contents)
     {
         global $injector, $prefs, $registry;
 
