@@ -1,14 +1,14 @@
 <?php
 /**
  * Copyright 2000-2001 Chris Hyde <chris@jeks.net>
- * Copyright 2000-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2000-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
  * @copyright 2000-2001 Chris Hyde
- * @copyright 2000-2013 Horde LLC
+ * @copyright 2000-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -24,7 +24,7 @@
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
  * @copyright 2000-2001 Chris Hyde
- * @copyright 2000-2013 Horde LLC
+ * @copyright 2000-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -50,7 +50,7 @@ class IMP_Message
     public function copy($targetMbox, $action, IMP_Indices $indices,
                          array $opts = array())
     {
-        global $conf, $injector, $notification;
+        global $conf, $notification;
 
         if (!count($indices)) {
             return false;
@@ -90,8 +90,6 @@ class IMP_Message
             break;
         }
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
-
         foreach ($indices as $ob) {
             try {
                 if ($targetMbox->readonly) {
@@ -105,6 +103,7 @@ class IMP_Message
                 $ob->mbox->uidvalid;
 
                 /* Attempt to copy/move messages to new mailbox. */
+                $imp_imap = $ob->mbox->imp_imap;
                 $imp_imap->copy($ob->mbox, $targetMbox, array(
                     'ids' => $imp_imap->getIdsOb($ob->uids),
                     'move' => $imap_move
@@ -161,17 +160,16 @@ class IMP_Message
         }
 
         $ajax_queue = $injector->getInstance('IMP_Ajax_Queue');
-        $imp_imap = $injector->getInstance('IMP_Imap');
-        $maillog = $injector->getInstance('IMP_Maillog');
-
-        $maillog_update = (empty($opts['keeplog']) && $maillog);
+        $maillog = empty($opts['keeplog'])
+            ? $injector->getInstance('IMP_Maillog')
+            : null;
         $return_value = 0;
 
         /* Check for Trash mailbox. */
         $no_expunge = $use_trash_mbox = $use_vtrash = false;
         if ($use_trash &&
             empty($opts['nuke']) &&
-            $imp_imap->access(IMP_Imap::ACCESS_TRASH)) {
+            $injector->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_TRASH)) {
             $use_vtrash = $trash->vtrash;
             $use_trash_mbox = !$use_vtrash;
         }
@@ -208,10 +206,14 @@ class IMP_Message
                 $return_value += count($ob->uids);
             }
 
+            $imp_imap = $ob->mbox->imp_imap;
             $ids_ob = $imp_imap->getIdsOb($ob->uids);
 
             /* Trash is only valid for IMAP mailboxes. */
-            if ($use_trash_mbox && ($ob->mbox != $trash)) {
+            if ($use_trash_mbox &&
+                ($ob->mbox != $trash) &&
+                /* TODO(?): Don't use Trash mailbox for remote accounts. */
+                !$ob->mbox->remote_mbox) {
                 if ($ob->mbox->access_expunge) {
                     try {
                         if ($mark_seen) {
@@ -243,24 +245,19 @@ class IMP_Message
                     }
                 }
             } else {
-                /* Get the list of Message-IDs for the deleted messages if
-                 * using mail logging. */
-                $fetch = null;
-                if ($maillog_update) {
-                    /* Optimization: make sure we open mailbox read-write,
-                     * since we are guaranteed to at least attempt to store
-                     * flags below and this query will likely open mailbox in
-                     * read-only mode. */
-                    $imp_imap->openMailbox($ob->mbox, Horde_Imap_Client::OPEN_READWRITE);
-
-                    $query = new Horde_Imap_Client_Fetch_Query();
-                    $query->envelope();
-
-                    try {
-                        $fetch = $imp_imap->fetch($ob->mbox, $query, array(
-                            'ids' => $ids_ob
-                        ));
-                    } catch (IMP_Imap_Exception $e) {}
+                /* Delete message logs now. This may result in loss of message
+                 * log data for messages that might not be deleted - i.e. if
+                 * an error occurs. But 1) the user has already indicated they
+                 * don't care about this data and 2) message IDs (used by some
+                 * maillog backends) won't be available after deletion. */
+                if ($maillog) {
+                    $delete_ids = array();
+                    foreach ($ids_ob as $val) {
+                        $delete_ids[] = new IMP_Maillog_Message(
+                            new IMP_Indices($ob->mbox, $val)
+                        );
+                    }
+                    $maillog->deleteLog($delete_ids);
                 }
 
                 /* Delete the messages. */
@@ -270,7 +267,8 @@ class IMP_Message
                 if (!$use_vtrash &&
                     (!$imp_imap->access(IMP_Imap::ACCESS_TRASH) ||
                      !empty($opts['nuke']) ||
-                     ($use_trash && ($ob->mbox == $trash)))) {
+                     ($use_trash &&
+                      ($ob->mbox == $trash) || $ob->mbox->remote_mbox))) {
                     /* Purge messages immediately. */
                     $expunge_now = !$no_expunge;
                 } elseif ($mark_seen) {
@@ -297,16 +295,6 @@ class IMP_Message
                         $ajax_queue->flag($del_flags, true, new IMP_Indices($ob->mbox, $ids_ob));
                     }
                 } catch (IMP_Imap_Exception $e) {}
-
-                /* Get the list of Message-IDs deleted, and remove the
-                 * information from the mail log. */
-                if (!is_null($fetch)) {
-                    $msg_ids = array();
-                    foreach ($fetch as $v) {
-                        $msg_ids[] = $v->getEnvelope()->message_id;
-                    }
-                    $maillog->deleteLog(array_filter($msg_ids));
-                }
             }
         }
 
@@ -510,7 +498,7 @@ class IMP_Message
         $url->uid = $uid;
         $url->uidvalidity = $uidvalidity;
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $mbox->imp_imap;
 
         /* Always add the header to output. */
         $url->section = 'HEADER';
@@ -634,6 +622,7 @@ class IMP_Message
      *                              and/or 'remove'.
      * @param IMP_Indices $indices  An indices object.
      * @param array $opts           Additional options:
+     *   - silent: (boolean) Don't output notification messages.
      *   - unchangedsince: (array) The unchangedsince value to pass to the
      *                     IMAP store command. Keys are mailbox names, values
      *                     are the unchangedsince values to use for that
@@ -655,7 +644,6 @@ class IMP_Message
         ), $opts);
 
         $ajax_queue = $injector->getInstance('IMP_Ajax_Queue');
-        $imp_imap = $injector->getInstance('IMP_Imap');
         $ret = true;
 
         foreach ($indices as $ob) {
@@ -671,6 +659,7 @@ class IMP_Message
                     : null;
 
                 /* Flag/unflag the messages now. */
+                $imp_imap = $ob->mbox->imp_imap;
                 $res = $imp_imap->store($ob->mbox, array_merge($action, array_filter(array(
                     'ids' => $imp_imap->getIdsOb($ob->uids),
                     'unchangedsince' => $unchangedsince
@@ -682,7 +671,10 @@ class IMP_Message
                     foreach ($res as $val) {
                         unset($flag_change[$val]);
                     }
-                    $notification->push(sprintf(_("Flags were not changed for at least one message in the mailbox \"%s\" because the flags were altered by another connection to the mailbox prior to this request. You may redo the flag action if desired; this warning is precautionary to ensure you don't overwrite flag changes."), $ob->mbox->display), 'horde.warning');
+                    if (empty($opts['silent'])) {
+                        $notification->push(sprintf(_("Flags were not changed for at least one message in the mailbox \"%s\" because the flags were altered by another connection to the mailbox prior to this request. You may redo the flag action if desired; this warning is precautionary to ensure you don't overwrite flag changes."), $ob->mbox->display), 'horde.warning');
+                        $ret = false;
+                    }
                 }
 
                 foreach ($action as $key => $val) {
@@ -692,7 +684,9 @@ class IMP_Message
                     }
                 }
             } catch (Exception $e) {
-                $notification->push(sprintf(_("There was an error flagging messages in the mailbox \"%s\": %s."), $ob->mbox->display, $e->getMessage()), 'horde.error');
+                if (empty($opts['silent'])) {
+                    $notification->push(sprintf(_("There was an error flagging messages in the mailbox \"%s\": %s."), $ob->mbox->display, $e->getMessage()), 'horde.error');
+                }
                 $ret = false;
             }
         }
@@ -721,7 +715,6 @@ class IMP_Message
             ? array('add' => $flags)
             : array('remove' => $flags);
         $ajax_queue = $GLOBALS['injector']->getInstance('IMP_Ajax_Queue');
-        $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
 
         $ajax_queue->poll($mboxes);
 
@@ -730,7 +723,7 @@ class IMP_Message
                 /* Grab list of UIDs before flagging, to make sure we
                  * determine the exact subset that has been flagged. */
                 $mailbox_list = $val->list_ob->getIndicesOb();
-                $imp_imap->store($val, $action_array);
+                $val->imp_imap->store($val, $action_array);
                 $ajax_queue->flag(reset($action_array), $action, $mailbox_list);
             } catch (IMP_Imap_Exception $e) {
                 return false;
@@ -765,14 +758,13 @@ class IMP_Message
             return $msg_list ? new IMP_Indices() : null;
         }
 
-        $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
         $process_list = $update_list = array();
 
         foreach ($mbox_list as $key => $val) {
             $key = IMP_Mailbox::get($key);
 
             if ($key->access_expunge) {
-                $ids = $imp_imap->getIdsOb(is_array($val) ? $val : Horde_Imap_Client_Ids::ALL);
+                $ids = $key->imp_imap->getIdsOb(is_array($val) ? $val : Horde_Imap_Client_Ids::ALL);
 
                 if ($key->search) {
                     foreach ($key->getSearchOb()->mboxes as $skey) {
@@ -797,7 +789,7 @@ class IMP_Message
             }
 
             try {
-                $update_list[strval($val[0])] = $imp_imap->expunge($val[0], array(
+                $update_list[strval($val[0])] = $val[0]->imp_imap->expunge($val[0], array(
                     'ids' => $val[1],
                     'list' => $msg_list
                 ));
@@ -821,9 +813,8 @@ class IMP_Message
      */
     public function emptyMailbox($mbox_list)
     {
-        global $injector, $notification, $prefs;
+        global $notification, $prefs;
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
         $trash = ($prefs->getValue('use_trash'))
             ? IMP_Mailbox::getPref(IMP_Mailbox::MBOX_TRASH)
             : null;
@@ -843,6 +834,7 @@ class IMP_Message
             /* Make sure there is at least 1 message before attempting to
              * delete. */
             try {
+                $imp_imap = $mbox->imp_imap;
                 $status = $imp_imap->status($mbox, Horde_Imap_Client::STATUS_MESSAGES);
                 if (empty($status['messages'])) {
                     $notification->push(sprintf(_("The mailbox %s is already empty."), $mbox->display), 'horde.message');
@@ -864,36 +856,4 @@ class IMP_Message
         }
     }
 
-    /**
-     * Obtains the size of a mailbox.
-     *
-     * @param IMP_Mailbox $mbox   The mailbox to obtain the size of.
-     * @param boolean $formatted  Whether to return a human readable value.
-     *
-     * @return mixed  Either the size of the mailbox (in bytes) or a formatted
-     *                string with this information.
-     */
-    public function sizeMailbox(IMP_Mailbox $mbox, $formatted = true)
-    {
-        $query = new Horde_Imap_Client_Fetch_Query();
-        $query->size();
-
-        try {
-            $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
-            $res = $imp_imap->fetch($mbox, $query, array(
-                'ids' => $imp_imap->getIdsOb(Horde_Imap_Client_Ids::ALL, true)
-            ));
-
-            $size = 0;
-            foreach ($res as $v) {
-                $size += $v->getSize();
-            }
-            return ($formatted)
-                ? sprintf(_("%.2fMB"), $size / (1024 * 1024))
-                : $size;
-        } catch (IMP_Imap_Exception $e) {
-            return 0;
-        }
-    }
-
 }
