@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -18,15 +18,20 @@
  * @author    Jon Parise <jon@horde.org>
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
 class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
 {
-    /* CSS background regex. */
+    /** CSS background regex. */
     const CSS_BG_PREG = '/(background(?:-image)?:[^;\}]*(?:url\(["\']?))(.*?)((?:["\']?\)))/i';
 
+    /** Blocked attributes. */
+    const CSSBLOCK = 'htmlcssblocked';
+    const IMGBLOCK = 'htmlimgblocked';
+    const SRCSETBLOCK = 'htmlimgblocked_srcset';
+
     /**
      * Temp array for storing data when parsing the HTML document.
      *
@@ -83,14 +88,14 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
             $page_output->addScriptPackage('IMP_Script_Package_Imp');
 
             $data['js'] = array(
-                'IMP_JS.iframeInject("' . $uid . '", ' . Horde_Serialize::serialize($data['data'], Horde_Serialize::JSON) . ')'
+                'IMP_JS.iframeInject("' . $uid . '", ' . json_encode($data['data']) . ')'
             );
 
             if ($view == $registry::VIEW_SMARTMOBILE) {
                 $data['js'][] = '$("#imp-message-body a[href=\'#unblock-image\']").button()';
             }
 
-            $data['data'] = '<div>' . _("Loading...") . '</div><iframe class="htmlMsgData" id="' . $uid . '" src="javascript:false" frameborder="0" style="display:none"></iframe>';
+            $data['data'] = '<div>' . _("Loading...") . '</div><iframe class="htmlMsgData" id="' . $uid . '" src="javascript:false" frameborder="0" style="display:none;height:auto;"></iframe>';
             $data['type'] = 'text/html; charset=UTF-8';
             break;
         }
@@ -148,30 +153,17 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
 
         /* Don't do IMP DOM processing if in mimp mode or converting to
          * text. */
-        $this->_imptmp = array(
-            'target' => strval(new Horde_Support_Randomid())
-        );
+        $this->_imptmp = array();
         if ($inline && !$convert_text) {
             $this->_imptmp += array(
-                'blockimg' => null,
                 'cid' => null,
                 'cid_used' => array(),
                 'cssblock' => false,
                 'cssbroken' => false,
-                'img' => false,
                 'imgblock' => false,
                 'inline' => $inline,
-                'style' => array(),
-                'target' => strval(new Horde_Support_Randomid())
+                'style' => array()
             );
-
-            if ($inline) {
-                /* Image filtering. */
-                if (!$injector->getInstance('IMP_Images')->showInlineImage($contents)) {
-                    $this->_imptmp['blockimg'] = strval(Horde_Themes::img('spacer_red.png'));
-                    $this->_imptmp['img'] = true;
-                }
-            }
         }
 
         /* Search for inlined data that we can display (multipart/related
@@ -314,6 +306,21 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
 
         $tag = Horde_String::lower($node->tagName);
 
+        /* Remove 'height' styles from HTML messages, because it can break
+         * sizing of IFRAME. */
+        foreach ($node->attributes as $key => $val) {
+            if ($key == 'style') {
+                /* Do simplistic style parsing here. */
+                $parts = array_filter(explode(';', $val->value));
+                foreach ($parts as $k2 => $v2) {
+                    if (preg_match("/^\s*height:\s*/i", $v2)) {
+                        unset($parts[$k2]);
+                    }
+                }
+                $val->value = implode(';', $parts);
+            }
+        }
+
         switch ($tag) {
         case 'a':
         case 'area':
@@ -334,13 +341,24 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
                     /* See Bug #8695: internal anchors are broken in
                      * Mozilla. */
                     $node->removeAttribute('href');
-                } elseif (!$node->hasAttribute('target') ||
-                          Horde_String::lower($node->getAttribute('target')) == '_self') {
-                    $node->setAttribute('target', $this->_imptmp['target']);
+                } elseif (empty($url)) {
+                    /* Empty URL - remove href/target so the link is not
+                     * clickable. */
+                    $node->removeAttribute('href');
+                    $node->removeAttribute('target');
+                } else {
+                    $node->setAttribute('target', strval(new Horde_Support_Randomid()));
                 }
             }
             break;
 
+        case 'body':
+            $style = $node->hasAttribute('style')
+                ? (rtrim($node->getAttribute('style'), ';') . ';')
+                : '';
+            $node->setAttribute('style', $style . 'width:auto !important');
+            break;
+
         case 'img':
         case 'input':
             if ($node->hasAttribute('src')) {
@@ -357,18 +375,27 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
                 }
 
                 /* Block images.*/
-                if (!empty($this->_imptmp['img'])) {
+                if ($this->_imgBlock()) {
                     if (Horde_Url_Data::isData($val)) {
                         $url = new Horde_Url_Data($val);
                     } else {
                         $url = new Horde_Url($val);
                         $url->setScheme();
                     }
-                    $node->setAttribute('htmlimgblocked', $url);
-                    $node->setAttribute('src', $this->_imptmp['blockimg']);
+                    $node->setAttribute(self::IMGBLOCK, $url);
+                    $node->setAttribute('src', $this->_imgBlockImg());
                     $this->_imptmp['imgblock'] = true;
                 }
             }
+
+            /* IMG only */
+            if (($tag == 'img') &&
+                $this->_imgBlock() &&
+                $node->hasAttribute('srcset')) {
+                $node->setAttribute(self::SRCSETBLOCK, $node->getAttribute('srcset'));
+                $node->setAttribute('srcset', '');
+                $this->_imptmp['imgblock'] = true;
+            }
             break;
 
         case 'link':
@@ -385,8 +412,8 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
 
                     if ($id = $this->_cidSearch($tmp, false)) {
                         $this->_imptmp['style'][] = $this->getConfigParam('imp_contents')->getMIMEPart($id)->getContents();
-                    } elseif (!empty($this->_imptmp['img'])) {
-                        $node->setAttribute('htmlcssblocked', $node->getAttribute('href'));
+                    } elseif ($this->_imgBlock()) {
+                        $node->setAttribute(self::CSSBLOCK, $node->getAttribute('href'));
                         $node->removeAttribute('href');
                         $this->_imptmp['cssblock'] = true;
                         $delete_link = false;
@@ -441,9 +468,9 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
                 }
 
                 /* Block images.*/
-                if (!empty($this->_imptmp['img'])) {
-                    $node->setAttribute('htmlimgblocked', $val);
-                    $node->setAttribute('background', $this->_imptmp['blockimg']);
+                if ($this->_imgBlock()) {
+                    $node->setAttribute(self::IMGBLOCK, $val);
+                    $node->setAttribute('background', $this->_imgBlockImg());
                     $this->_imptmp['imgblock'] = true;
                 }
             }
@@ -467,8 +494,7 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
             if (strpos($node->getAttribute('style'), 'content:') !== false) {
                 // TODO: Figure out way to unblock?
                 $node->removeAttribute('style');
-            } elseif (!empty($this->_imptmp['img']) ||
-                      !empty($this->_imptmp['cid'])) {
+            } elseif (!empty($this->_imptmp['cid']) || $this->_imgBlock()) {
                 $this->_imptmp['node'] = $node;
                 $style = preg_replace_callback(self::CSS_BG_PREG, array($this, '_styleCallback'), $node->getAttribute('style'), -1, $matches);
                 if ($matches) {
@@ -514,7 +540,8 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
         }
 
         if (strlen($css_text)) {
-            $style_elt = $doc->createElement('style', $css_text);
+            $style_elt = $doc->createElement('style');
+            $style_elt->appendChild(new DOMText($css_text));
             $style_elt->setAttribute('type', 'text/css');
             $headelt->appendChild($style_elt);
         }
@@ -523,7 +550,8 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
          * output - then we simply need to change the type attribute to
          * text/css, and the browser should load the definitions on-demand. */
         if (strlen($blocked_text)) {
-            $block_elt = $doc->createElement('style', $blocked_text);
+            $block_elt = $doc->createElement('style');
+            $block_elt->appendChild(new DOMText($blocked_text));
             $block_elt->setAttribute('type', 'text/x-imp-cssblocked');
             $headelt->appendChild($block_elt);
         }
@@ -586,9 +614,9 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
                 'imp_img_view' => 'data'
             )));
         } else {
-            $this->_imptmp['node']->setAttribute('htmlimgblocked', $matches[2]);
+            $this->_imptmp['node']->setAttribute(self::IMGBLOCK, $matches[2]);
             $this->_imptmp['imgblock'] = true;
-            $replace = $this->_imptmp['blockimg'];
+            $replace = $this->_imgBlockImg();
         }
         return $matches[1] . $replace . $matches[3];
     }
@@ -616,4 +644,38 @@ class IMP_Mime_Viewer_Html extends Horde_Mime_Viewer_Html
         return $id;
     }
 
+    /**
+     * Are we blocking images?
+     *
+     * @return boolean  True if blocking images.
+     */
+    protected function _imgBlock()
+    {
+        global $injector;
+
+        /* Done on demand, since we potentially save a contacts API call if
+         * not needed/used in a message. */
+        if (!isset($this->_imptmp['img'])) {
+            $this->_imptmp['img'] =
+                ($this->_imptmp['inline'] &&
+                !$injector->getInstance('IMP_Images')->showInlineImage($this->getConfigParam('imp_contents')));
+        }
+
+        return $this->_imptmp['img'];
+    }
+
+    /**
+     * The HTML image source to use for blocked images.
+     *
+     * @return string  The HTML image source.
+     */
+    protected function _imgBlockImg()
+    {
+        if (!isset($this->_imptmp['blockimg'])) {
+            $this->_imptmp['blockimg'] = strval(Horde_Themes::img('spacer_red.png'));
+        }
+
+        return $this->_imptmp['blockimg'];
+    }
+
 }
