@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -25,17 +25,14 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
 class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
 {
     /* Metadata constants. */
-    const PGP_ARMOR = 'imp-pgp-armor';
-    const PGP_SIG = 'imp-pgp-signature';
     const PGP_SIGN_ENC = 'imp-pgp-signed-encrypted';
-    const PGP_CHARSET = 'imp-pgp-charset';
 
     /**
      * This driver's display capabilities.
@@ -71,13 +68,6 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
      */
     protected $_sender = null;
 
-    /**
-     * Cached data.
-     *
-     * @var array
-     */
-    static protected $_cache = array();
-
     /**
      * Return the full rendered version of the Horde_Mime_Part object.
      *
@@ -121,7 +111,7 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
 
         switch ($this->_mimepart->getType()) {
         case 'application/pgp-signature':
-            $parts = $GLOBALS['injector']->getInstance('IMP_Crypt_Pgp')->parsePGPData($this->_mimepart->getContents());
+            $parts = $GLOBALS['injector']->getInstance('IMP_Crypt_Pgp_Parse')->parse($this->_mimepart->getContents());
             foreach (array_keys($parts) as $key) {
                 if ($parts[$key]['type'] == Horde_Crypt_Pgp::ARMOR_SIGNATURE) {
                     $ret['data'] = implode("\r\n", $parts[$key]['data']);
@@ -153,20 +143,17 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
             return $this->_outputPGPSigned();
 
         case 'multipart/encrypted':
-            if (!isset($headers)) {
-                $headers = $this->getConfigParam('imp_contents')->getHeader();
-            }
+            $cache = $this->getConfigParam('imp_contents')->getViewCache();
 
-            $mid = $headers->getValue('message-id');
-            if (isset(self::$_cache[$mid][$id])) {
+            if (isset($cache->pgp[$id])) {
                 return array_merge(array(
                     $id => array(
                         'data' => null,
-                        'status' => self::$_cache[$mid][$id]['status'],
+                        'status' => $cache->pgp[$id]['status'],
                         'type' => 'text/plain; charset=' . $this->getConfigParam('charset'),
-                        'wrap' => self::$_cache[$mid][$id]['wrap']
+                        'wrap' => $cache->pgp[$id]['wrap']
                     )
-                ), self::$_cache[$mid][$id]['other']);
+                ), $cache->pgp[$id]['other']);
             }
             // Fall-through
 
@@ -190,8 +177,6 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
             return null;
         }
 
-        $mid = $this->getConfigParam('imp_contents')->getHeader()->getValue('message-id');
-
         $partlist = array_keys($this->_mimepart->contentTypeMap());
         $base_id = reset($partlist);
         $version_id = next($partlist);
@@ -200,7 +185,8 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
         $status = new IMP_Mime_Status();
         $status->icon('mime/encryption.png', 'PGP');
 
-        self::$_cache[$mid][$base_id] = array(
+        $cache = $this->getConfigParam('imp_contents')->getViewCache();
+        $cache->pgp[$base_id] = array(
             'status' => array($status),
             'other' => array(
                 $version_id => null,
@@ -310,7 +296,7 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
             return null;
         }
 
-        self::$_cache[$mid][$base_id]['wrap'] = 'mimePartWrapValid';
+        $cache->pgp[$base_id]['wrap'] = 'mimePartWrapValid';
 
         /* Check for combined encryption/signature data. */
         if ($decrypted_data->result) {
@@ -321,11 +307,11 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
             $status2 = new IMP_Mime_Status($sig_text);
             $status2->action(IMP_Mime_Status::SUCCESS);
 
-            self::$_cache[$mid][$base_id]['status'][] = $status2;
+            $cache->pgp[$base_id]['status'][] = $status2;
         }
 
         /* Force armor data as text/plain data. */
-        if ($this->_mimepart->getMetadata(self::PGP_ARMOR)) {
+        if ($this->_mimepart->getMetadata(Horde_Crypt_Pgp_Parse::PGP_ARMOR)) {
             $decrypted_data->message = "Content-Type: text/plain\n\n" .
                                        $decrypted_data->message;
         }
@@ -400,12 +386,21 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
      */
     protected function _outputPGPSigned()
     {
+        global $conf, $injector, $prefs, $registry, $session;
+
         $partlist = array_keys($this->_mimepart->contentTypeMap());
         $base_id = reset($partlist);
         $signed_id = next($partlist);
         $sig_id = Horde_Mime::mimeIdArithmetic($signed_id, 'next');
 
+        if (!$prefs->getValue('use_pgp') || empty($conf['gnupg']['path'])) {
+            return array(
+                $sig_id => null
+            );
+        }
+
         $status = new IMP_Mime_Status();
+        $status->addText(_("The data in this part has been digitally signed via PGP."));
         $status->icon('mime/encryption.png', 'PGP');
 
         $ret = array(
@@ -419,48 +414,48 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
             $sig_id => null
         );
 
-        if (!$GLOBALS['prefs']->getValue('use_pgp') ||
-            empty($GLOBALS['conf']['gnupg']['path'])) {
-            /* If PGP not active, hide signature data and output status
-             * information. */
-            $status->addText(_("The data in this part has been digitally signed via PGP, but the signature cannot be verified."));
-            return $ret;
-        }
-
-        $status->addText(_("The data in this part has been digitally signed via PGP."));
-
-        if ($GLOBALS['prefs']->getValue('pgp_verify') ||
-            $GLOBALS['injector']->getInstance('Horde_Variables')->pgp_verify_msg) {
+        if ($prefs->getValue('pgp_verify') ||
+            $injector->getInstance('Horde_Variables')->pgp_verify_msg) {
             $imp_contents = $this->getConfigParam('imp_contents');
             $sig_part = $imp_contents->getMIMEPart($sig_id);
 
             $status2 = new IMP_Mime_Status();
 
-            try {
-                $imp_pgp = $GLOBALS['injector']->getInstance('IMP_Crypt_Pgp');
-                if ($sig_part->getMetadata(self::PGP_SIG)) {
-                    $sig_result = $imp_pgp->verifySignature($sig_part->getContents(array('canonical' => true)), $this->_getSender()->bare_address, null, $sig_part->getMetadata(self::PGP_CHARSET));
-                } else {
-                    $stream = $imp_contents->isEmbedded($signed_id)
-                        ? $this->_mimepart->getMetadata(self::PGP_SIGN_ENC)
-                        : $imp_contents->getBodyPart($signed_id, array('mimeheaders' => true, 'stream' => true));
-
-                    rewind($stream);
-                    stream_filter_register('horde_eol', 'Horde_Stream_Filter_Eol');
-                    stream_filter_append($stream, 'horde_eol', STREAM_FILTER_READ, array(
-                        'eol' => Horde_Mime_Part::RFC_EOL
-                    ));
-
-                    $sig_result = $imp_pgp->verifySignature(stream_get_contents($stream), $this->_getSender()->bare_address, $sig_part->getContents());
-                }
-
-                $status2->action(IMP_Mime_Status::SUCCESS);
-                $sig_text = $sig_result->message;
-                $ret[$base_id]['wrap'] = 'mimePartWrapValid';
-            } catch (Horde_Exception $e) {
+            if (!$sig_part) {
                 $status2->action(IMP_Mime_Status::ERROR);
-                $sig_text = $e->getMessage();
+                $sig_text = _("This digitally signed message is broken.");
                 $ret[$base_id]['wrap'] = 'mimePartWrapInvalid';
+            } else {
+                /* Close session, since this may be a long-running
+                 * operation. */
+                $session->close();
+
+                try {
+                    $imp_pgp = $injector->getInstance('IMP_Crypt_Pgp');
+                    if ($sig_raw = $sig_part->getMetadata(Horde_Crypt_Pgp_Parse::SIG_RAW)) {
+                        $sig_result = $imp_pgp->verifySignature($sig_raw, $this->_getSender()->bare_address, null, $sig_part->getMetadata(Horde_Crypt_Pgp_Parse::SIG_CHARSET));
+                    } else {
+                        $stream = $imp_contents->isEmbedded($signed_id)
+                            ? $this->_mimepart->getMetadata(self::PGP_SIGN_ENC)
+                            : $imp_contents->getBodyPart($signed_id, array('mimeheaders' => true, 'stream' => true))->data;
+
+                        rewind($stream);
+                        stream_filter_register('horde_eol', 'Horde_Stream_Filter_Eol');
+                        stream_filter_append($stream, 'horde_eol', STREAM_FILTER_READ, array(
+                            'eol' => Horde_Mime_Part::RFC_EOL
+                        ));
+
+                        $sig_result = $imp_pgp->verifySignature(stream_get_contents($stream), $this->_getSender()->bare_address, $sig_part->getContents());
+                    }
+
+                    $status2->action(IMP_Mime_Status::SUCCESS);
+                    $sig_text = $sig_result->message;
+                    $ret[$base_id]['wrap'] = 'mimePartWrapValid';
+                } catch (Horde_Exception $e) {
+                    $status2->action(IMP_Mime_Status::ERROR);
+                    $sig_text = $e->getMessage();
+                    $ret[$base_id]['wrap'] = 'mimePartWrapInvalid';
+                }
             }
 
             $status2->addText($this->_textFilter($sig_text, 'text2html', array(
@@ -468,7 +463,7 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
             )));
             $ret[$base_id]['status'][] = $status2;
         } else {
-            switch ($GLOBALS['registry']->getView()) {
+            switch ($registry->getView()) {
             case Horde_Registry::VIEW_BASIC:
                 $status->addText(Horde::link(Horde::selfUrlParams()->add(array('pgp_verify_msg' => 1))) . _("Click HERE to verify the message.") . '</a>');
                 break;
@@ -526,7 +521,7 @@ class IMP_Mime_Viewer_Pgp extends Horde_Mime_Viewer_Base
 
         case 'raw':
             if (($this->_mimepart->getType() == 'application/pgp-signature') &&
-                $this->_mimepart->getMetadata(self::PGP_SIG)) {
+                $this->_mimepart->getMetadata(Horde_Crypt_Pgp_Parse::SIG_RAW)) {
                 return true;
             }
             break;
