@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2009-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2009-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2009-2013 Horde LLC
+ * @copyright 2009-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,16 +17,39 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2009-2013 Horde LLC
+ * @copyright 2009-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
 class IMP_Mime_Viewer_Rfc822 extends Horde_Mime_Viewer_Rfc822
 {
     /**
-     * Return the rendered information about the Horde_Mime_Part object.
+     * This driver's display capabilities.
      *
-     * @return array  See parent::render().
+     * @var array
+     */
+    protected $_capability = array(
+        'full' => true,
+        'info' => true,
+        'inline' => false,
+        'raw' => true
+    );
+
+    /**
+     */
+    protected function _render()
+    {
+        /* Need to display raw text from server, since this is essentially
+         * a View Source action for the part and the current value of
+         * $_mimepart may contain altered data (e.g. data that has been
+         * content transfer decoded). */
+        return $this->_renderReturn(
+            $this->getConfigParam('imp_contents')->getBodyPart($this->_mimepart->getMimeId())->data,
+            'text/plain; charset=' . $this->getConfigParam('charset')
+        );
+    }
+
+    /**
      */
     protected function _renderInfo()
     {
@@ -37,6 +60,14 @@ class IMP_Mime_Viewer_Rfc822 extends Horde_Mime_Viewer_Rfc822
         return $ret;
     }
 
+    /**
+     */
+    protected function _renderRaw()
+    {
+        /* Needed for same reason as explained in _render(). */
+        return $this->_render();
+    }
+
     /**
      */
     protected function _getHeaderValue($ob, $header)
