@@ -1,18 +1,18 @@
 <?php
 /**
- * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2000-2013 Horde LLC
+ * @copyright 2000-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
 
 /**
- * Renderer for viewing/decrypting of S/MIME messages (RFC 2633).
+ * Renderer for viewing/decrypting of S/MIME v3.2 messages (RFC 5751).
  *
  * This class handles the following MIME types:
  *   application/pkcs7-mime
@@ -27,7 +27,7 @@
  * @author    Mike Cochrane <mike@graftonhall.co.nz>
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2000-2013 Horde LLC
+ * @copyright 2000-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -63,13 +63,6 @@ class IMP_Mime_Viewer_Smime extends Horde_Mime_Viewer_Base
      */
     protected $_impsmime = null;
 
-    /**
-     * Cached data.
-     *
-     * @var array
-     */
-    static protected $_cache = array();
-
     /**
      * Init the S/MIME Horde_Crypt object.
      */
@@ -112,17 +105,19 @@ class IMP_Mime_Viewer_Smime extends Horde_Mime_Viewer_Base
 
         case 'application/pkcs7-mime':
         case 'application/x-pkcs7-mime':
-            if (isset(self::$_cache[$id])) {
+            $cache = $this->getConfigParam('imp_contents')->getViewCache();
+
+            if (isset($cache->smime[$id])) {
                 $ret = array(
                     $id => array(
                         'data' => null,
-                        'status' => self::$_cache[$id]['status'],
+                        'status' => $cache->smime[$id]['status'],
                         'type' => 'text/plain; charset=' . $this->getConfigParam('charset'),
-                        'wrap' => self::$_cache[$id]['wrap']
+                        'wrap' => $cache->smime[$id]['wrap']
                     )
                 );
-                if (isset(self::$_cache[$id]['sig'])) {
-                    $ret[self::$_cache[$id]['sig']] = null;
+                if (isset($cache->smime[$id]['sig'])) {
+                    $ret[$cache->smime[$id]['sig']] = null;
                 }
                 return $ret;
             }
@@ -173,7 +168,9 @@ class IMP_Mime_Viewer_Smime extends Horde_Mime_Viewer_Base
         /* Initialize inline data. */
         $status = new IMP_Mime_Status(_("The data in this part has been encrypted via S/MIME."));
         $status->icon('mime/encryption.png', 'S/MIME');
-        self::$_cache[$base_id] = array(
+
+        $cache = $this->getConfigParam('imp_contents')->getViewCache();
+        $cache->smime[$base_id] = array(
             'status' => $status,
             'wrap' => ''
         );
@@ -209,7 +206,7 @@ class IMP_Mime_Viewer_Smime extends Horde_Mime_Viewer_Base
             return null;
         }
 
-        self::$_cache[$base_id]['wrap'] = 'mimePartWrapValid';
+        $cache->smime[$base_id]['wrap'] = 'mimePartWrapValid';
 
         $new_part = Horde_Mime_Part::parseMessage($decrypted_data, array(
             'forcemime' => true
@@ -259,13 +256,15 @@ class IMP_Mime_Viewer_Smime extends Horde_Mime_Viewer_Base
     {
         $partlist = array_keys($this->_mimepart->contentTypeMap());
         $base_id = reset($partlist);
-        $sig_id = Horde_Mime::mimeIdArithmetic(next($partlist), 'next');
+        $data_id = next($partlist);
+        $sig_id = Horde_Mime::mimeIdArithmetic($data_id, 'next');
 
         /* Initialize inline data. */
         $status = new IMP_Mime_Status(_("The data in this part has been digitally signed via S/MIME."));
         $status->icon('mime/encryption.png', 'S/MIME');
 
-        self::$_cache[$base_id] = array(
+        $cache = $this->getConfigParam('imp_contents')->getViewCache();
+        $cache->smime[$base_id] = array(
             'sig' => $sig_id,
             'status' => $status,
             'wrap' => 'mimePartWrap'
@@ -276,6 +275,16 @@ class IMP_Mime_Viewer_Smime extends Horde_Mime_Viewer_Base
             return null;
         }
 
+        /* Sanity checking to make sure MIME structure is correct. */
+        if (!in_array($sig_id, $partlist)) {
+            $status->action(IMP_Mime_Status::ERROR);
+            $cache->smime[$base_id]['wrap'] = 'mimePartWrapInvalid';
+            $status->addText(_("Invalid S/MIME data."));
+            /* This will suppress displaying the invalid part. */
+            $cache->smime[$base_id]['sig'] = $data_id;
+            return null;
+        }
+
         $imp_contents = $this->getConfigParam('imp_contents');
         $stream = $imp_contents->isEmbedded($base_id)
             ? $this->_mimepart->getMetadata('imp-smime-decrypt')->stream
@@ -294,7 +303,7 @@ class IMP_Mime_Viewer_Smime extends Horde_Mime_Viewer_Base
                 } else {
                     $status->action(IMP_Mime_Status::WARNING);
                 }
-                self::$_cache[$base_id]['wrap'] = 'mimePartWrapValid';
+                $cache->smime[$base_id]['wrap'] = 'mimePartWrapValid';
 
                 $email = is_array($sig_result->email)
                     ? implode(', ', $sig_result->email)
@@ -337,7 +346,7 @@ class IMP_Mime_Viewer_Smime extends Horde_Mime_Viewer_Base
                 }
             } catch (Horde_Exception $e) {
                 $status->action(IMP_Mime_Status::ERROR);
-                self::$_cache[$base_id]['wrap'] = 'mimePartWrapInvalid';
+                $cache->smime[$base_id]['wrap'] = 'mimePartWrapInvalid';
                 $status->addText($e->getMessage());
             }
         } else {
@@ -402,7 +411,7 @@ class IMP_Mime_Viewer_Smime extends Horde_Mime_Viewer_Base
     protected function _getPartStream($id)
     {
         return $id
-            ? $this->getConfigParam('imp_contents')->getBodyPart($id, array('mimeheaders' => true, 'stream' => true))
+            ? $this->getConfigParam('imp_contents')->getBodyPart($id, array('mimeheaders' => true, 'stream' => true))->data
             : $this->getConfigParam('imp_contents')->fullMessageText();
     }
 
