@@ -2,7 +2,7 @@
 /**
  * Base class for minimal view pages.
  *
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -89,7 +89,7 @@ abstract class IMP_Minimal_Base
         }
 
         if (!in_array($page, array('folders', 'search')) &&
-            $GLOBALS['injector']->getInstance('IMP_Imap')->access(IMP_Imap::ACCESS_FOLDERS)) {
+            $GLOBALS['injector']->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_FOLDERS)) {
             $items[] = array(_("Folders"), IMP_Minimal_Folders::url());
         }
 
