@@ -2,7 +2,7 @@
 /**
  * Compose page for minimal view.
  *
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -101,8 +101,11 @@ class IMP_Minimal_Compose extends IMP_Minimal_Base
             strlen($_FILES['upload_1']['name'])) {
             try {
                 $atc_ob = $imp_compose->addAttachmentFromUpload('upload_1');
+                if ($atc_ob[0] instanceof IMP_Compose_Exception) {
+                    throw $atc_ob[0];
+                }
                 if ($this->vars->a == _("Expand Names")) {
-                    $notification->push(sprintf(_("Added \"%s\" as an attachment."), $atc_ob->getPart()->getName()), 'horde.success');
+                    $notification->push(sprintf(_("Added \"%s\" as an attachment."), $atc_ob[0]->getPart()->getName()), 'horde.success');
                 }
             } catch (IMP_Compose_Exception $e) {
                 $this->vars->a = null;
@@ -180,23 +183,26 @@ class IMP_Minimal_Compose extends IMP_Minimal_Base
         case 'r':
         case 'ra':
         case 'rl':
-            try {
-                $imp_contents = $this->_getContents();
-            } catch (IMP_Exception $e) {
-                $notification->push($e, 'horde.error');
-                break;
-            }
-
             $actions = array(
                 'r' => IMP_Compose::REPLY_SENDER,
                 'ra' => IMP_Compose::REPLY_ALL,
                 'rl' => IMP_Compose::REPLY_LIST
             );
 
-            $reply_msg = $imp_compose->replyMessage($actions[$this->vars->a], $imp_contents, array(
-                'format' => 'text',
-                'to' => $header['to']
-            ));
+            try {
+                $reply_msg = $imp_compose->replyMessage(
+                    $actions[$this->vars->a],
+                    $this->_getContents(),
+                    array(
+                        'format' => 'text',
+                        'to' => $header['to']
+                    )
+                );
+            } catch (IMP_Exception $e) {
+                $notification->push($e, 'horde.error');
+                break;
+            }
+
             $header = $this->_convertToHeader($reply_msg);
 
             $notification->push(_("Reply text will be automatically appended to your outgoing message."), 'horde.message');
@@ -206,13 +212,16 @@ class IMP_Minimal_Compose extends IMP_Minimal_Base
         // 'f' = forward
         case 'f':
             try {
-                $imp_contents = $this->_getContents();
+                $fwd_msg = $imp_compose->forwardMessage(
+                    IMP_Compose::FORWARD_ATTACH,
+                    $this->_getContents(),
+                    false
+                );
             } catch (IMP_Exception $e) {
                 $notification->push($e, 'horde.error');
                 break;
             }
 
-            $fwd_msg = $imp_compose->forwardMessage(IMP_Compose::FORWARD_ATTACH, $imp_contents, false);
             $header = $this->_convertToHeader($fwd_msg);
 
             $notification->push(_("Forwarded message will be automatically added to your outgoing message."), 'horde.message');
@@ -261,16 +270,25 @@ class IMP_Minimal_Compose extends IMP_Minimal_Base
 
             switch ($imp_compose->replyType(true)) {
             case IMP_Compose::REPLY:
-                $reply_msg = $imp_compose->replyMessage(IMP_Compose::REPLY_SENDER, $imp_compose->getContentsOb(), array(
-                    'to' => $f_to
-                ));
-                $msg = $reply_msg['body'];
+                try {
+                    $reply_msg = $imp_compose->replyMessage(IMP_Compose::REPLY_SENDER, $imp_compose->getContentsOb(), array(
+                        'to' => $f_to
+                    ));
+                    $msg = $reply_msg['body'];
+                } catch (IMP_Exception $e) {
+                    $notification->push($e, 'horde.error');
+                    $msg = '';
+                }
                 $message .= "\n" . $msg;
                 break;
 
             case IMP_Compose::FORWARD:
-                $fwd_msg = $imp_compose->forwardMessage(IMP_Compose::FORWARD_ATTACH, $imp_compose->getContentsOb());
-                $msg = $fwd_msg['body'];
+                try {
+                    $fwd_msg = $imp_compose->forwardMessage(IMP_Compose::FORWARD_ATTACH, $imp_compose->getContentsOb());
+                    $msg = $fwd_msg['body'];
+                } catch (IMP_Exception $e) {
+                    $notification->push($e, 'horde.error');
+                }
                 $message .= "\n" . $msg;
                 break;
             }
@@ -349,6 +367,7 @@ class IMP_Minimal_Compose extends IMP_Minimal_Base
         }
 
         $this->view->cacheid = $imp_compose->getCacheId();
+        $this->view->hmac = $imp_compose->getHmac();
         $this->view->menu = $this->getMenu('compose');
         $this->view->url = self::url();
         $this->view->user = $registry->getAuth();
@@ -366,7 +385,7 @@ class IMP_Minimal_Compose extends IMP_Minimal_Base
 
             $this->view->compose_enable = !$compose_disable;
             $this->view->msg = $msg;
-            $this->view->save_draft = ($injector->getInstance('IMP_Imap')->access(IMP_Imap::ACCESS_DRAFTS) && !$readonly_drafts);
+            $this->view->save_draft = ($injector->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_DRAFTS) && !$readonly_drafts);
             $this->view->subject = $header['subject'];
 
             $select_list = $identity->getSelectList();
