@@ -2,7 +2,7 @@
 /**
  * This class defines the base IMP status notification types.
  *
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -43,7 +43,10 @@ class IMP_Notification_Event_Status extends Horde_Core_Notification_Event_Status
             return parent::__toString();
         }
 
-        return Horde::img(Horde_Themes::img($img, 'imp'), $label) . parent::__toString();
+        return Horde_Themes_Image::tag(
+            Horde_Themes::img($img, 'imp'),
+            array('alt' => $label)
+        ) . '<div class="noticetext">' . parent::__toString() . '</div>';
     }
 
 }
