@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2013-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -40,11 +40,26 @@ class IMP_Perms
                 'title' => _("Allow folder navigation?"),
                 'type' => 'boolean'
             ),
+            'allow_remote' => array(
+                'imaponly' => true,
+                'title' => _("Allow remote account access?"),
+                'type' => 'boolean'
+            ),
             'create_mboxes' => array(
                 'imaponly' => true,
                 'title' => _("Allow mailbox creation?"),
                 'type' => 'boolean'
             ),
+            'max_bodysize' => array(
+                'global' => true,
+                'handle' => function($allowed, $opts) {
+                    return isset($opts['value'])
+                        ? (intval($allowed[0]) >= $opts['value'])
+                        : $allowed;
+                },
+                'title' => _("Maximum size (bytes) of compose body"),
+                'type' => 'int'
+            ),
             'max_recipients' => array(
                 'global' => true,
                 'handle' => function($allowed, $opts) {
@@ -68,9 +83,7 @@ class IMP_Perms
                         return true;
                     }
 
-                    try {
-                        $opts['value'] += $sentmail->numberOfRecipients($sentmail->limit_period, true);
-                    } catch (IMP_Exception $e) {}
+                    $opts['value'] += $sentmail->numberOfRecipients($sentmail->limit_period, true);
 
                     return (intval($allowed[0]) >= $opts['value']);
                 },
@@ -79,7 +92,7 @@ class IMP_Perms
             ),
             'max_create_mboxes' => array(
                 'handle' => function($allowed, $opts) {
-                    return (intval($allowed[0]) >= count($GLOBALS['injector']->getInstance('IMP_Imap_Tree')));
+                    return (intval($allowed[0]) >= count($GLOBALS['injector']->getInstance('IMP_Ftree')));
                 },
                 'imaponly' => true,
                 'title' => _("Maximum Number of Mailboxes"),
