@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2001-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2001-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2001-2013 Horde LLC
+ * @copyright 2001-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -19,7 +19,7 @@
  * @author    Jan Schneider <jan@horde.org>
  * @author    Chuck Hagenbuch <chuck@horde.org>
  * @category  Horde
- * @copyright 2001-2013 Horde LLC
+ * @copyright 2001-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -173,7 +173,7 @@ class IMP_Prefs_Identity extends Horde_Core_Prefs_Identity
             }
 
             if (!strstr($val, '@')) {
-                $val .= '@' . $GLOBALS['injector']->getInstance('IMP_Imap')->config->maildomain;
+                $val .= '@' . $GLOBALS['injector']->getInstance('IMP_Factory_Imap')->create()->config->maildomain;
             }
 
             $ob = new Horde_Mail_Rfc822_Address($val);
@@ -421,7 +421,7 @@ class IMP_Prefs_Identity extends Horde_Core_Prefs_Identity
                 $val = str_replace("\r\n", "\n", $val);
                 $val = ($this->getValue('sig_dashes', $ident))
                     ? "\n-- \n" . $val
-                    : "\n" . $val;
+                    : "\n\n" . $val;
             }
         }
 
@@ -454,6 +454,30 @@ class IMP_Prefs_Identity extends Horde_Core_Prefs_Identity
         return $list;
     }
 
+    /**
+     * Does the user have any signatures (either text or HTML)?
+     *
+     * @param boolean $compose_page  If true, checks for signatures as used on
+     *                               the compose pages.
+     *
+     * @return boolean  True if the user has at least one signature.
+     */
+    public function hasSignature($compose_page = false)
+    {
+        global $prefs;
+
+        if (!$compose_page || $prefs->getValue('signature_show_compose')) {
+            foreach (array_keys($this->_identities) as $key) {
+                if (strlen(trim($this->getValue('signature_html', $key))) ||
+                    strlen(trim($this->getValue('signature', $key)))) {
+                    return true;
+                }
+            }
+        }
+
+        return false;
+    }
+
     /**
      * Returns a property from one of the identities.
      *
@@ -508,11 +532,8 @@ class IMP_Prefs_Identity extends Horde_Core_Prefs_Identity
             break;
 
         case IMP_Mailbox::MBOX_SENT:
-            if ($val) {
-                $val->expire(IMP_Mailbox::CACHE_SPECIALMBOXES);
-            } else {
-                IMP_Mailbox::get('INBOX')->expire(IMP_Mailbox::CACHE_SPECIALMBOXES);
-            }
+            $GLOBALS['injector']->getInstance('IMP_Mailbox_SessionCache')
+                ->expire(IMP_Mailbox_SessionCache::CACHE_SPECIALMBOXES);
             $val = IMP_Mailbox::prefTo($val);
             break;
         }
@@ -547,7 +568,7 @@ class IMP_Prefs_Identity extends Horde_Core_Prefs_Identity
      */
     public function saveSentmail($ident = null)
     {
-        return $GLOBALS['injector']->getInstance('IMP_Imap')->access(IMP_Imap::ACCESS_FOLDERS)
+        return $GLOBALS['injector']->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_FOLDERS)
             ? $this->getValue('save_sent_mail', $ident)
             : false;
     }
