@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -55,6 +55,11 @@ class IMP_Prefs_Special_Acl implements Horde_Core_Prefs_Ui_Special
 
         $rightslist = $acl->getRights();
 
+        $iterator = new IMP_Ftree_IteratorFilter(
+            $injector->getInstance('IMP_Ftree')
+        );
+        $iterator->add($iterator::NONIMAP);
+
         $view = new Horde_View(array(
             'templatePath' => IMP_TEMPLATES . '/prefs'
         ));
@@ -66,8 +71,9 @@ class IMP_Prefs_Special_Acl implements Horde_Core_Prefs_Ui_Special
         $view->current = sprintf(_("Current access to %s"), $mbox->display_html);
         $view->hasacl = count($curr_acl);
         $view->mbox = $mbox->form_to;
-        $view->options = IMP::flistSelect(array(
+        $view->options = new IMP_Ftree_Select(array(
             'basename' => true,
+            'iterator' => $iterator,
             'selected' => $mbox
         ));
 
