@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -32,10 +32,11 @@ class IMP_Prefs_Special_HtmlSignature implements Horde_Core_Prefs_Ui_Special
      */
     public function display(Horde_Core_Prefs_Ui $ui)
     {
-        global $injector, $page_output, $prefs;
+        global $conf, $injector, $page_output, $prefs;
 
+        $page_output->addScriptFile('editor.js');
         $page_output->addScriptFile('signaturehtml.js');
-        $injector->getInstance('IMP_Editor')->init('signature_html');
+        $page_output->addScriptPackage('IMP_Script_Package_Editor');
 
         $identity = $injector->getInstance('IMP_Identity');
 
@@ -44,10 +45,7 @@ class IMP_Prefs_Special_HtmlSignature implements Horde_Core_Prefs_Ui_Special
             $js[$key] = $identity->getValue('signature_html', $key);
         };
 
-        $jsfile = new Horde_Script_File_JsDir('ckeditor/pasteignore.js', 'imp');
-
         $page_output->addInlineJsVars(array(
-            'ImpHtmlSignaturePrefs.pasteignore' => $jsfile->url->url,
             'ImpHtmlSignaturePrefs.sigs' => $js
         ));
 
@@ -56,6 +54,7 @@ class IMP_Prefs_Special_HtmlSignature implements Horde_Core_Prefs_Ui_Special
         ));
         $view->addHelper('Text');
 
+        $view->img_limit = $conf['compose']['htmlsig_img_size'];
         $view->signature = $prefs->getValue('signature_html');
 
         return $view->render('signaturehtml');
@@ -65,7 +64,20 @@ class IMP_Prefs_Special_HtmlSignature implements Horde_Core_Prefs_Ui_Special
      */
     public function update(Horde_Core_Prefs_Ui $ui)
     {
-        return $GLOBALS['injector']->getInstance('IMP_Identity')->setValue('signature_html', $ui->vars->signature_html);
+        global $injector, $notification;
+
+        try {
+            /* Throws exception if over image size limit. */
+            new IMP_Compose_HtmlSignature($ui->vars->signature_html);
+        } catch (IMP_Exception $e) {
+            $notification->push($e, 'horde.error');
+            return false;
+        }
+
+        return $injector->getInstance('IMP_Identity')->setValue(
+            'signature_html',
+            $ui->vars->signature_html
+        );
     }
 
 }
