@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -49,9 +49,15 @@ class IMP_Prefs_Special_Trash extends IMP_Prefs_Special_SpecialMboxes implements
         $view->addHelper('Horde_Core_View_Helper_Label');
         $view->addHelper('Tag');
 
-        $view->flist = IMP::flistSelect(array(
+        $iterator = new IMP_Ftree_IteratorFilter(
+            $injector->getInstance('IMP_Ftree')
+        );
+        $iterator->add($iterator::NONIMAP);
+        $iterator->mboxes = array('INBOX');
+
+        $view->flist = new IMP_Ftree_Select(array(
             'basename' => true,
-            'filter' => array('INBOX'),
+            'iterator' => $iterator,
             'new_mbox' => true,
             'selected' => $trash
         ));
@@ -86,7 +92,7 @@ class IMP_Prefs_Special_Trash extends IMP_Prefs_Special_SpecialMboxes implements
             return false;
         }
 
-        $injector->getInstance('IMP_Imap')->updateFetchIgnore();
+        $injector->getInstance('IMP_Factory_Imap')->create()->updateFetchIgnore();
 
         /* Switching to/from Virtual Trash requires us to expire all currently
          * cached mailbox lists (hide deleted status may have changed). */
