@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -21,7 +21,7 @@
  *
  * @author    Michael Redinger <Michael.Redinger@uibk.ac.at>
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -32,7 +32,11 @@ class IMP_Quota_Hook extends IMP_Quota
     public function getQuota($mailbox = null)
     {
         try {
-            $quota = Horde::callHook('quota', array($this->_params), 'imp');
+            $quota = $GLOBALS['injector']->getInstance('Horde_Core_Hooks')->callHook(
+                'quota',
+                'imp',
+                array($this->_params)
+            );
         } catch (Horde_Exception_HookNotSet $e) {
             throw new IMP_Exception($e->getMessage());
         }
