@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,11 +16,11 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
-class IMP_Search implements ArrayAccess, Iterator, Serializable
+class IMP_Search implements ArrayAccess, IteratorAggregate, Serializable
 {
     /* The mailbox search prefix. */
     const MBOX_PREFIX = "impsearch\0";
@@ -30,12 +30,6 @@ class IMP_Search implements ArrayAccess, Iterator, Serializable
     const DIMP_FILTERSEARCH = 'dimpfsearch';
     const DIMP_QUICKSEARCH = 'dimpqsearch';
 
-    /* Bitmask filters for iterator. */
-    const LIST_FILTER = 1;
-    const LIST_QUERY = 2;
-    const LIST_VFOLDER = 4;
-    const LIST_DISABLED = 8;
-
     /* Query creation types. */
     const CREATE_FILTER = 1;
     const CREATE_QUERY = 2;
@@ -48,13 +42,6 @@ class IMP_Search implements ArrayAccess, Iterator, Serializable
      */
     public $changed = false;
 
-    /**
-     * Iterator filter mask.
-     *
-     * @var integer
-     */
-    protected $_filter = 0;
-
     /**
      * Search queries.
      *
@@ -106,6 +93,8 @@ class IMP_Search implements ArrayAccess, Iterator, Serializable
      */
     public function createQuery($criteria, array $opts = array())
     {
+        global $injector;
+
         $opts = array_merge(array(
             'id' => null,
             'label' => null,
@@ -163,12 +152,16 @@ class IMP_Search implements ArrayAccess, Iterator, Serializable
             /* This will overwrite previous value, if it exists. */
             $this->_search['vfolders'][$ob->id] = $ob;
             $this->setVFolders($this->_search['vfolders']);
-            $ob->mbox_ob->expire(array(
-                IMP_Mailbox::CACHE_DISPLAY,
-                IMP_Mailbox::CACHE_LABEL
-            ));
-            $GLOBALS['injector']->getInstance('IMP_Imap_Tree')->delete(strval($ob));
-            $GLOBALS['injector']->getInstance('IMP_Imap_Tree')->insert($ob);
+            $injector->getInstance('IMP_Mailbox_SessionCache')->expire(
+                array(
+                    IMP_Mailbox_SessionCache::CACHE_DISPLAY,
+                    IMP_Mailbox_SessionCache::CACHE_LABEL
+                ),
+                $ob->mbox_ob
+            );
+            $ftree = $injector->getInstance('IMP_Ftree');
+            $ftree->delete($ob);
+            $ftree->insert($ob);
             break;
         }
 
@@ -408,10 +401,10 @@ class IMP_Search implements ArrayAccess, Iterator, Serializable
 
         case $registry::VIEW_DYNAMIC:
             return IMP_Dynamic_Mailbox::url()->setAnchor(
-                'search:' . Horde_Serialize::serialize(array(
+                'search:' . json_encode(array(
                     'edit_query' => 1,
                     'mailbox' => $mbox->form_to
-                ), Horde_Serialize::JSON, 'UTF-8')
+                ))
             );
         }
     }
@@ -507,9 +500,9 @@ class IMP_Search implements ArrayAccess, Iterator, Serializable
                 if ($key == 'vfolders') {
                     $this->setVFolders($this->_search['vfolders']);
 
-                    $imaptree = $GLOBALS['injector']->getInstance('IMP_Imap_Tree');
-                    $imaptree->delete($value);
-                    $imaptree->insert($value);
+                    $ftree = $GLOBALS['injector']->getInstance('IMP_Ftree');
+                    $ftree->delete($value);
+                    $ftree->insert($value);
                 }
                 return;
             }
@@ -535,119 +528,24 @@ class IMP_Search implements ArrayAccess, Iterator, Serializable
 
                 if ($val == 'vfolders') {
                     $this->setVFolders($this->_search['vfolders']);
-                    $GLOBALS['injector']->getInstance('IMP_Imap_Tree')->delete($value);
+                    $GLOBALS['injector']->getInstance('IMP_Ftree')->delete($value);
                 }
                 break;
             }
         }
     }
 
-    /* Iterator methods. */
-
-    public function current()
-    {
-        return (($key = key($this->_search)) !== null)
-            ? current($this->_search[$key])
-            : null;
-    }
-
-    public function key()
-    {
-        return (($key = key($this->_search)) !== null)
-            ? key($this->_search[$key])
-            : null;
-    }
-
-    public function next()
-    {
-        $curr = null;
-
-        while (($skey = key($this->_search)) !== null) {
-            /* When switching between search types, need to catch the first
-             * element of the new array. */
-            $curr = is_null($curr)
-                ? next($this->_search[$skey])
-                : current($this->_search[$skey]);
-
-            while ($curr !== false) {
-                if ($this->_currValid()) {
-                    return;
-                }
-                $curr = next($this->_search[$skey]);
-            }
-
-            next($this->_search);
-        }
-    }
-
-    public function rewind()
-    {
-        foreach (array_keys($this->_search) as $key) {
-            reset($this->_search[$key]);
-        }
-        reset($this->_search);
-
-        if ($this->valid() && !$this->_currValid()) {
-            $this->next();
-        }
-    }
-
-    public function valid()
-    {
-        return (key($this->_search) !== null);
-    }
-
-    /* Helper functions for Iterator methods. */
+    /* IteratorAggregate method. */
 
     /**
-     * Set the current iterator filter and reset the internal pointer.
-     *
-     * @param integer $mask  A mask with the following possible elements:
-     *   - IMP_Search::LIST_DISABLED: List even if disabled.
-     *   - IMP_Search::LIST_FILTER: List filters.
-     *   - IMP_Search::LIST_QUERY: List search queries.
-     *   - IMP_Search::LIST_VFOLDER: List virtual folders.
      */
-    public function setIteratorFilter($mask = 0)
+    public function getIterator()
     {
-        $this->_filter = $mask;
-        reset($this);
-    }
-
-    /**
-     * Returns true if the current object is valid given the current filter.
-     *
-     * @return boolean  True if the object should be displayed.
-     */
-    protected function _currValid()
-    {
-        if (!($ob = $this->current())) {
-            return false;
+        $iterator = new AppendIterator();
+        foreach ($this->_search as $val) {
+            $iterator->append(new ArrayIterator($val));
         }
-
-        if ($ob->enabled ||
-            ($this->_filter & self::LIST_DISABLED)) {
-            if (!$this->_filter) {
-                return true;
-            }
-
-            if (($this->_filter & self::LIST_FILTER) &&
-                $this->isFilter($ob)) {
-                return true;
-            }
-
-            if (($this->_filter & self::LIST_QUERY) &&
-                $this->isQuery($ob)) {
-                return true;
-            }
-
-            if (($this->_filter & self::LIST_VFOLDER) &&
-                $this->isVFolder($ob)) {
-                return true;
-            }
-        }
-
-        return false;
+        return $iterator;
     }
 
     /* Serializable methods. */
@@ -659,7 +557,13 @@ class IMP_Search implements ArrayAccess, Iterator, Serializable
      */
     public function serialize()
     {
-        return serialize($this->_search);
+        return $GLOBALS['injector']->getInstance('Horde_Pack')->pack(
+            $this->_search,
+            array(
+                'compression' => false,
+                'phpob' => true
+            )
+        );
     }
 
     /**
@@ -671,12 +575,7 @@ class IMP_Search implements ArrayAccess, Iterator, Serializable
      */
     public function unserialize($data)
     {
-        $data = @unserialize($data);
-        if (!is_array($data)) {
-            throw new Exception('Cache version change');
-        }
-
-        $this->_search = $data;
+        $this->_search = $GLOBALS['injector']->getInstance('Horde_Pack')->unpack($data);
     }
 
 }
