@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -50,9 +50,9 @@ class IMP_Search_Vfolder_Vinbox extends IMP_Search_Vfolder_Builtin
     {
         switch ($name) {
         case 'mboxes':
-            $imp_tree = $GLOBALS['injector']->getInstance('IMP_Imap_Tree');
-            $imp_tree->prunePollList();
-            return $imp_tree->getPollList(true);
+            $poll = $GLOBALS['injector']->getInstance('IMP_Ftree')->poll;
+            $poll->prunePollList();
+            return $poll->getPollList(true);
         }
 
         return parent::__get($name);
