@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -48,9 +48,13 @@ class IMP_Test extends Horde_Test
      */
     protected $_appList = array(
         'ingo' => array(
-            'error' => 'Ingo provides basic mail filtering capabilities to IMP.',
+            'error' => 'Ingo provides mail filtering capabilities to IMP.',
             'version' => '3.0'
         ),
+        'kronolith' => array(
+            'error' => 'Kronolith provides calendaring capabilities to IMP.',
+            'version' => '4.0'
+        ),
         'nag' => array(
             'error' => 'Nag allows tasks to be directly created from e-mail data.',
             'version' => '4.0'
@@ -147,24 +151,27 @@ class IMP_Test extends Horde_Test
                 '<blockquote><em>Namespace Information</em><blockquote><pre>';
 
             try {
-                $namespaces = $imap_client->getNamespaces();
+                $namespaces = $imap_client->getNamespaces(
+                    array(),
+                    array('ob_return' => true)
+                );
                 foreach ($namespaces as $val) {
-                    switch ($val['type']) {
-                    case Horde_Imap_Client::NS_PERSONAL:
+                    switch ($val->type) {
+                    case $val::NS_PERSONAL:
                         $type = 'Personal';
                         break;
 
-                    case Horde_Imap_Client::NS_OTHER:
+                    case $val::NS_OTHER:
                         $type = 'Other Users\'';
                         break;
 
-                    case Horde_Imap_Client::NS_SHARED:
+                    case $val::NS_SHARED:
                         $type = 'Shared';
                         break;
                     }
 
-                    $ret .= 'NAMESPACE: "' . htmlspecialchars($val['name']) . "\"\n" .
-                        'DELIMITER: ' . htmlspecialchars($val['delimiter']) . "\n" .
+                    $ret .= 'NAMESPACE: "' . htmlspecialchars($val->name) . "\"\n" .
+                        'DELIMITER: ' . htmlspecialchars($val->delimiter) . "\n" .
                         'TYPE: ' . htmlspecialchars($type) . "\n\n";
                 }
             } catch (Horde_Imap_Client_Exception $e) {
