@@ -2,15 +2,16 @@
 /**
  * IMP mailbox RSS feed.
  *
- * Copyright 2007-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2007-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
- * @author   Eric Garrido <ekg2002@columbia.edu>
- * @category Horde
- * @license  http://www.horde.org/licenses/gpl GPL
- * @package  IMP
+ * @author    Eric Garrido <ekg2002@columbia.edu>
+ * @category  Horde
+ * @copyright 2007-2014 Horde LLC
+ * @license   http://www.horde.org/licenses/gpl GPL
+ * @package   IMP
  */
 
 require_once __DIR__ . '/lib/Application.php';
@@ -37,8 +38,8 @@ $request = Horde_Util::getPathInfo();
 if (!empty($request)) {
     $request_parts = explode('/-/', $request);
     if (!empty($request_parts[0])) {
-        $ns_info = $injector->getInstance('IMP_Imap')->getNamespace();
-        $mailbox = IMP_Mailbox::get(preg_replace('/\//', $ns_info['delimiter'], trim($request_parts[0], '/')))->namespace_append;
+        $ns_info = $mailbox->imp_imap->getNamespace(IMP_Imap::NS_DEFAULT);
+        $mailbox = IMP_Mailbox::get(preg_replace('/\//', $ns_info->delimiter, trim($request_parts[0], '/')))->namespace_append;
 
         /* Make sure mailbox exists or else exit immediately. */
         if (!$mailbox->exists) {
