@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category   Horde
- * @copyright  2010-2013 Horde LLC
+ * @copyright  2010-2014 Horde LLC
  * @license    http://www.horde.org/licenses/gpl GPL
  * @package    IMP
  * @subpackage UnitTests
@@ -18,7 +18,7 @@
  * @author     Michael Slusarz <slusarz@horde.org>
  * @author     Gunnar Wrobel <wrobel@pardus.de>
  * @category   Horde
- * @copyright  2010-2013 Horde LLC
+ * @copyright  2010-2014 Horde LLC
  * @ignore
  * @license    http://www.horde.org/licenses/gpl GPL
  * @package    IMP
@@ -33,6 +33,7 @@ extends PHPUnit_Framework_TestCase
     private $_contentsFactory;
     private $_identity;
     private $_identityId = 'default';
+    private $_imapFactory;
     private $_mail;
     private $_mailbox;
     private $_notifyStack = array();
@@ -96,6 +97,16 @@ extends PHPUnit_Framework_TestCase
             }
             return $this->_contentsFactory;
 
+        case 'IMP_Factory_Imap':
+            if (!isset($this->_imapFactory)) {
+                $imap = $this->getMock('IMP_Factory_Imap', array(), array(), '', false);
+                $imap->expects($this->any())
+                    ->method('create')
+                    ->will($this->returnValue(new IMP_Stub_Imap()));
+                $this->_imapFactory = $imap;
+            }
+            return $this->_imapFactory;
+
         case 'IMP_Factory_Mailbox':
             if (!isset($this->_mailbox)) {
                 $mbox = $this->getMock('IMP_Factory_Mailbox', array(), array(), '', false);
@@ -136,9 +147,6 @@ extends PHPUnit_Framework_TestCase
                 $this->_mail = new Horde_Mail_Transport_Mock();
             }
             return $this->_mail;
-
-        case 'IMP_Imap':
-            return new IMP_Imap();
         }
     }
 
